@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2013-2022 the xine project
- * Copyright (C) 2013-2020 Petri Hintukainen <phintuka@users.sourceforge.net>
+ * Copyright (C) 2013 the xine project
+ * Copyright (C) 2013 Petri Hintukainen <phintuka@users.sourceforge.net>
  *
  * This file is part of xine, a free video player.
  *
@@ -26,11 +26,7 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <pthread.h>
-#include <errno.h>
 
-#include <libavutil/avutil.h>
-#include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
 #include <libavformat/avio.h>
 
@@ -47,20 +43,8 @@
 
 #include "ffmpeg_decoder.h"
 
-#include "ffmpeg_compat.h"
-
-#ifndef LIBAVFORMAT_VERSION_INT
-#  if defined(LIBAVFORMAT_VERSION_MAJOR) && defined(LIBAVFORMAT_VERSION_MINOR)
-#    define LIBAVFORMAT_VERSION_INT XFF_INT_VERSION(LIBAVFORMAT_VERSION_MAJOR,LIBAVFORMAT_VERSION_MINOR,0)
-#  endif
-#endif
-
-#if LIBAVFORMAT_VERSION_INT < XFF_INT_VERSION(57,62,0)
-#define CODEC_ID(st)   ((st)->codec->codec_id)
-#else
-#define CODEC_ID(st)   ((st)->codecpar->codec_id)
-#define XFF_CODECPAR 1
-#endif
+#include "ff_video_list.h"
+#include "ff_audio_list.h"
 
 /*
  * avformat dummy input plugin
@@ -75,43 +59,30 @@ typedef struct {
 } avformat_input_plugin_t;
 
 static off_t input_avformat_read (input_plugin_t *this_gen, void *buf_gen, off_t len) {
-  (void)this_gen;
-  (void)buf_gen;
-  (void)len;
   return 0;
 }
 
 static buf_element_t *input_avformat_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo, off_t todo) {
-  (void)this_gen;
-  (void)fifo;
-  (void)todo;
   return NULL;
 }
 
 static off_t input_avformat_get_length (input_plugin_t *this_gen) {
-  (void)this_gen;
   return -1;
 }
 
 static uint32_t input_avformat_get_capabilities (input_plugin_t *this_gen) {
-  (void)this_gen;
   return INPUT_CAP_SEEKABLE;
 }
 
 static uint32_t input_avformat_get_blocksize (input_plugin_t *this_gen) {
-  (void)this_gen;
   return 0;
 }
 
 static off_t input_avformat_get_current_pos (input_plugin_t *this_gen) {
-  (void)this_gen;
   return 0;
 }
 
 static off_t input_avformat_seek (input_plugin_t *this_gen, off_t offset, int origin) {
-  (void)this_gen;
-  (void)offset;
-  (void)origin;
   return -1;
 }
 
@@ -145,7 +116,6 @@ static int input_avformat_get_optional_data (input_plugin_t *this_gen,
 }
 
 static int input_avformat_open (input_plugin_t *this_gen) {
-  (void)this_gen;
   return 1;
 }
 
@@ -174,8 +144,6 @@ static input_plugin_t *input_avformat_get_instance (input_class_t *cls_gen, xine
     return NULL;
   }
 
-  init_once_routine();
-
   /* always accept own protocol */
   /* avformat+http://... --> use avformat instead of xine native input/demux plugins */
   if (!strncasecmp (mrl, DEMUX_AVFORMAT_ID"+", proto_len)) {
@@ -192,7 +160,7 @@ static input_plugin_t *input_avformat_get_instance (input_class_t *cls_gen, xine
     real_mrl = strdup(mrl);
     memmove(real_mrl + 4, real_mrl + 8, strlen(real_mrl) - 8 + 1);
   }
-  else if (!strncmp(mrl, "rtsp+http", 9)) {
+  if (!strncmp(mrl, "rtsp+http", 9)) {
     av_dict_set(&options, "rtsp_transport", "http", 0);
     real_mrl = strdup(mrl);
     memmove(real_mrl + 4, real_mrl + 9, strlen(real_mrl) - 9 + 1);
@@ -221,10 +189,6 @@ static input_plugin_t *input_avformat_get_instance (input_class_t *cls_gen, xine
   avformat_input_plugin_t *this;
 
   this = calloc(1, sizeof(avformat_input_plugin_t));
-  if (!this) {
-    return NULL;
-  }
-
   this->mrl           = _x_mrl_remove_auth(mrl);
   this->fmt_ctx       = fmt_ctx;
 
@@ -247,21 +211,29 @@ static input_plugin_t *input_avformat_get_instance (input_class_t *cls_gen, xine
   return &this->input_plugin;
 }
 
-void *init_avformat_input_plugin (xine_t *xine, const void *data) {
+void *init_avformat_input_plugin (xine_t *xine, void *data) {
 
-  static const input_class_t input_class = {
-    .get_instance      = input_avformat_get_instance,
-    .description       = N_("libavformat input plugin"),
-    .identifier        = DEMUX_AVFORMAT_ID,
-    .dispose           = NULL,
-  };
+  input_class_t  *this;
 
-  (void)xine;
-  (void)data;
+  this = calloc(1, sizeof(input_class_t));
 
-  return (void *)&input_class;
+  pthread_once( &once_control, init_once_routine );
+
+  this->get_instance      = input_avformat_get_instance;
+  this->description       = N_("libavformat input plugin");
+  this->identifier        = DEMUX_AVFORMAT_ID;
+  this->get_dir           = NULL;
+  this->get_autoplay_list = NULL;
+  this->dispose           = default_input_class_dispose;
+  this->eject_media       = NULL;
+
+  return this;
 }
 
+input_info_t input_info_avformat = {
+  -2   /* priority */
+};
+
 /*
  * avformat demux plugin
  */
@@ -298,7 +270,7 @@ typedef struct {
 static void check_newpts(avformat_demux_plugin_t *this, int64_t pts) {
 
   int64_t diff = this->last_pts - pts;
-  if (this->seek_flag || this->send_newpts || (this->last_pts && llabs(diff) > WRAP_THRESHOLD)) {
+  if (this->seek_flag || this->send_newpts || (this->last_pts && abs(diff) > WRAP_THRESHOLD)) {
 
     _x_demux_control_newpts(this->stream, pts, this->seek_flag);
     this->send_newpts = 0;
@@ -307,10 +279,10 @@ static void check_newpts(avformat_demux_plugin_t *this, int64_t pts) {
   }
 }
 
-static uint32_t video_codec_lookup(avformat_demux_plugin_t *this, unsigned id) {
+static uint32_t video_codec_lookup(avformat_demux_plugin_t *this, int id) {
 
-  size_t i;
-  for (i = 0; i < ff_video_lookup_entries; i++) {
+  int i;
+  for (i = 0; i < sizeof(ff_video_lookup)/sizeof(ff_codec_t); i++) {
     if (ff_video_lookup[i].id == id) {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
                LOG_MODULE": found video codec '%s'\n", ff_video_lookup[i].name);
@@ -321,10 +293,10 @@ static uint32_t video_codec_lookup(avformat_demux_plugin_t *this, unsigned id) {
   return 0;
 }
 
-static uint32_t audio_codec_lookup(avformat_demux_plugin_t *this, unsigned id) {
+static uint32_t audio_codec_lookup(avformat_demux_plugin_t *this, int id) {
 
-  size_t i;
-  for (i = 0; i < ff_audio_lookup_entries; i++) {
+  int i;
+  for (i = 0; i < sizeof(ff_audio_lookup)/sizeof(ff_codec_t); i++) {
     if (ff_audio_lookup[i].id == id) {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
                LOG_MODULE": found audio codec '%s'\n", ff_audio_lookup[i].name);
@@ -333,42 +305,23 @@ static uint32_t audio_codec_lookup(avformat_demux_plugin_t *this, unsigned id) {
   }
 
   switch (id) {
-    case CODEC_ID_PCM_S16LE:
+    case AV_CODEC_ID_PCM_S16LE:
       return BUF_AUDIO_LPCM_LE;
-    case CODEC_ID_PCM_S16BE:
+    case AV_CODEC_ID_PCM_S16BE:
       return BUF_AUDIO_LPCM_BE;
-    case CODEC_ID_MP2:
+    case AV_CODEC_ID_MP2:
       return BUF_AUDIO_MPEG;
-    case CODEC_ID_AC3:
+    case AV_CODEC_ID_AC3:
       return BUF_AUDIO_A52;
   }
 
   return 0;
 }
 
-/* copied from libavformat/utils.c (not available in recent libav API) */
-static AVProgram *_find_program_from_stream(AVFormatContext *ic, AVProgram *last, int s)
-{
-  unsigned i, j;
-
-  for (i = 0; i < ic->nb_programs; i++) {
-    if (ic->programs[i] == last) {
-      last = NULL;
-    } else {
-      if (!last)
-        for (j = 0; j < ic->programs[i]->nb_stream_indexes; j++)
-          if (ic->programs[i]->stream_index[j] == (unsigned int)s)
-            return ic->programs[i];
-    }
-  }
-  return NULL;
-}
-
 static int find_avformat_streams(avformat_demux_plugin_t *this) {
 
   AVProgram *p = NULL;
-  unsigned int nb_streams;
-  unsigned int u;
+  int i, nb_streams;
 
   /* find avformat streams */
 
@@ -384,23 +337,20 @@ static int find_avformat_streams(avformat_demux_plugin_t *this) {
   this->num_streams = this->fmt_ctx->nb_streams;
   this->xine_buf_type = calloc(this->num_streams, sizeof(uint32_t));
   this->audio_stream_idx = calloc(this->num_streams, sizeof(int));
-  if (!this->xine_buf_type || !this->audio_stream_idx) {
-    return 0;
-  }
 
   /* map video stream to xine buffer type */
 
   if (this->video_stream_idx >= 0) {
     AVStream *st = this->fmt_ctx->streams[this->video_stream_idx];
-    uint32_t xine_video_type = video_codec_lookup(this, CODEC_ID(st));
+    uint32_t xine_video_type = video_codec_lookup(this, st->codec->codec_id);
 
     if (!xine_video_type) {
       this->video_stream_idx = -1;
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-               LOG_MODULE": ffmpeg video codec id %d --> NO xine buffer type\n", CODEC_ID(st));
+               LOG_MODULE": ffmpeg video codec id %d --> NO xine buffer type\n", st->codec->codec_id);
     } else {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-               LOG_MODULE": ffmpeg video codec id %d --> xine buffer type 0x%08x\n", CODEC_ID(st), xine_video_type);
+               LOG_MODULE": ffmpeg video codec id %d --> xine buffer type 0x%08x\n", st->codec->codec_id, xine_video_type);
 
       this->xine_buf_type[this->video_stream_idx] = xine_video_type;
     }
@@ -409,35 +359,30 @@ static int find_avformat_streams(avformat_demux_plugin_t *this) {
   /* get audio tracks of the program */
 
   if (this->video_stream_idx >= 0) {
-    p = _find_program_from_stream(this->fmt_ctx, NULL, this->video_stream_idx);
+    p = av_find_program_from_stream(this->fmt_ctx, NULL, this->video_stream_idx);
   }
   nb_streams = p ? p->nb_stream_indexes : this->fmt_ctx->nb_streams;
 
-  for (u = 0; u < nb_streams; u++) {
-    unsigned int stream_index = p ? p->stream_index[u] : u;
+  for (i = 0; i < nb_streams; i++) {
+    int stream_index = p ? p->stream_index[i] : i;
     AVStream *st = this->fmt_ctx->streams[stream_index];
 
     if (stream_index >= this->num_streams) {
-      xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": Too many streams, ignoring stream #%u\n", u);
+      xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": Too many streams, ignoring stream #%d\n", i);
       continue;
     }
 
-#ifdef XFF_CODECPAR
-    if (st->codecpar && st->codecpar->codec_type == AVMEDIA_TYPE_AUDIO &&
-        st->codecpar->sample_rate != 0 && st->codecpar->channels != 0)
-#else
     if (st->codec && st->codec->codec_type == AVMEDIA_TYPE_AUDIO &&
-        st->codec->sample_rate != 0 && st->codec->channels != 0)
-#endif
-    {
-      uint32_t xine_audio_type = audio_codec_lookup(this, CODEC_ID(st));
+        st->codec->sample_rate != 0 && st->codec->channels != 0) {
+
+      int xine_audio_type = audio_codec_lookup(this, st->codec->codec_id);
       if (!xine_audio_type) {
         xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-                 LOG_MODULE": ffmpeg audio codec id %d --> NO xine buffer type\n", CODEC_ID(st));
+                 LOG_MODULE": ffmpeg audio codec id %d --> NO xine buffer type\n", st->codec->codec_id);
         continue;
       }
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-               LOG_MODULE": ffmpeg audio codec id %d --> xine buffer type 0x%08x\n", CODEC_ID(st), xine_audio_type);
+               LOG_MODULE": ffmpeg audio codec id %d --> xine buffer type 0x%08x\n", st->codec->codec_id, xine_audio_type);
 
       this->audio_stream_idx[this->audio_track_count] = stream_index;
 
@@ -467,28 +412,20 @@ static int find_avformat_streams(avformat_demux_plugin_t *this) {
 
 static void send_headers_audio(avformat_demux_plugin_t *this) {
 
-  unsigned ii;
+  int ii;
   for (ii = 0; ii < this->audio_track_count; ii++) {
 
-#ifdef XFF_CODECPAR
-  AVCodecParameters *ctx = this->fmt_ctx->streams[this->audio_stream_idx[ii]]->codecpar;
-#else
   AVCodecContext    *ctx = this->fmt_ctx->streams[this->audio_stream_idx[ii]]->codec;
-#endif
   buf_element_t     *buf = this->stream->audio_fifo->buffer_pool_alloc (this->stream->audio_fifo);
   size_t             extradata_size = ctx->extradata_size;
   xine_waveformatex *fmt = (xine_waveformatex *)buf->content;
 
-  if (!ctx->extradata || extradata_size + sizeof(xine_waveformatex) > (size_t)buf->max_size) {
-    if (extradata_size) {
-      xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": ignoring large audio extradata (%zd bytes)\n", extradata_size);
-    }
+  if (!ctx->extradata || extradata_size + sizeof(xine_waveformatex) > buf->max_size) {
     extradata_size = 0;
   }
 
   _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_FOURCC, ctx->codec_tag);
 
-  memset(fmt, 0, sizeof(*fmt));
   fmt->cbSize          = extradata_size;
   fmt->nBlockAlign     = ctx->block_align;
   fmt->nAvgBytesPerSec = ctx->bit_rate / 8;
@@ -510,25 +447,17 @@ static void send_headers_audio(avformat_demux_plugin_t *this) {
 
 static void send_headers_video(avformat_demux_plugin_t *this) {
 
-#ifdef XFF_CODECPAR
-  AVCodecParameters *ctx = this->fmt_ctx->streams[this->video_stream_idx]->codecpar;
-#else
   AVCodecContext *ctx = this->fmt_ctx->streams[this->video_stream_idx]->codec;
-#endif
   buf_element_t  *buf = this->stream->video_fifo->buffer_pool_alloc (this->stream->video_fifo);
   size_t          extradata_size = ctx->extradata_size;
   xine_bmiheader *bih = (xine_bmiheader *)buf->content;
 
-  if (!ctx->extradata || extradata_size + sizeof(xine_bmiheader) > (size_t)buf->max_size) {
-    if (extradata_size) {
-      xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": ignoring large video extradata (%zd bytes)\n", extradata_size);
-    }
+  if (!ctx->extradata || extradata_size + sizeof(xine_bmiheader) > buf->max_size) {
     extradata_size = 0;
   }
 
   _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_FOURCC, ctx->codec_tag);
 
-  memset(bih, 0, sizeof(*bih));
   bih->biSize     = sizeof(xine_bmiheader) + extradata_size;
   bih->biBitCount = ctx->bits_per_coded_sample;
   bih->biWidth    = ctx->width;
@@ -549,28 +478,27 @@ static int send_avpacket(avformat_demux_plugin_t *this)
 {
   int64_t  stream_pos    = avio_tell(this->fmt_ctx->pb);
   int64_t  stream_length = avio_size(this->fmt_ctx->pb);
-  XFF_PACKET_DECL (pkt);
+  AVPacket pkt;
   uint32_t buffer_type = 0;
   fifo_buffer_t *fifo = NULL;
 
-  XFF_PACKET_NEW (pkt);
-  pkt->data = NULL;
-  pkt->size = 0;
+  av_init_packet(&pkt);
+  pkt.data = NULL;
+  pkt.size = 0;
 
   /* read frame from the file */
-  if (av_read_frame(this->fmt_ctx, pkt) < 0) {
-    xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": av_read_frame() failed\n");
+  if (av_read_frame(this->fmt_ctx, &pkt) < 0) {
     return -1;
   }
 
   /* map to xine fifo / buffer type */
-  if (pkt->stream_index >= 0 && (unsigned)pkt->stream_index < this->num_streams) {
-    buffer_type = this->xine_buf_type [pkt->stream_index];
+  if (pkt.stream_index < this->num_streams) {
+    buffer_type = this->xine_buf_type[pkt.stream_index];
   } else {
     // TODO: new streams found
   }
 
-  if (this->video_stream_idx >= 0 && pkt->stream_index == this->video_stream_idx) {
+  if (this->video_stream_idx >= 0 && pkt.stream_index == this->video_stream_idx) {
     fifo = this->stream->video_fifo;
   } else {
     fifo = this->stream->audio_fifo;
@@ -583,17 +511,17 @@ static int send_avpacket(avformat_demux_plugin_t *this)
     int      total_time    = (int)((int64_t)this->fmt_ctx->duration * 1000 / AV_TIME_BASE);
     int      input_time    = input_normpos * total_time / 65535;
 
-    if (pkt->pts != AV_NOPTS_VALUE) {
-      AVStream *stream = this->fmt_ctx->streams [pkt->stream_index];
-      pts = (int64_t)(pkt->pts * stream->time_base.num * 90000 / stream->time_base.den);
+    if (pkt.pts != AV_NOPTS_VALUE) {
+      AVStream *stream = this->fmt_ctx->streams[pkt.stream_index];
+      pts = (int64_t)(pkt.pts * stream->time_base.num * 90000 / stream->time_base.den);
       check_newpts(this, pts);
     }
 
-    _x_demux_send_data (fifo, pkt->data, pkt->size, pts, buffer_type, 0/*decoder_flags*/,
+    _x_demux_send_data(fifo, pkt.data, pkt.size, pts, buffer_type, 0/*decoder_flags*/,
                        input_normpos, input_time, total_time, 0/*frame_number*/);
   }
 
-  XFF_PACKET_UNREF (pkt);
+  av_free_packet(&pkt);
 
   return 1;
 }
@@ -619,7 +547,6 @@ static int demux_avformat_get_stream_length (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_avformat_get_capabilities(demux_plugin_t *this_gen) {
-  (void)this_gen;
   return DEMUX_CAP_AUDIOLANG;
 }
 
@@ -636,7 +563,7 @@ static int demux_avformat_get_optional_data(demux_plugin_t *this_gen,
 
   switch (data_type) {
     case DEMUX_OPTIONAL_DATA_AUDIOLANG:
-      if (channel >= 0 && (unsigned)channel < this->audio_track_count) {
+      if (channel >= 0 && channel < this->audio_track_count) {
 
         AVStream *st = this->fmt_ctx->streams[this->audio_stream_idx[channel]];
         AVDictionaryEntry *tag = NULL;
@@ -702,16 +629,13 @@ static int avformat_seek (avformat_demux_plugin_t *this,
     if (av_seek_frame(this->fmt_ctx, -1, pos, 0) >= 0) {
       return 0;
     }
-    return -1;
   }
 
-#if 0
   /* seek to byte offset */
   pos = (int64_t)start_pos * avio_size(this->fmt_ctx->pb) / 65535;
   if (av_seek_frame(this->fmt_ctx, -1, pos, AVSEEK_FLAG_BYTE) >= 0) {
     return 0;
   }
-#endif
 
   /* stream does not support seeking to byte offset. Final try with timestamp. */
   pos = (int64_t)start_pos * this->fmt_ctx->duration / 65535;
@@ -755,35 +679,12 @@ static void demux_avformat_dispose (demux_plugin_t *this_gen) {
 
 static int pb_input_read_packet(void *opaque, uint8_t *buf, int buf_size) {
   input_plugin_t *input = (input_plugin_t *)opaque;
-  int r = input->read (input, buf, buf_size);
-  /* avoid eternal misunderstanding :-O */
-  if (r > 0)
-    return r;
-  if (r == 0) {
-#ifdef AVERROR_EOF
-    return AVERROR_EOF;
-#else
-    return 0;
-#endif
-  }
-  return AVERROR (errno);
+  return input->read(input, buf, buf_size);
 }
 
 static int64_t pb_input_seek(void *opaque, int64_t offset, int whence) {
   input_plugin_t *input = (input_plugin_t *)opaque;
-  int64_t r;
-
-  if (whence == AVSEEK_SIZE) {
-    off_t len = input->get_length(input);
-    if (len > 0)
-      return len;
-    return -1;
-  }
-
-  r = input->seek (input, offset, whence);
-  if (r >= 0)
-    return r;
-  return AVERROR (errno);
+  return input->seek(input, offset, whence);
 }
 
 static AVIOContext *get_io_context(xine_stream_t *stream, input_plugin_t *input)
@@ -804,11 +705,6 @@ static AVIOContext *get_io_context(xine_stream_t *stream, input_plugin_t *input)
     /* create AVIO wrapper for native input plugin */
     xprintf (stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": creating AVIOContext wrapper for input plugin\n");
     pb = avio_alloc_context(av_malloc(4096), 4096, 0/*write_flag*/, input, pb_input_read_packet, NULL, pb_input_seek);
-
-    if (input->get_capabilities(input) & INPUT_CAP_SEEKABLE)
-      pb->seekable = AVIO_SEEKABLE_NORMAL;
-    else
-      pb->seekable = 0;
   }
 
   avio_seek(pb, 0, SEEK_SET);
@@ -858,8 +754,6 @@ static demux_plugin_t *open_demux_avformat_plugin (demux_class_t *class_gen,
                                                    xine_stream_t *stream,
                                                    input_plugin_t *input) {
 
-  init_once_routine();
-
   /* get AVFormatContext */
   AVFormatContext *fmt_ctx = get_format_context(stream, input);
   if (!fmt_ctx) {
@@ -882,10 +776,6 @@ static demux_plugin_t *open_demux_avformat_plugin (demux_class_t *class_gen,
   avformat_demux_plugin_t *this;
 
   this         = calloc(1, sizeof(avformat_demux_plugin_t));
-  if (!this) {
-    avformat_close_input(&fmt_ctx);
-    return NULL;
-  }
   this->stream = stream;
 
   this->demux_plugin.send_headers      = demux_avformat_send_headers;
@@ -911,17 +801,21 @@ static demux_plugin_t *open_demux_avformat_plugin (demux_class_t *class_gen,
   return &this->demux_plugin;
 }
 
-void *init_avformat_demux_plugin (xine_t *xine, const void *data) {
-  static const demux_class_t this = {
-    .open_plugin     = open_demux_avformat_plugin,
-    .description     = N_("libavformat demux plugin"),
-    .identifier      = DEMUX_AVFORMAT_ID,
-    .mimetypes       = NULL,
-    .extensions      = "",
-    .dispose         = NULL
-  };
-  (void)xine;
-  (void)data;
-  return (demux_class_t *)&this;
+void *init_avformat_demux_plugin (xine_t *xine, void *data) {
+  demux_class_t     *this;
+
+  this  = calloc(1, sizeof(demux_class_t));
+
+  this->open_plugin     = open_demux_avformat_plugin;
+  this->description     = N_("libavformat demux plugin");
+  this->identifier      = DEMUX_AVFORMAT_ID;
+  this->mimetypes       = NULL;
+  this->extensions      = "";
+  this->dispose         = default_demux_class_dispose;
+
+  return this;
 }
 
+demuxer_info_t demux_info_avformat = {
+  -1                       /* priority */
+};
