@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2008 the xine project
+ * Copyright (C) 2001-2014 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -44,8 +44,6 @@
 #include "ffmpeg_decoder.h"
 #include "ff_mpeg_parser.h"
 
-#define FF_API_CODEC_ID 1
-
 #ifdef HAVE_FFMPEG_AVUTIL_H
 #  include <postprocess.h>
 #else
@@ -131,7 +129,7 @@ struct ff_video_decoder_s {
   double            aspect_ratio;
   int               aspect_ratio_prio;
   int               frame_flags;
-  int               crop_right, crop_bottom;
+  int               edge;
 
   int               output_format;
 
@@ -139,7 +137,7 @@ struct ff_video_decoder_s {
 
 #if AVPALETTE == 1
   AVPaletteControl  palette_control;
-#elif AVPALETTE == 2
+#elif AVPALETTE == 2 || AVPALETTE == 3
   uint32_t          palette[256];
   int               palette_changed;
 #endif
@@ -157,13 +155,25 @@ struct ff_video_decoder_s {
   uint8_t           set_stream_info;
 
 #ifdef ENABLE_VAAPI
+  int                   vaapi_width, vaapi_height;
+  int                   vaapi_profile;
   struct vaapi_context  vaapi_context;
   vaapi_accel_t         *accel;
   vo_frame_t            *accel_img;
 #endif
+
+  /* Ugly: 2nd guess the reason for flush.
+     ff_flush () should really have an extra argument telling this. */
+#define STATE_RESET         0
+#define STATE_DISCONTINUITY 1
+#define STATE_READING_DATA  2
+#define STATE_FRAME_SENT    3
+#define STATE_FLUSHED       4
+  int               state;
 };
 
 /* import color matrix names */
+#define CM_HAVE_YCGCO_SUPPORT 1
 #include "../../video_out/color_matrix.c"
 
 static void ff_check_colorspace (ff_video_decoder_t *this) {
@@ -188,7 +198,7 @@ static void ff_check_colorspace (ff_video_decoder_t *this) {
   if (cm != this->color_matrix) {
     this->color_matrix = cm;
     xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-      "ffmpeg_video_dec: color matrix #%d [%s]\n", cm >> 1, cm_names[cm & 15]);
+      "ffmpeg_video_dec: color matrix #%d [%s]\n", cm >> 1, cm_names[cm & 31]);
 
     caps = this->stream->video_out->get_capabilities (this->stream->video_out);
 
@@ -219,29 +229,78 @@ static void set_stream_info(ff_video_decoder_t *this) {
 }
 
 #ifdef ENABLE_DIRECT_RENDERING
+#ifdef AV_BUFFER
+typedef struct {
+  int                 refs;
+  ff_video_decoder_t *this;
+  vo_frame_t         *vo_frame;
+#ifdef ENABLE_VAAPI
+  ff_vaapi_surface_t *va_surface;
+#endif
+} ff_saved_frame_t;
+
+static void release_frame (void *saved_frame, uint8_t *data) {
+  ff_saved_frame_t *ffsf = saved_frame;
+  /* At this point in time, AVFrame may already be reused. So take our saved values instead. */
+  if (ffsf) {
+    if (--(ffsf->refs))
+      return;
+#ifdef ENABLE_VAAPI
+    if (ffsf->va_surface)
+      ffsf->this->accel->release_vaapi_surface (ffsf->this->accel_img, ffsf->va_surface);
+#endif
+    if (ffsf->vo_frame) {
+      xine_list_iterator_t it;
+      ffsf->vo_frame->free (ffsf->vo_frame);
+      it = xine_list_find (ffsf->this->dr1_frames, ffsf->vo_frame);
+      if (it)
+        xine_list_remove (ffsf->this->dr1_frames, it);
+    }
+    free (ffsf);
+  }
+}
+#endif
+
 /* called from ffmpeg to do direct rendering method 1 */
-static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
+#ifdef AV_BUFFER
+static int get_buffer (AVCodecContext *context, AVFrame *av_frame, int flags)
+#else
+static int get_buffer (AVCodecContext *context, AVFrame *av_frame)
+#endif
+  {
   ff_video_decoder_t *this = (ff_video_decoder_t *)context->opaque;
   vo_frame_t *img;
+#ifdef AV_BUFFER
+  ff_saved_frame_t *ffsf;
+#endif
+  int buf_width  = av_frame->width;
+  int buf_height = av_frame->height;
+  /* The visible size, may be smaller. */
   int width  = context->width;
   int height = context->height;
   int guarded_render = 0;
 
+  /* A bit of unmotivated paranoia... */
+  if (buf_width < width)
+    buf_width = width;
+  if (buf_height < height)
+    buf_height = height;
+
   ff_check_colorspace (this);
 
   if (!this->bih.biWidth || !this->bih.biHeight) {
     this->bih.biWidth = width;
     this->bih.biHeight = height;
-
-    if (this->aspect_ratio_prio == 0) {
-      this->aspect_ratio = (double)width / (double)height;
-      this->aspect_ratio_prio = 1;
-      lprintf("default aspect ratio: %f\n", this->aspect_ratio);
-      this->set_stream_info = 1;
-    }
   }
 
-  avcodec_align_dimensions(context, &width, &height);
+  if (this->aspect_ratio_prio == 0) {
+    this->aspect_ratio = (double)width / (double)height;
+    this->aspect_ratio_prio = 1;
+    lprintf("default aspect ratio: %f\n", this->aspect_ratio);
+    this->set_stream_info = 1;
+  }
+
+  avcodec_align_dimensions(context, &buf_width, &buf_height);
 
 #ifdef ENABLE_VAAPI
   if( this->context->pix_fmt == PIX_FMT_VAAPI_VLD) {
@@ -251,12 +310,37 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
     av_frame->data[1] = NULL;
     av_frame->data[2] = NULL;
     av_frame->data[3] = NULL;
-    av_frame->type = FF_BUFFER_TYPE_USER;
 #ifdef AVFRAMEAGE
     av_frame->age = 1;
 #endif
     av_frame->reordered_opaque = context->reordered_opaque;
 
+#ifdef AV_BUFFER
+    ffsf = calloc (1, sizeof (ff_saved_frame_t));
+    if (!ffsf)
+      return AVERROR (ENOMEM);
+    ffsf->this     = this;
+#endif
+
+    /* reinitialize vaapi for new image size */
+    if (width != this->vaapi_width || height != this->vaapi_height) {
+      VAStatus status;
+
+      this->vaapi_width  = width;
+      this->vaapi_height = height;
+      status = this->accel->vaapi_init (this->accel_img, this->vaapi_profile, width, height, 0);
+
+      if (status == VA_STATUS_SUCCESS) {
+        ff_vaapi_context_t *va_context = this->accel->get_context (this->accel_img);
+
+        if (va_context) {
+          this->vaapi_context.config_id  = va_context->va_config_id;
+          this->vaapi_context.context_id = va_context->va_context_id;
+          this->vaapi_context.display    = va_context->va_display;
+        }
+      }
+    }
+
     if(!this->accel->guarded_render(this->accel_img)) {
       img = this->stream->video_out->get_frame (this->stream->video_out,
                                             width,
@@ -275,6 +359,9 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
         av_frame->data[0] = (void *)va_surface;//(void *)(uintptr_t)va_surface->va_surface_id;
         av_frame->data[3] = (void *)(uintptr_t)va_surface->va_surface_id;
       }
+#ifdef AV_BUFFER
+      ffsf->vo_frame = img;
+#endif
     } else {
       ff_vaapi_surface_t *va_surface = this->accel->get_vaapi_surface(this->accel_img);
 
@@ -282,6 +369,9 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
         av_frame->data[0] = (void *)va_surface;//(void *)(uintptr_t)va_surface->va_surface_id;
         av_frame->data[3] = (void *)(uintptr_t)va_surface->va_surface_id;
       }
+#ifdef AV_BUFFER
+      ffsf->va_surface = va_surface;
+#endif
     }
 
     lprintf("1: 0x%08x\n", av_frame->data[3]);
@@ -291,6 +381,16 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
     av_frame->linesize[2] = 0;
     av_frame->linesize[3] = 0;
 
+#ifdef AV_BUFFER
+    /* Does this really work???? */
+    av_frame->buf[0] = av_buffer_create (NULL, 0, release_frame, ffsf, 0);
+    if (av_frame->buf[0])
+      (ffsf->refs)++;
+    av_frame->buf[1] = NULL;
+    av_frame->buf[2] = NULL;
+#else
+    av_frame->type = FF_BUFFER_TYPE_USER;
+#endif
     this->is_direct_rendering_disabled = 1;
 
     return 0;
@@ -305,6 +405,12 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
     guarded_render = this->accel->guarded_render(this->accel_img);
 #endif /* ENABLE_VAAPI */
 
+  /* The alignment rhapsody */
+  buf_width  += 2 * this->edge + 15;
+  buf_width  &= ~15;
+  buf_height += 2 * this->edge + 15;
+  buf_height &= ~15;
+
   if ((this->full2mpeg || (this->context->pix_fmt != PIX_FMT_YUV420P &&
 			   this->context->pix_fmt != PIX_FMT_YUVJ420P)) || guarded_render) {
     if (!this->is_direct_rendering_disabled) {
@@ -317,14 +423,15 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
     av_frame->data[0]= NULL;
     av_frame->data[1]= NULL;
     av_frame->data[2]= NULL;
+#ifdef AV_BUFFER
+    return avcodec_default_get_buffer2(context, av_frame, flags);
+#else
     return avcodec_default_get_buffer(context, av_frame);
+#endif
   }
 
-  if((width != this->bih.biWidth) || (height != this->bih.biHeight)) {
-    if(this->stream->video_out->get_capabilities(this->stream->video_out) & VO_CAP_CROP) {
-      this->crop_right = width - this->bih.biWidth;
-      this->crop_bottom = height - this->bih.biHeight;
-    } else {
+  if ((buf_width != width) || (buf_height != height)) {
+    if (!(this->stream->video_out->get_capabilities(this->stream->video_out) & VO_CAP_CROP)) {
       if (!this->is_direct_rendering_disabled) {
         xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
                 _("ffmpeg_video_dec: unsupported frame dimensions, DR1 disabled.\n"));
@@ -334,21 +441,59 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
       av_frame->data[0]= NULL;
       av_frame->data[1]= NULL;
       av_frame->data[2]= NULL;
+#ifdef AV_BUFFER
+      return avcodec_default_get_buffer2(context, av_frame, flags);
+#else
       return avcodec_default_get_buffer(context, av_frame);
+#endif
     }
   }
 
   this->is_direct_rendering_disabled = 0;
 
   img = this->stream->video_out->get_frame (this->stream->video_out,
-                                            width,
-                                            height,
+                                            buf_width,
+                                            buf_height,
                                             this->aspect_ratio,
                                             this->output_format,
                                             VO_BOTH_FIELDS|this->frame_flags);
 
+#ifdef AV_BUFFER
+  /* Sigh. Wrap vo image planes into AVBufferRefs. When ff unref's them, they unref our trigger.
+    That one then fires image release. */
+  ffsf = calloc (1, sizeof (ff_saved_frame_t));
+  if (!ffsf) {
+    img->free (img);
+    return AVERROR (ENOMEM);
+  }
+  ffsf->this     = this;
+  ffsf->vo_frame = img;
+
+  av_frame->buf[0] = av_buffer_create (img->base[0], img->height * img->pitches[0],
+    release_frame, ffsf, 0);
+  if (av_frame->buf[0]) {
+    (ffsf->refs)++;
+  } else {
+    img->free (img);
+    free (ffsf);
+    return AVERROR (ENOMEM);
+  }
+  av_frame->buf[1] = av_buffer_create (img->base[1], (img->height + 1) / 2 * img->pitches[1],
+    release_frame, ffsf, 0);
+  if (av_frame->buf[1])
+    (ffsf->refs)++;
+  av_frame->buf[2] = av_buffer_create (img->base[2], (img->height + 1) / 2 * img->pitches[2],
+    release_frame, ffsf, 0);
+  if (av_frame->buf[2])
+    (ffsf->refs)++;
+#else
+  av_frame->type = FF_BUFFER_TYPE_USER;
+#endif
+
   av_frame->opaque = img;
 
+  av_frame->extended_data = av_frame->data;
+
   av_frame->data[0]= img->base[0];
   av_frame->data[1]= img->base[1];
   av_frame->data[2]= img->base[2];
@@ -357,6 +502,16 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
   av_frame->linesize[1] = img->pitches[1];
   av_frame->linesize[2] = img->pitches[2];
 
+  if (this->output_format == XINE_IMGFMT_YV12) {
+    av_frame->data[0] += (img->pitches[0] + 1) * this->edge;
+    av_frame->data[1] += (img->pitches[1] + 1) * this->edge / 2;
+    av_frame->data[2] += (img->pitches[2] + 1) * this->edge / 2;
+    img->crop_left   = this->edge;
+    img->crop_top    = this->edge;
+    img->crop_right  = buf_width  - width  - this->edge;
+    img->crop_bottom = buf_height - height - this->edge;
+  }
+
   /* We should really keep track of the ages of xine frames (see
    * avcodec_default_get_buffer in libavcodec/utils.c)
    * For the moment tell ffmpeg that every frame is new (age = bignumber) */
@@ -364,8 +519,6 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
   av_frame->age = 256*256*256*64;
 #endif
 
-  av_frame->type= FF_BUFFER_TYPE_USER;
-
   /* take over pts for this frame to have it reordered */
   av_frame->reordered_opaque = context->reordered_opaque;
 
@@ -374,6 +527,7 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
   return 0;
 }
 
+#ifndef AV_BUFFER
 static void release_buffer(struct AVCodecContext *context, AVFrame *av_frame){
   ff_video_decoder_t *this = (ff_video_decoder_t *)context->opaque;
 
@@ -390,29 +544,33 @@ static void release_buffer(struct AVCodecContext *context, AVFrame *av_frame){
 #endif
 
   if (av_frame->type == FF_BUFFER_TYPE_USER) {
-    if ( av_frame->opaque ) {
+    if (av_frame->opaque) {
       vo_frame_t *img = (vo_frame_t *)av_frame->opaque;
+      xine_list_iterator_t it;
 
-      img->free(img);
+      img->free (img);
+
+      it = xine_list_find (this->dr1_frames, av_frame->opaque);
+      assert (it);
+      if (it != NULL) {
+        xine_list_remove (this->dr1_frames, it);
+      }
     }
 
-    xine_list_iterator_t it;
+    av_frame->opaque  = NULL;
+    av_frame->data[0] = NULL;
+    av_frame->data[1] = NULL;
+    av_frame->data[2] = NULL;
+    av_frame->linesize[0] = 0;
+    av_frame->linesize[1] = 0;
+    av_frame->linesize[2] = 0;
 
-    it = xine_list_find(this->dr1_frames, av_frame->opaque);
-    assert(it);
-    if( it != NULL ) {
-      xine_list_remove(this->dr1_frames, it);
-    }
   } else {
     avcodec_default_release_buffer(context, av_frame);
   }
-
-  av_frame->opaque = NULL;
-  av_frame->data[0]= NULL;
-  av_frame->data[1]= NULL;
-  av_frame->data[2]= NULL;
 }
-#endif
+#endif /* !AV_BUFFER */
+#endif /* ENABLE_DR1 */
 
 #include "ff_video_list.h"
 
@@ -436,9 +594,14 @@ static const int skip_loop_filter_enum_values[] = {
 };
 
 #ifdef ENABLE_VAAPI
+/* TJ. libavcodec calls this with a list of supported pixel formats and lets us choose 1.
+   Returning PIX_FMT_VAAPI_VLD enables VAAPI.
+   However, at this point we only got image width and height from container, being unreliable
+   or zero (MPEG-TS). Thus we repeat vaapi_context initialization in get_buffer when needed.
+   This should be OK since NAL unit parsing is always done in software. */
 static enum PixelFormat get_format(struct AVCodecContext *context, const enum PixelFormat *fmt)
 {
-  int i, profile;
+  int i;
   ff_video_decoder_t *this = (ff_video_decoder_t *)context->opaque;
 
   if(!this->class->enable_vaapi || !this->accel_img)
@@ -450,12 +613,21 @@ static enum PixelFormat get_format(struct AVCodecContext *context, const enum Pi
     if (fmt[i] != PIX_FMT_VAAPI_VLD)
       continue;
 
-    profile = accel->profile_from_imgfmt(this->accel_img, fmt[i], context->codec_id, this->class->vaapi_mpeg_softdec);
+    this->vaapi_profile = accel->profile_from_imgfmt (this->accel_img, fmt[i],
+      context->codec_id, this->class->vaapi_mpeg_softdec);
 
-    if (profile >= 0) {
+    if (this->vaapi_profile >= 0) {
+      int width  = context->width;
+      int height = context->height;
       VAStatus status;
 
-      status = accel->vaapi_init(this->accel_img, profile, context->width, context->height, 0);
+      if (!width || !height) {
+        width  = 1920;
+        height = 1080;
+      }
+      this->vaapi_width  = width;
+      this->vaapi_height = height;
+      status = accel->vaapi_init (this->accel_img, this->vaapi_profile, width, height, 0);
 
       if( status == VA_STATUS_SUCCESS ) {
         ff_vaapi_context_t *va_context = accel->get_context(this->accel_img);
@@ -513,10 +685,20 @@ static void init_video_codec (ff_video_decoder_t *this, unsigned int codec_type)
     _x_stream_info_get(this->stream, XINE_STREAM_INFO_VIDEO_FOURCC);
 
 
-  /* Some codecs (eg rv10) copy flags in init so it's necessary to set
-   * this flag here in case we are going to use direct rendering */
+  this->stream->video_out->open (this->stream->video_out, this->stream);
+
+  this->edge = 0;
   if(this->codec->capabilities & CODEC_CAP_DR1 && this->class->enable_dri) {
-    this->context->flags |= CODEC_FLAG_EMU_EDGE;
+    if (this->stream->video_out->get_capabilities (this->stream->video_out) & VO_CAP_CROP) {
+      /* We can crop. Fine. Lets allow decoders to paint over the frame edges.
+         This will be slightly faster. And it is also a workaround for buggy
+         v54 who likes to ignore EMU_EDGE for wmv2 and xvid. */
+      this->edge = avcodec_get_edge_width ();
+    } else {
+      /* Some codecs (eg rv10) copy flags in init so it's necessary to set
+       * this flag here in case we are going to use direct rendering */
+      this->context->flags |= CODEC_FLAG_EMU_EDGE;
+    }
   }
 
   /* TJ. without this, it wont work at all on my machine */
@@ -547,8 +729,12 @@ static void init_video_codec (ff_video_decoder_t *this, unsigned int codec_type)
   this->output_format = XINE_IMGFMT_YV12;
 #ifdef ENABLE_DIRECT_RENDERING
   if( this->codec->capabilities & CODEC_CAP_DR1 && this->class->enable_dri ) {
+#ifdef AV_BUFFER
+    this->context->get_buffer2 = get_buffer;
+#else
     this->context->get_buffer = get_buffer;
     this->context->release_buffer = release_buffer;
+#endif
     xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
 	    _("ffmpeg_video_dec: direct rendering enabled\n"));
   }
@@ -558,9 +744,13 @@ static void init_video_codec (ff_video_decoder_t *this, unsigned int codec_type)
   if( this->class->enable_vaapi ) {
     this->class->enable_dri = 1;
     this->output_format = XINE_IMGFMT_VAAPI;
+#ifdef AV_BUFFER
+    this->context->get_buffer2 = get_buffer;
+#else
     this->context->get_buffer = get_buffer;
     this->context->reget_buffer = get_buffer;
     this->context->release_buffer = release_buffer;
+#endif
     this->context->get_format = get_format;
     xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
 	    _("ffmpeg_video_dec: direct rendering enabled\n"));
@@ -575,6 +765,7 @@ static void init_video_codec (ff_video_decoder_t *this, unsigned int codec_type)
     free(this->context);
     this->context = NULL;
     _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_HANDLED, 0);
+    this->stream->video_out->close (this->stream->video_out, this->stream);
     return;
   }
 
@@ -590,6 +781,7 @@ static void init_video_codec (ff_video_decoder_t *this, unsigned int codec_type)
       free(this->context);
       this->context = NULL;
       _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_HANDLED, 0);
+      this->stream->video_out->close (this->stream->video_out, this->stream);
       return;
     }
   }
@@ -620,8 +812,6 @@ static void init_video_codec (ff_video_decoder_t *this, unsigned int codec_type)
     set_stream_info(this);
   }
 
-  (this->stream->video_out->open) (this->stream->video_out, this->stream);
-
   this->skipframes = 0;
 
   /* flag for interlaced streams */
@@ -1035,6 +1225,35 @@ static int ff_vc1_find_header(ff_video_decoder_t *this, buf_element_t *buf)
     }
 
     lprintf("ff_video_decoder: found VC1 sequence header\n");
+
+#if AVPARSE > 1
+    AVCodecParserContext *parser_context;
+    uint8_t *outbuf;
+    int      outsize;
+
+    parser_context = av_parser_init(CODEC_ID_VC1);
+    if (!parser_context) {
+      xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+              "ffmpeg_video_dec: couldn't init VC1 parser\n");
+      return 1;
+    }
+
+    parser_context->flags |= PARSER_FLAG_COMPLETE_FRAMES;
+    av_parser_parse2 (parser_context, this->context,
+                      &outbuf, &outsize, this->context->extradata, this->context->extradata_size,
+                      0, 0, 0);
+
+
+    xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+            "ffmpeg_video_dec: parsed VC1 video size %dx%d\n",
+            this->context->width, this->context->height);
+
+    this->bih.biWidth  = this->context->width;
+    this->bih.biHeight = this->context->height;
+
+    av_parser_close(parser_context);
+#endif /* AVPARSE > 1 */
+
     return 1;
   }
 
@@ -1221,7 +1440,7 @@ static void ff_handle_special_buffer (ff_video_decoder_t *this, buf_element_t *b
     decoder_palette->palette_changed = 1;
     this->context->palctrl = decoder_palette;
 
-#elif AVPALETTE == 2
+#elif AVPALETTE == 2 || AVPALETTE == 3
     lprintf ("BUF_SPECIAL_PALETTE\n");
     for (i = 0; i < buf->decoder_info[2]; i++) {
       this->palette[i] =
@@ -1450,9 +1669,6 @@ static void ff_handle_mpeg12_buffer (ff_video_decoder_t *this, buf_element_t *bu
       else
         img->duration = this->video_step;
 
-      img->crop_right  = this->crop_right;
-      img->crop_bottom = this->crop_bottom;
-
 #ifdef ENABLE_VAAPI
       if( this->context->pix_fmt == PIX_FMT_VAAPI_VLD) {
         if(this->accel->guarded_render(this->accel_img)) {
@@ -1464,6 +1680,7 @@ static void ff_handle_mpeg12_buffer (ff_video_decoder_t *this, buf_element_t *bu
 #endif /* ENABLE_VAAPi */
 
       this->skipframes = img->draw(img, this->stream);
+      this->state = STATE_FRAME_SENT;
 
       if(free_img)
         img->free(img);
@@ -1494,6 +1711,30 @@ static void ff_handle_mpeg12_buffer (ff_video_decoder_t *this, buf_element_t *bu
   }
 }
 
+static void ff_postprocess (ff_video_decoder_t *this, vo_frame_t *img) {
+  int qstride, qtype;
+  int8_t *qtable;
+#ifdef AV_BUFFER
+# if LIBAVUTIL_VERSION_MAJOR < 53
+  qtable = av_frame_get_qp_table (this->av_frame, &qstride, &qtype);
+# else
+  /* Why should they keep these long deprecated fields, and remove
+    their safe accessor av_frame_get_qp_table () instead?? */
+  qtable  = this->av_frame->qscale_table;
+  qstride = this->av_frame->qstride;
+  qtype   = this->av_frame->qscale_type;
+# endif
+#else
+  qtable  = this->av_frame->qscale_table;
+  qstride = this->av_frame->qstride;
+  qtype   = 0;
+#endif
+  pp_postprocess ((const uint8_t **)this->av_frame->data, this->av_frame->linesize,
+                  img->base, img->pitches, this->bih.biWidth, this->bih.biHeight,
+                  qtable, qstride, this->our_mode, this->our_context,
+                  this->av_frame->pict_type | (qtype ? PP_PICT_TYPE_QP2 : 0));
+}
+
 static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
   uint8_t *chunk_buf = this->buf;
   AVRational avr00 = {0, 1};
@@ -1587,7 +1828,7 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
 	avpkt.data = (uint8_t *)&chunk_buf[offset];
 	avpkt.size = this->size;
 	avpkt.flags = AV_PKT_FLAG_KEY;
-# if AVPALETTE == 2
+# if AVPALETTE == 2 || AVPALETTE == 3
 	if (this->palette_changed) {
 	  uint8_t *sd = av_packet_new_side_data (&avpkt, AV_PKT_DATA_PALETTE, 256 * 4);
 	  if (sd)
@@ -1604,15 +1845,20 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
 	len = avcodec_decode_video2 (this->context, this->av_frame,
 				     &got_picture, &avpkt);
 	}
-# if AVPALETTE == 2
+# if AVPALETTE == 2 || AVPALETTE == 3
 	if (this->palette_changed) {
+	  /* Prevent freeing our data buffer */
+	  avpkt.data = NULL;
+	  avpkt.size = 0;
+#  if AVPALETTE == 2
 	  /* TJ. Oh dear and sigh.
 	      AVPacket side data handling is broken even in ffmpeg 1.1.1 - see avcodec/avpacket.c
 	      The suggested av_free_packet () would leave a memory leak here, and
 	      ff_packet_free_side_data () is private. */
-	  avpkt.data = NULL;
-	  avpkt.size = 0;
 	  av_destruct_packet (&avpkt);
+#  else /* AVPALETTE == 3 */
+	  av_free_packet (&avpkt);
+#  endif
 	  this->palette_changed = 0;
 	}
 # endif
@@ -1693,9 +1939,11 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
 
           /* prepare for colorspace conversion */
 #ifdef ENABLE_VAAPI
-          if (this->context->pix_fmt != PIX_FMT_VAAPI_VLD) {
+          if (this->context->pix_fmt != PIX_FMT_VAAPI_VLD && !this->cs_convert_init)
+#else
+          if (!this->cs_convert_init)
 #endif
-          if (!this->cs_convert_init) {
+          {
             xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "ff_video_dec: PIX_FMT %d\n", this->context->pix_fmt);
             switch (this->context->pix_fmt) {
               case PIX_FMT_ARGB:
@@ -1713,9 +1961,6 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
             }
             this->cs_convert_init = 1;
           }
-#ifdef ENABLE_VAAPI
-          }
-#endif
 
 	  if (this->aspect_ratio_prio == 0) {
 	    this->aspect_ratio = (double)this->bih.biWidth / (double)this->bih.biHeight;
@@ -1731,6 +1976,8 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
                                                     this->aspect_ratio,
                                                     this->output_format,
                                                     VO_BOTH_FIELDS|this->frame_flags);
+          img->crop_right  = img->width  - this->bih.biWidth;
+          img->crop_bottom = img->height - this->bih.biHeight;
           free_img = 1;
         } else {
           /* DR1 */
@@ -1745,23 +1992,19 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
         if(this->pp_available && this->pp_quality && this->context->pix_fmt != PIX_FMT_VAAPI_VLD) {
 
           if(this->av_frame->opaque) {
-            /* DR1 */
+            /* DR1: filter into a new frame. Same size to avoid reallcation, just move the
+               image to top left corner. */
             img = this->stream->video_out->get_frame (this->stream->video_out,
-                                                      (img->width  + 15) & ~15,
-                                                      (img->height + 15) & ~15,
+                                                      img->width,
+                                                      img->height,
                                                       this->aspect_ratio,
                                                       this->output_format,
                                                       VO_BOTH_FIELDS|this->frame_flags);
+            img->crop_right  = img->width  - this->bih.biWidth;
+            img->crop_bottom = img->height - this->bih.biHeight;
             free_img = 1;
           }
-
-          pp_postprocess((const uint8_t **)this->av_frame->data, this->av_frame->linesize,
-                        img->base, img->pitches,
-                        img->width, img->height,
-                        this->av_frame->qscale_table, this->av_frame->qstride,
-                        this->our_mode, this->our_context,
-                        this->av_frame->pict_type);
-
+          ff_postprocess (this, img);
         } else if (!this->av_frame->opaque) {
 	  /* colorspace conversion or copy */
           if( this->context->pix_fmt != PIX_FMT_VAAPI_VLD)
@@ -1786,10 +2029,6 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
         else
           img->duration = video_step_to_use;
 
-        /* additionally crop away the extra pixels due to adjusting frame size above */
-        img->crop_right  = img->width  - this->bih.biWidth;
-        img->crop_bottom = img->height - this->bih.biHeight;
-
         /* transfer some more frame settings for deinterlacing */
         img->progressive_frame = !this->av_frame->interlaced_frame;
         img->top_field_first   = this->av_frame->top_field_first;
@@ -1806,6 +2045,7 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
 #endif /* ENABLE_VAAPI */
 
         this->skipframes = img->draw(img, this->stream);
+        this->state = STATE_FRAME_SENT;
 
         if(free_img)
           img->free(img);
@@ -1836,6 +2076,7 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
       img->bad_frame = 1;
       this->skipframes = img->draw(img, this->stream);
       img->free(img);
+      this->state = STATE_FRAME_SENT;
     }
 
     this->assume_bad_field_picture = !got_one_picture;
@@ -1887,6 +2128,7 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
       if (buf->pts && (buf->pts != this->last_pts))
 	this->last_pts = this->pts = buf->pts;
 
+      this->state = STATE_READING_DATA;
       if ((buf->type & 0xFFFF0000) == BUF_VIDEO_MPEG) {
 	ff_handle_mpeg12_buffer(this, buf);
       } else {
@@ -1897,8 +2139,193 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
   }
 }
 
+static void ff_flush_internal (ff_video_decoder_t *this, int display) {
+  vo_frame_t *img;
+  int         free_img, got_picture, len, frames = 0;
+  int         video_step_to_use = this->video_step;
+  AVRational  avr00 = {0, 1};
+
+  /* This is a stripped version of ff_handle_buffer (). It shall return yet undisplayed frames. */
+  if (!this->context || !this->decoder_ok || this->state == STATE_FLUSHED)
+    return;
+  this->state = STATE_FLUSHED;
+
+  while (1) {
+#if AVVIDEO > 1
+    AVPacket avpkt;
+    av_init_packet (&avpkt);
+    avpkt.data  = NULL;
+    avpkt.size  = 0;
+    got_picture = 0;
+# if ENABLE_VAAPI
+    if (this->accel)
+      len = this->accel->avcodec_decode_video2 (this->accel_img, this->context,
+        this->av_frame, &got_picture, &avpkt);
+    else
+# endif
+      len = avcodec_decode_video2 (this->context, this->av_frame, &got_picture, &avpkt);
+#else
+    got_picture = 0;
+# if ENABLE_VAAPI
+    if (this->accel)
+      len = this->accel->avcodec_decode_video (this->accel_img, this->context,
+        this->av_frame, &got_picture, NULL, 0);
+    else
+# endif
+      len = avcodec_decode_video (this->context, this->av_frame, &got_picture, NULL, 0);
+#endif
+    if (len < 0 || !got_picture || !this->av_frame->data[0])
+      break;
+
+    frames++;
+    if (!display)
+      continue;
+
+    /* All that jizz just to view the last 2 frames of a stream ;-) */
+    video_step_to_use = this->video_step || !this->context->time_base.den ?
+      this->video_step :
+      90000ll * this->context->ticks_per_frame * this->context->time_base.num /
+        this->context->time_base.den;
+
+    if ((this->aspect_ratio_prio < 2) && av_cmp_q (this->context->sample_aspect_ratio, avr00)) {
+      if (!this->bih.biWidth || !this->bih.biHeight) {
+        this->bih.biWidth  = this->context->width;
+        this->bih.biHeight = this->context->height;
+      }
+      this->aspect_ratio = av_q2d(this->context->sample_aspect_ratio) *
+        (double)this->bih.biWidth / (double)this->bih.biHeight;
+      this->aspect_ratio_prio = 2;
+      set_stream_info (this);
+    }
+
+    if (this->set_stream_info) {
+      set_stream_info (this);
+      this->set_stream_info = 0;
+    }
+
+    if (!this->av_frame->opaque) {
+      /* indirect rendering */
+      if (this->aspect_ratio_prio == 0) {
+        this->aspect_ratio = (double)this->bih.biWidth / (double)this->bih.biHeight;
+        this->aspect_ratio_prio = 1;
+        lprintf ("default aspect ratio: %f\n", this->aspect_ratio);
+        set_stream_info (this);
+      }
+      /* xine-lib expects the framesize to be a multiple of 16x16 (macroblock) */
+      img = this->stream->video_out->get_frame (this->stream->video_out,
+        (this->bih.biWidth  + 15) & ~15, (this->bih.biHeight + 15) & ~15,
+        this->aspect_ratio, this->output_format, VO_BOTH_FIELDS | this->frame_flags);
+      img->crop_right  = img->width  - this->bih.biWidth;
+      img->crop_bottom = img->height - this->bih.biHeight;
+      free_img = 1;
+    } else {
+      /* DR1 */
+      img = (vo_frame_t*)this->av_frame->opaque;
+      free_img = 0;
+    }
+
+    /* post processing */
+    if (this->pp_quality != this->class->pp_quality && this->context->pix_fmt != PIX_FMT_VAAPI_VLD)
+      pp_change_quality (this);
+    if (this->pp_available && this->pp_quality && this->context->pix_fmt != PIX_FMT_VAAPI_VLD) {
+      if (this->av_frame->opaque) {
+        /* DR1: filter into a new frame. Same size to avoid reallcation, just move the
+           image to top left corner. */
+        img = this->stream->video_out->get_frame (this->stream->video_out, img->width, img->height,
+          this->aspect_ratio, this->output_format, VO_BOTH_FIELDS | this->frame_flags);
+        img->crop_right  = img->width  - this->bih.biWidth;
+        img->crop_bottom = img->height - this->bih.biHeight;
+        free_img = 1;
+      }
+      ff_postprocess (this, img);
+    } else if (!this->av_frame->opaque) {
+      /* colorspace conversion or copy */
+      if (this->context->pix_fmt != PIX_FMT_VAAPI_VLD)
+        ff_convert_frame (this, img, this->av_frame);
+    }
+
+    img->pts = ff_untag_pts (this, this->av_frame->reordered_opaque);
+    ff_check_pts_tagging (this, this->av_frame->reordered_opaque);
+
+    if (video_step_to_use == 750)
+      video_step_to_use = 0;
+    img->duration = this->av_frame->repeat_pict ? video_step_to_use * 3 / 2 : video_step_to_use;
+    img->progressive_frame = !this->av_frame->interlaced_frame;
+    img->top_field_first   = this->av_frame->top_field_first;
+
+#ifdef ENABLE_VAAPI
+    if (this->context->pix_fmt == PIX_FMT_VAAPI_VLD) {
+      if (this->accel->guarded_render(this->accel_img)) {
+        ff_vaapi_surface_t *va_surface = (ff_vaapi_surface_t *)this->av_frame->data[0];
+        this->accel->render_vaapi_surface (img, va_surface);
+      }
+    }
+#endif /* ENABLE_VAAPI */
+
+    this->skipframes = img->draw (img, this->stream);
+    if (free_img)
+      img->free (img);
+  }
+
+  if (frames)
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+      "ffmpeg_video_dec: flushed out %s%d frames.\n",
+      display ? "and displayed " : "", frames);
+}
+
+static void ff_free_dr1_frames (ff_video_decoder_t *this, int all) {
+  xine_list_iterator_t it;
+  int frames;
+  /* Some codecs (wmv2, vp6, svq3...) are hard-wired to a few reference frames.
+     They will only be replaced when new ones arrive, and freed on codec close.
+     They also have no AVCodec.flush () callback for manual freeing (that is,
+     avcodec_flush_buffers () does nothing).
+     So lets tolerate this behaviour on plain stream seek. */
+  if (!all) {
+    it = NULL;
+    frames = 0;
+    while ((it = xine_list_next (this->dr1_frames, it)) != NULL)
+      frames++;
+    if (!frames)
+      return;
+    if (frames < 5) {
+      xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+        "ffmpeg_video_dec: tolerating %d held DR1 frames.\n", frames);
+      return;
+    }
+  }
+  /* frame garbage collector here - workaround for buggy ffmpeg codecs that
+   * don't release their DR1 frames */
+  /* TJ. A note on libavcodec v55.
+     Looks like I found a way not to get to this point.
+     If it should happen anyway, we may earn some small memory leaks here.
+     Using the old get_buffer () API emulation makes even bigger leaks
+     (per frame, +3*AVBufferRef, +1*AVBuffer, +1*AVCodecContext, +1*AVFrame).
+     Tracking and killing the AVBufferRef's themselves would risk heap
+     corruption and segfaults.
+     Now tell me whether I really sigh too much... */
+  it = NULL;
+  frames = 0;
+  while ((it = xine_list_next (this->dr1_frames, it)) != NULL) {
+    vo_frame_t *img = (vo_frame_t *)xine_list_get_value(this->dr1_frames, it);
+    if (img) {
+      img->free (img);
+      frames++;
+    }
+  }
+  xine_list_clear (this->dr1_frames);
+  /* we probably never get this. */
+  if (frames)
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+      "ffmpeg_video_dec: freed %d orphaned DR1 frames.\n", frames);
+}
+
 static void ff_flush (video_decoder_t *this_gen) {
+  ff_video_decoder_t *this = (ff_video_decoder_t *) this_gen;
+
   lprintf ("ff_flush\n");
+  if (this->state == STATE_FRAME_SENT)
+    ff_flush_internal (this, 1);
 }
 
 static void ff_reset (video_decoder_t *this_gen) {
@@ -1907,22 +2334,16 @@ static void ff_reset (video_decoder_t *this_gen) {
   lprintf ("ff_reset\n");
 
   this->size = 0;
+  this->state = STATE_RESET;
 
   if(this->context && this->decoder_ok)
   {
-    xine_list_iterator_t it = NULL;
-
+    /* Discard any undisplayed frames. */
+    ff_flush_internal (this, 0);
+    /* Ask decoder to free held reference frames (which it may ignore). */
     avcodec_flush_buffers(this->context);
-
-    /* frame garbage collector here - workaround for buggy ffmpeg codecs that
-     * don't release their DR1 frames */
-    while ((it = xine_list_next (this->dr1_frames, it)) != NULL)
-    {
-      vo_frame_t *img = (vo_frame_t *)xine_list_get_value(this->dr1_frames, it);
-      if (img)
-        img->free(img);
-    }
-    xine_list_clear(this->dr1_frames);
+    /* Free obviously too many DR1 frames. */
+    ff_free_dr1_frames (this, 0);
   }
 
   if (this->is_mpeg12)
@@ -1939,6 +2360,7 @@ static void ff_discontinuity (video_decoder_t *this_gen) {
 
   lprintf ("ff_discontinuity\n");
   this->pts = 0;
+  this->state = STATE_DISCONTINUITY;
 
   /*
    * there is currently no way to reset all the pts which are stored in the decoder.
@@ -1979,23 +2401,17 @@ static void ff_dispose (video_decoder_t *this_gen) {
 
   lprintf ("ff_dispose\n");
 
+  ff_flush_internal (this, 0);
+
   rgb2yuy2_free (this->rgb2yuy2);
 
   if (this->decoder_ok) {
-    xine_list_iterator_t it = NULL;
 
     pthread_mutex_lock(&ffmpeg_lock);
     avcodec_close (this->context);
     pthread_mutex_unlock(&ffmpeg_lock);
 
-    /* frame garbage collector here - workaround for buggy ffmpeg codecs that
-     * don't release their DR1 frames */
-    while ((it = xine_list_next (this->dr1_frames, it)) != NULL)
-    {
-      vo_frame_t *img = (vo_frame_t *)xine_list_get_value(this->dr1_frames, it);
-      if (img)
-        img->free(img);
-    }
+    ff_free_dr1_frames (this, 1);
 
     this->stream->video_out->close(this->stream->video_out, this->stream);
     this->decoder_ok = 0;
