@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2020 the xine project
+ * Copyright (C) 2001-2013 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -21,34 +21,40 @@
 #ifndef HAVE_XINE_DECODER_H
 #define HAVE_XINE_DECODER_H
 
-#include <sys/types.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
-#include <xine.h>
-
-#if defined LIBAVCODEC_VERSION_INT
+#ifdef HAVE_FFMPEG_AVUTIL_H
+#  include <avcodec.h>
+#else
+#  include <libavcodec/avcodec.h>
+#  include <libavutil/mem.h>
+#endif
 
 typedef struct ff_codec_s {
   uint32_t          type;
-# if LIBAVCODEC_VERSION_INT >= ((54<<16)|(25<<8))
+#if defined LIBAVCODEC_VERSION_INT && LIBAVCODEC_VERSION_INT >= ((54<<16)|(25<<8))
   enum AVCodecID    id;
-# else
+#else
   enum CodecID      id;
-# endif
+#endif
   const char       *name;
 } ff_codec_t;
 
-extern const ff_codec_t ff_audio_lookup[];
-extern const ff_codec_t ff_video_lookup[];
-extern const size_t ff_video_lookup_entries;
-extern const size_t ff_audio_lookup_entries;
+void *init_audio_plugin (xine_t *xine, void *data);
+void *init_video_plugin (xine_t *xine, void *data);
+void *init_avio_input_plugin (xine_t *xine, void *data);
+void *init_avformat_input_plugin (xine_t *xine, void *data);
+void *init_avformat_demux_plugin (xine_t *xine, void *data);
 
-#endif
-
-void *init_audio_plugin (xine_t *xine, const void *data);
-void *init_video_plugin (xine_t *xine, const void *data);
-void *init_avio_input_plugin (xine_t *xine, const void *data);
-void *init_avformat_input_plugin (xine_t *xine, const void *data);
-void *init_avformat_demux_plugin (xine_t *xine, const void *data);
+extern decoder_info_t dec_info_ffmpeg_video;
+extern decoder_info_t dec_info_ffmpeg_wmv8;
+extern decoder_info_t dec_info_ffmpeg_wmv9;
+extern decoder_info_t dec_info_ffmpeg_audio;
+extern demuxer_info_t demux_info_avformat;
+extern input_info_t   input_info_avformat;
+extern input_info_t   input_info_avio;
 
 /* communication between avio/avformat input and avformat demux plugins */
 #define INPUT_OPTIONAL_DATA_pb         0x1000
@@ -58,6 +64,7 @@ void *init_avformat_demux_plugin (xine_t *xine, const void *data);
 #define INPUT_AVIO_ID     "avio"
 #define DEMUX_AVFORMAT_ID "avformat"
 
+extern pthread_once_t once_control;
 void init_once_routine(void);
 
 extern pthread_mutex_t ffmpeg_lock;
