@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2019 the xine project
+ * Copyright (C) 2013 the xine project
  * Copyright (C) 2013 Petri Hintukainen <phintuka@users.sourceforge.net>
  *
  * This file is part of xine, a free video player.
@@ -26,7 +26,6 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <pthread.h>
 
 #include <libavformat/avio.h>
 
@@ -55,11 +54,10 @@ typedef struct {
   char            *mrl_private; /* 'private' mrl with authentication credentials */
   AVIOContext     *pb;
 
-  off_t            curpos;
-
   /* preview support */
-  size_t           preview_size;
   char             preview[MAX_PREVIEW_SIZE];
+  off_t            preview_size;
+  off_t            curpos;
 
 } avio_input_plugin_t;
 
@@ -71,12 +69,12 @@ static off_t input_avio_read (input_plugin_t *this_gen, void *buf_gen, off_t len
   if (len < 0)
     return -1;
 
-  if (this->curpos < (off_t)this->preview_size) {
+  if (this->curpos < this->preview_size) {
     off_t n = this->preview_size - this->curpos;
-    if (n > len)
-      n = len;
+    if (n > (len - total))
+      n = len - total;
 
-    memcpy (buf, &this->preview[this->curpos], n);
+    memcpy (&buf[total], &this->preview[this->curpos], n);
     this->curpos += n;
     total += n;
     len -= n;
@@ -95,9 +93,6 @@ static off_t input_avio_read (input_plugin_t *this_gen, void *buf_gen, off_t len
 }
 
 static buf_element_t *input_avio_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo, off_t todo) {
-  (void)this_gen;
-  (void)fifo;
-  (void)todo;
   return NULL;
 }
 
@@ -138,14 +133,13 @@ static off_t input_avio_seek_time (input_plugin_t *this_gen, int time_offset, in
 }
 
 static uint32_t input_avio_get_blocksize (input_plugin_t *this_gen) {
-  (void)this_gen;
   return 0;
 }
 
 static off_t input_avio_get_current_pos (input_plugin_t *this_gen) {
   avio_input_plugin_t *this = (avio_input_plugin_t *) this_gen;
 
-  if (this->pb && this->curpos >= (off_t)this->preview_size) {
+  if (this->pb && this->curpos >= this->preview_size) {
     this->curpos = avio_tell(this->pb);
   }
 
@@ -184,7 +178,7 @@ static off_t input_avio_seek (input_plugin_t *this_gen, off_t offset, int origin
   /* seek, take care of preview buffer */
 
   newpos = offset;
-  if (offset < (off_t)this->preview_size) {
+  if (offset < this->preview_size) {
     offset = this->preview_size;
   }
 
@@ -231,14 +225,14 @@ static int input_avio_open (input_plugin_t *this_gen) {
     /* try to open libavio protocol */
     if (avio_open2(&this->pb, this->mrl_private, AVIO_FLAG_READ, NULL, NULL) < 0) {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": failed to open avio protocol for '%s'\n", this->mrl);
-      _x_freep_wipe_string (&this->mrl_private);
+      _x_freep (&this->mrl_private);
 
       return 0;
     }
 
     xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": opened avio protocol for '%s'\n", this->mrl);
   }
-  _x_freep_wipe_string (&this->mrl_private);
+  _x_freep (&this->mrl_private);
 
   while ((toread > 0) && (trycount < 10)) {
     off_t n = avio_read (this->pb, this->preview + this->preview_size, toread);
@@ -257,7 +251,7 @@ static void input_avio_dispose (input_plugin_t *this_gen ) {
 
   avio_close(this->pb);
   _x_freep (&this->mrl);
-  _x_freep_wipe_string (&this->mrl_private);
+  _x_freep (&this->mrl_private);
   free (this_gen);
 }
 
@@ -317,10 +311,6 @@ static input_plugin_t *input_avio_get_instance (input_class_t *cls_gen, xine_str
   }
 
   this = calloc(1, sizeof(avio_input_plugin_t));
-  if (!this) {
-    return NULL;
-  }
-
   this->stream = stream;
   this->mrl    = _x_mrl_remove_auth(mrl);
   this->mrl_private = strdup(mrl);
@@ -345,27 +335,30 @@ static input_plugin_t *input_avio_get_instance (input_class_t *cls_gen, xine_str
   return &this->input_plugin;
 }
 
-void *init_avio_input_plugin (xine_t *xine, const void *data) {
+void *init_avio_input_plugin (xine_t *xine, void *data) {
+  input_class_t  *this;
   const char     *protocol;
   void           *iter;
-  static const input_class_t this = {
-    .get_instance      = input_avio_get_instance,
-    .description       = N_("libavio input plugin"),
-    .identifier        = INPUT_AVIO_ID,
-    .get_dir           = NULL,
-    .get_autoplay_list = NULL,
-    .dispose           = NULL,
-    .eject_media       = NULL
-  };
 
-  (void)data;
   for (iter = NULL; NULL != (protocol = avio_enum_protocols(&iter, 0)); ) {
     xprintf (xine, XINE_VERBOSITY_DEBUG, LOG_MODULE": found avio protocol '%s'\n", protocol);
   }
+
+  this = calloc(1, sizeof(input_class_t));
+
   pthread_once( &once_control, init_once_routine );
-  return (input_class_t *)&this;
+
+  this->get_instance      = input_avio_get_instance;
+  this->description       = N_("libavio input plugin");
+  this->identifier        = INPUT_AVIO_ID;
+  this->get_dir           = NULL;
+  this->get_autoplay_list = NULL;
+  this->dispose           = default_input_class_dispose;
+  this->eject_media       = NULL;
+
+  return this;
 }
 
-const input_info_t input_info_avio = {
+input_info_t input_info_avio = {
   -1   /* priority */
 };
