@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2013 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -43,8 +43,6 @@
 #define LOG
 */
 
-#include "group_video.h"
-
 #include <xine/xine_internal.h>
 #include <xine/xineutils.h>
 #include <xine/demux.h>
@@ -64,7 +62,7 @@ static void check_newpts (demux_matroska_t *this, int64_t pts,
 
     diff = pts - track->last_pts;
 
-    if (pts && (this->send_newpts || (track->last_pts && llabs(diff)>WRAP_THRESHOLD)) ) {
+    if (pts && (this->send_newpts || (track->last_pts && abs(diff)>WRAP_THRESHOLD)) ) {
       int i;
 
       lprintf ("sending newpts %" PRId64 ", diff %" PRId64 ", track %d\n", pts, diff, track->track_num);
@@ -177,11 +175,10 @@ static int parse_info(demux_matroska_t *this) {
 
 static int parse_video_track (demux_matroska_t *this, matroska_video_track_t *vt) {
   ebml_parser_t *ebml = this->ebml;
-  int this_level = ebml->level;
-  int next_level = this_level;
+  int next_level = 4;
   uint64_t val;
 
-  while (next_level == this_level) {
+  while (next_level == 4) {
     ebml_elem_t elem;
 
     if (!ebml_read_elem_head(ebml, &elem))
@@ -231,10 +228,9 @@ static int parse_video_track (demux_matroska_t *this, matroska_video_track_t *vt
 
 static int parse_audio_track (demux_matroska_t *this, matroska_audio_track_t *at) {
   ebml_parser_t *ebml = this->ebml;
-  int this_level = ebml->level;
-  int next_level = this_level;
+  int next_level = 4;
 
-  while (next_level == this_level) {
+  while (next_level == 4) {
     ebml_elem_t elem;
     uint64_t    val;
     double      fval;
@@ -280,10 +276,9 @@ static int parse_audio_track (demux_matroska_t *this, matroska_audio_track_t *at
 
 static int parse_content_compression (demux_matroska_t *this, matroska_track_t *track) {
   ebml_parser_t *ebml = this->ebml;
-  int this_level = ebml->level;
-  int next_level = this_level;
+  int next_level = 6;
 
-  while (next_level == this_level) {
+  while (next_level == 6) {
     ebml_elem_t elem;
     uint64_t    val;
 
@@ -330,10 +325,9 @@ static int parse_content_compression (demux_matroska_t *this, matroska_track_t *
 
 static int parse_content_encoding (demux_matroska_t *this, matroska_track_t *track) {
   ebml_parser_t *ebml = this->ebml;
-  int this_level = ebml->level;
-  int next_level = this_level;
+  int next_level = 5;
 
-  while (next_level == this_level) {
+  while (next_level == 5) {
     ebml_elem_t elem;
     uint64_t    val;
 
@@ -370,7 +364,6 @@ static int parse_content_encoding (demux_matroska_t *this, matroska_track_t *tra
         lprintf("ContentCompression\n");
         if (!ebml_read_master (ebml, &elem))
           return 0;
-	track->compress_algo = 0; /* default */
         if ((elem.len > 0) && !parse_content_compression(this, track))
           return 0;
         break;
@@ -392,10 +385,9 @@ static int parse_content_encoding (demux_matroska_t *this, matroska_track_t *tra
 
 static int parse_content_encodings (demux_matroska_t *this, matroska_track_t *track) {
   ebml_parser_t *ebml = this->ebml;
-  int this_level = ebml->level;
-  int next_level = this_level;
+  int next_level = 4;
 
-  while (next_level == this_level) {
+  while (next_level == 4) {
     ebml_elem_t elem;
 
     if (!ebml_read_elem_head(ebml, &elem))
@@ -887,8 +879,6 @@ static void handle_sub_ssa (demux_plugin_t *this_gen, matroska_track_t *track,
                             uint8_t *data, size_t data_len,
                             int64_t data_pts, int data_duration,
                             int input_normpos, int input_time) {
-  demux_matroska_t *this = (demux_matroska_t *)this_gen;
-
   buf_element_t *buf;
   uint32_t *val;
   int commas = 0;
@@ -909,8 +899,8 @@ static void handle_sub_ssa (demux_plugin_t *this_gen, matroska_track_t *track,
   buf->type = track->buf_type;
   buf->decoder_flags = decoder_flags | BUF_FLAG_SPECIAL;
   buf->decoder_info[1] = BUF_SPECIAL_CHARSET_ENCODING;
-  buf->decoder_info_ptr[2] = this->class->literal_utf_8;
-  buf->decoder_info[2] = LITERAL_UTF_8_LEN;
+  buf->decoder_info_ptr[2] = "utf-8";
+  buf->decoder_info[2] = strlen(buf->decoder_info_ptr[2]);
 
   val = (uint32_t *)buf->content;
   *val++ = data_pts / 90;                    /* start time */
@@ -978,8 +968,8 @@ static void handle_sub_utf8 (demux_plugin_t *this_gen, matroska_track_t *track,
     buf->type = track->buf_type;
     buf->decoder_flags = decoder_flags | BUF_FLAG_SPECIAL;
     buf->decoder_info[1] = BUF_SPECIAL_CHARSET_ENCODING;
-    buf->decoder_info_ptr[2] = this->class->literal_utf_8;
-    buf->decoder_info[2] = LITERAL_UTF_8_LEN;
+    buf->decoder_info_ptr[2] = "utf-8";
+    buf->decoder_info[2] = strlen(buf->decoder_info_ptr[2]);
 
     val = (uint32_t *)buf->content;
     *val++ = data_pts / 90;                    /* start time */
@@ -999,157 +989,6 @@ static void handle_sub_utf8 (demux_plugin_t *this_gen, matroska_track_t *track,
   }
 }
 
-/* 0x02 (not a keyframe) | 0x01 (is visible) */
-static int vp9_frametype (const uint8_t *h) {
-  if ((h[0] & 0xc0) != 0x80) /* frame marker missing */
-    return 1;
-  if ((h[0] & 0x30) == 0x30) { /* profile 3+ */
-    if (h[0] & 0x08) /* profile 4+ not supported */
-      return 0;
-    if (h[0] & 0x04) /* 3 bits ref */
-      return 1;
-    return h[0] & 3;
-  }
-  if (h[0] & 0x08) /* 3 bits ref */
-    return 1;
-  return (h[0] >> 1) & 3;
-}
-
-static void handle_vp9 (demux_plugin_t *this_gen, matroska_track_t *track, int decoder_flags,
-  uint8_t *data, size_t data_len, int64_t data_pts, int data_duration, int input_normpos, int input_time) {
-  /* !@$%*#~ undocumented multiframe feature */
-  do {
-    uint8_t mfhead;
-    uint8_t *l;
-    int mfframes, i;
-    size_t mfbytes, mfsize;
-
-    if (!data_len)
-      return;
-    mfhead = data[data_len - 1];
-    if ((mfhead & 0xe0) != 0xc0)
-      break;
-    mfframes =  (mfhead & 7) + 1;
-    mfbytes  = ((mfhead >> 3) & 3) + 1;
-    mfsize   =   mfframes * mfbytes + 2;
-    if (mfsize > data_len)
-      break;
-    l = data + data_len - mfsize;
-    if (*l++ != mfhead)
-      break;
-    data_len -= mfsize;
-    data_duration /= mfframes;
-    for (i = 0; i < mfframes; i++) {
-      size_t flen = *l++;
-      if (mfbytes > 1)
-        flen += (size_t)(*l++) << 8;
-      if (mfbytes > 2)
-        flen += (size_t)(*l++) << 16;
-      if (mfbytes > 3)
-        flen += (size_t)(*l++) << 24;
-      if (flen > data_len)
-        flen = data_len;
-      if (flen) {
-        /* Nasty kludge: take invisible frames out of pts sequence.
-         * A conventional b-frame sequence looks like this:
-         * I1 P1 B1 B2 B3 P2 B4 ...
-         * after reordering:
-         * I1 B1 B2 B3 P1 B4 ... P2
-         * VP9 now avoids reordering and storing both dts and pts
-         * by adding "P" as "invisible" to a (multiframe group):
-         * I1 (P1 B1) B2 B3 (P2 B4) ...
-         * "B3" usually is very small, and yields a quite verbatim copy of "P1",
-         * which does not stay completely hidden this way.
-         */
-        int64_t pts = data_pts;
-        int type = vp9_frametype (data);
-        if (type & 2)
-          decoder_flags &= ~BUF_FLAG_KEYFRAME;
-        else
-          decoder_flags |= BUF_FLAG_KEYFRAME;
-        if (type & 1) { /* visible */
-          if (!pts)
-            pts = track->delayed_pts;
-          track->delayed_pts = data_pts = 0;
-        } else { /* invisible */
-          pts = 0;
-          track->delayed_pts = data_pts;
-        }
-        _x_demux_send_data (track->fifo, data, flen, pts, track->buf_type,
-          decoder_flags, input_normpos, input_time, data_duration, 0);
-        data += flen;
-        data_len -= flen;
-      }
-    }
-    return;
-  } while (0);
-
-  {
-    int64_t pts = data_pts;
-    int type = vp9_frametype (data);
-    if (type & 2)
-      decoder_flags &= ~BUF_FLAG_KEYFRAME;
-    else
-      decoder_flags |= BUF_FLAG_KEYFRAME;
-    if (type & 1) { /* visible */
-      if (!pts)
-        pts = track->delayed_pts;
-      track->delayed_pts = data_pts = 0;
-    } else { /* invisible */
-      pts = 0;
-      track->delayed_pts = data_pts;
-    }
-    _x_demux_send_data (track->fifo, data, data_len, pts, track->buf_type,
-      decoder_flags, input_normpos, input_time, data_duration, 0);
-  }
-}
-
-
-static int uncompress_zlib(demux_matroska_t *this,
-                           const uint8_t *data, size_t data_len, uint8_t **out_data, size_t *out_data_len)
-{
-    z_stream zstream;
-    uint8_t *dest;
-    int result;
-
-    *out_data = NULL;
-
-    zstream.zalloc = (alloc_func) 0;
-    zstream.zfree = (free_func) 0;
-    zstream.opaque = (voidpf) 0;
-    if (inflateInit (&zstream) != Z_OK) {
-      xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-              "demux_matroska: zlib inflateInit failed.\n");
-      return -1;
-    }
-    zstream.next_in = (Bytef *)data;
-    zstream.avail_in = data_len;
-
-    dest = (uint8_t *)malloc(data_len);
-    zstream.avail_out = data_len;
-    do {
-      data_len += 4000;
-      dest = (uint8_t *)realloc(dest, data_len);
-      zstream.next_out = (Bytef *)(dest + zstream.total_out);
-      result = inflate (&zstream, Z_NO_FLUSH);
-      if ((result != Z_OK) && (result != Z_STREAM_END)) {
-        xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-                "demux_matroska: zlib decompression failed: %d\n", result);
-        free(dest);
-        inflateEnd(&zstream);
-	return 0;
-      }
-      zstream.avail_out += 4000;
-    } while ((zstream.avail_out == 4000) &&
-            (zstream.avail_in != 0) && (result != Z_STREAM_END));
-
-    *out_data = dest;
-    *out_data_len = zstream.total_out;
-
-    inflateEnd(&zstream);
-
-    return 1;
-}
 
 /* Note: This function assumes that the VobSub track is compressed with zlib.
  * This is not necessarily true - or not enough. The Matroska 'content
@@ -1164,33 +1003,65 @@ static void handle_vobsub (demux_plugin_t *this_gen, matroska_track_t *track,
                            int input_normpos, int input_time) {
   demux_matroska_t *this = (demux_matroska_t *) this_gen;
   buf_element_t *buf;
-  uint8_t *new_data = NULL;
-  size_t new_data_len = 0;
 
   if (track->compress_algo == MATROSKA_COMPRESS_ZLIB ||
       track->compress_algo == MATROSKA_COMPRESS_UNKNOWN) {
+    z_stream zstream;
+    uint8_t *dest;
+    int old_data_len, result;
 
-    if (uncompress_zlib(this, data, data_len, &new_data, &new_data_len) < 0) {
+    old_data_len = data_len;
+    zstream.zalloc = (alloc_func) 0;
+    zstream.zfree = (free_func) 0;
+    zstream.opaque = (voidpf) 0;
+    if (inflateInit (&zstream) != Z_OK) {
+      xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+              "demux_matroska: VobSub: zlib inflateInit failed.\n");
       return;
     }
+    zstream.next_in = (Bytef *)data;
+    zstream.avail_in = data_len;
 
-    if (!new_data) {
-      if (track->compress_algo == MATROSKA_COMPRESS_UNKNOWN) {
-	track->compress_algo = MATROSKA_COMPRESS_NONE;
-	xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-		"demux_matroska: VobSub: falling back to uncompressed mode.\n");
-      } else {
-	return;
+    dest = (uint8_t *)malloc(data_len);
+    zstream.avail_out = data_len;
+    do {
+      data_len += 4000;
+      dest = (uint8_t *)realloc(dest, data_len);
+      zstream.next_out = (Bytef *)(dest + zstream.total_out);
+      result = inflate (&zstream, Z_NO_FLUSH);
+      if ((result != Z_OK) && (result != Z_STREAM_END)) {
+        xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+                "demux_matroska: VobSub: zlib decompression failed for track %d (result = %d).\n",
+                (int)track->track_num, result);
+        free(dest);
+        inflateEnd(&zstream);
+
+        if (result == Z_DATA_ERROR && track->compress_algo == MATROSKA_COMPRESS_UNKNOWN) {
+          track->compress_algo = MATROSKA_COMPRESS_NONE;
+          data_len = old_data_len;
+          xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+                  "demux_matroska: VobSub: falling back to uncompressed mode.\n");
+          break;
+        }
+        return;
       }
-    } else {
+      zstream.avail_out += 4000;
+    } while ((zstream.avail_out == 4000) &&
+            (zstream.avail_in != 0) && (result != Z_STREAM_END));
+
+    if (track->compress_algo != MATROSKA_COMPRESS_NONE) {
+      data_len = zstream.total_out;
+      inflateEnd(&zstream);
+
+      data = dest;
       track->compress_algo = MATROSKA_COMPRESS_ZLIB;
-      data = new_data;
-      data_len = new_data_len;
+      lprintf("VobSub: decompression for track %d from %d to %d\n",
+              (int)track->track_num, old_data_len, data_len);
     }
   }
   else
   {
-    lprintf("VobSub: track %d isn't compressed (%zu bytes)\n",
+    lprintf("VobSub: track %d isn't compressed (%d bytes)\n",
             (int)track->track_num, data_len);
   }
 
@@ -1217,123 +1088,8 @@ static void handle_vobsub (demux_plugin_t *this_gen, matroska_track_t *track,
     buf->free_buffer(buf);
   }
 
-  free(new_data);
-}
-
-static void handle_hdmv_pgs (demux_plugin_t *this_gen, matroska_track_t *track,
-                             int decoder_flags,
-                             uint8_t *data, size_t data_len,
-                             int64_t data_pts, int data_duration,
-                             int input_normpos, int input_time) {
-  demux_matroska_t *this = (demux_matroska_t *) this_gen;
-  uint8_t *new_data = NULL;
-  size_t new_data_len = 0;
-
-  if (track->compress_algo == MATROSKA_COMPRESS_ZLIB) {
-    uncompress_zlib(this, data, data_len, &new_data, &new_data_len);
-    if (!new_data)
-      return;
-    data = new_data;
-    data_len = new_data_len;
-  }
-
-  _x_demux_send_data(track->fifo, data, data_len,
-		     data_pts, track->buf_type, decoder_flags,
-		     input_normpos, input_time,
-		     0, 0);
-
-  free(new_data);
-}
-
-static void handle_hdmv_textst (demux_plugin_t *this_gen, matroska_track_t *track,
-                                int decoder_flags,
-                                uint8_t *data, size_t data_len,
-                                int64_t data_pts, int data_duration,
-                                int input_normpos, int input_time) {
-  demux_matroska_t *this = (demux_matroska_t *)this_gen;
-
-  buf_element_t *buf;
-  uint32_t *val;
-  char *dest;
-  int dest_len;
-  int dest_pos;
-  int i;
-  uint8_t *ptr;
-  int palette_update_flag;
-  int num_regions;
-
-  ptr = data;
-  if (*ptr != 0x82) {
-    lprintf("Not a dialog presentation segment\n");
-    return;
-  }
-  ++ptr;
-
-  // Skip length and start/end pts values
-  ptr += 12;
-
-  palette_update_flag = *ptr++;
-  if (palette_update_flag) {
-    lprintf("Dunno what to do with palette updates\n");
-    return;
-  }
-
-  buf = track->fifo->buffer_pool_alloc(track->fifo);
-  buf->type = track->buf_type;
-  buf->decoder_flags = decoder_flags | BUF_FLAG_SPECIAL;
-  buf->decoder_info[1] = BUF_SPECIAL_CHARSET_ENCODING;
-  buf->decoder_info_ptr[2] = this->class->literal_utf_8;
-  buf->decoder_info[2] = LITERAL_UTF_8_LEN;
-
-  val = (uint32_t *)buf->content;
-  *val++ = data_pts / 90;                    /* start time */
-  *val++ = (data_pts + data_duration) / 90;  /* end time   */
-
-  dest = buf->content + 8;
-  dest_len = buf->max_size - 8;
-  dest_pos = 0;
-
-  num_regions = *ptr++;
-  for (i = 0; i < num_regions; ++i) {
-    int length;
-    uint8_t *end;
-
-    // skip flags and style id reference
-    ptr += 2;
-
-    length = (ptr[0]<<8) | ptr[1];
-    ptr += 2;
-
-    end = ptr + length;
-    while (ptr < end && dest_pos < dest_len - 2) {
-      int marker;
-      int type;
-      int length;
-
-      marker = *ptr++;
-      if (marker != 0x1B)
-        continue;
-
-      type = *ptr++;
-      length = *ptr++;
-
-      if (type == 0x01) {
-        int k;
-        for (k = 0; k < length && dest_pos < dest_len - 1; ++k)
-          dest[dest_pos++] = ptr[k];
-      } else if (type == 0x0A) {
-        dest[dest_pos++] = '\n';
-      } else {
-        lprintf("Unknown TextST data %02X\n", type);
-      }
-
-      ptr += length;
-    }
-  }
-
-  dest[dest_pos] = 0;
-
-  track->fifo->put(track->fifo, buf);
+  if (track->compress_algo == MATROSKA_COMPRESS_ZLIB)
+    free(data);
 }
 
 static void fill_extra_data(matroska_track_t *track, uint32_t fourcc) {
@@ -1349,8 +1105,7 @@ static void fill_extra_data(matroska_track_t *track, uint32_t fourcc) {
   bih->biCompression = fourcc;
   bih->biWidth = track->video_track->pixel_width;
   bih->biHeight = track->video_track->pixel_height;
-  /* this is to be passed to decoder in native machine endian */
-  /* (no _x_bmiheader_le2me(bih);) */
+  _x_bmiheader_le2me(bih);
 
   /* add bih extra data */
   memcpy(bih + 1, track->codec_private, track->codec_private_len);
@@ -1361,10 +1116,9 @@ static void fill_extra_data(matroska_track_t *track, uint32_t fourcc) {
 
 static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
   ebml_parser_t *ebml = this->ebml;
-  int this_level = ebml->level;
-  int next_level = this_level;
+  int next_level = 3;
 
-  while (next_level == this_level) {
+  while (next_level == 3) {
     ebml_elem_t elem;
 
     if (!ebml_read_elem_head(ebml, &elem))
@@ -1544,16 +1298,49 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
     } else if ((!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_SP)) ||
                (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_ASP)) ||
                (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_AP))) {
+      xine_bmiheader *bih;
 
       lprintf("MATROSKA_CODEC_ID_V_MPEG4_*\n");
-      fill_extra_data(track, ME_FOURCC('M', 'P', '4', 'S'));
+      if (track->codec_private_len > 0x7fffffff - sizeof(xine_bmiheader))
+        track->codec_private_len = 0x7fffffff - sizeof(xine_bmiheader);
+
+      /* create a bitmap info header struct for MPEG 4 */
+      bih = calloc(1, sizeof(xine_bmiheader) + track->codec_private_len);
+      bih->biSize = sizeof(xine_bmiheader) + track->codec_private_len;
+      bih->biCompression = ME_FOURCC('M', 'P', '4', 'S');
+      bih->biWidth = track->video_track->pixel_width;
+      bih->biHeight = track->video_track->pixel_height;
+      _x_bmiheader_le2me(bih);
+
+      /* add bih extra data */
+      memcpy(bih + 1, track->codec_private, track->codec_private_len);
+      free(track->codec_private);
+      track->codec_private = (uint8_t *)bih;
+      track->codec_private_len = bih->biSize;
       track->buf_type = BUF_VIDEO_MPEG4;
 
       /* init as a vfw decoder */
       init_codec = init_codec_video;
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_AVC)) {
+      xine_bmiheader *bih;
+
       lprintf("MATROSKA_CODEC_ID_V_MPEG4_AVC\n");
-      fill_extra_data(track, ME_FOURCC('a', 'v', 'c', '1'));
+      if (track->codec_private_len > 0x7fffffff - sizeof(xine_bmiheader))
+        track->codec_private_len = 0x7fffffff - sizeof(xine_bmiheader);
+
+      /* create a bitmap info header struct for h264 */
+      bih = calloc(1, sizeof(xine_bmiheader) + track->codec_private_len);
+      bih->biSize = sizeof(xine_bmiheader) + track->codec_private_len;
+      bih->biCompression = ME_FOURCC('a', 'v', 'c', '1');
+      bih->biWidth = track->video_track->pixel_width;
+      bih->biHeight = track->video_track->pixel_height;
+      _x_bmiheader_le2me(bih);
+
+      /* add bih extra data */
+      memcpy(bih + 1, track->codec_private, track->codec_private_len);
+      free(track->codec_private);
+      track->codec_private = (uint8_t *)bih;
+      track->codec_private_len = bih->biSize;
       track->buf_type = BUF_VIDEO_H264;
 
       /* init as a vfw decoder */
@@ -1578,7 +1365,6 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       fill_extra_data(track, ME_FOURCC('v', 'p', '9', '0'));
       track->buf_type = BUF_VIDEO_VP9;
       init_codec = init_codec_video;
-      track->handle_content = handle_vp9;
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_HEVC)) {
       lprintf("MATROSKA_CODEC_ID_V_HEVC\n");
       fill_extra_data(track, ME_FOURCC('h', 'e', 'v', 'c'));
@@ -1611,15 +1397,7 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       init_codec = init_codec_audio;
 
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_PCM_INT_BE)) {
-      lprintf("MATROSKA_CODEC_ID_A_PCM_INT_BE");
-      track->buf_type = BUF_AUDIO_LPCM_BE;
-      init_codec = init_codec_audio;
-
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_PCM_INT_LE)) {
-      lprintf("MATROSKA_CODEC_ID_A_PCM_INT_LE");
-      track->buf_type = BUF_AUDIO_LPCM_LE;
-      init_codec = init_codec_audio;
-
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_PCM_FLOAT)) {
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_AC3)) {
       lprintf("MATROSKA_CODEC_ID_A_AC3\n");
@@ -1631,11 +1409,6 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       track->buf_type = BUF_AUDIO_EAC3;
       init_codec = init_codec_audio;
 
-    } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_TRUEHD)) {
-      lprintf("MATROSKA_CODEC_ID_A_TRUEHD\n");
-      track->buf_type = BUF_AUDIO_TRUEHD;
-      init_codec = init_codec_audio;
-
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_FLAC)) {
       lprintf("MATROSKA_CODEC_ID_A_FLAC\n");
       track->buf_type = BUF_AUDIO_FLAC;
@@ -1652,11 +1425,6 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       track->buf_type = BUF_AUDIO_VORBIS;
       init_codec = init_codec_xiph;
 
-    } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_OPUS)) {
-      lprintf("MATROSKA_CODEC_ID_A_OPUS\n");
-      track->buf_type = BUF_AUDIO_OPUS;
-      init_codec = init_codec_audio;
-
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_ACM)) {
       xine_waveformatex *wfh;
       lprintf("MATROSKA_CODEC_ID_A_ACM\n");
@@ -1725,12 +1493,6 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_S_HDMV_PGS)) {
       lprintf("MATROSKA_CODEC_ID_S_HDMV_PGS\n");
       track->buf_type = BUF_SPU_HDMV;
-      track->handle_content = handle_hdmv_pgs;
-      init_codec = init_codec_spu;
-    } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_S_HDMV_TEXTST)) {
-      lprintf("MATROSKA_CODEC_ID_S_HDMV_TEXTST\n");
-      track->buf_type = BUF_SPU_OGM;
-      track->handle_content = handle_hdmv_textst;
       init_codec = init_codec_spu;
     } else {
       xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "unknown codec %s\n", track->codec_id);
@@ -1777,10 +1539,9 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
 
 static int parse_tracks(demux_matroska_t *this) {
   ebml_parser_t *ebml = this->ebml;
-  int this_level = ebml->level;
-  int next_level = this_level;
+  int next_level = 2;
 
-  while (next_level == this_level) {
+  while (next_level == 2) {
     ebml_elem_t elem;
 
     if (!ebml_read_elem_head(ebml, &elem))
@@ -2127,7 +1888,7 @@ static int parse_block (demux_matroska_t *this, size_t block_size,
 
   /*gap = flags & 1;*/
   lacing = (flags >> 1) & 0x3;
-  lprintf("lacing: %x\n", lacing);
+/*fprintf(stderr, "lacing: %x\n", lacing);*/
 
   if (!find_track_by_id(this, (int)track_num, &track)) {
      xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
@@ -2171,11 +1932,8 @@ static int parse_block (demux_matroska_t *this, size_t block_size,
     size_t block_size_left;
     lprintf("no lacing\n");
 
-    if (is_key)
-      decoder_flags |= BUF_FLAG_KEYFRAME;
-
     block_size_left = (this->block_data + block_size + this->compress_maxlen) - data;
-    lprintf("size: %zu, block_size: %zu, block_offset: %zu\n", block_size_left, block_size, this->compress_maxlen);
+    lprintf("size: %d, block_size: %u, block_offset: %u\n", block_size_left, block_size, this->compress_maxlen);
 
     if (headers_len) {
       data -= headers_len;
@@ -2184,13 +1942,17 @@ static int parse_block (demux_matroska_t *this, size_t block_size,
     }
 
     if (track->handle_content != NULL) {
-      track->handle_content ((demux_plugin_t *)this, track, decoder_flags,
-        data, block_size_left, pts, xduration, normpos, pts / 90);
+      track->handle_content((demux_plugin_t *)this, track,
+                             decoder_flags,
+                             data, block_size_left,
+                             pts, xduration,
+                             normpos, pts / 90);
     } else {
-      _x_demux_send_data (track->fifo, data, block_size_left, pts, track->buf_type,
-        decoder_flags, normpos, pts / 90, this->duration, 0);
+      _x_demux_send_data(track->fifo, data, block_size_left,
+                         pts, track->buf_type, decoder_flags,
+                         normpos, pts / 90,
+                         this->duration, 0);
     }
-
   } else {
 
     size_t block_size_left;
@@ -2234,7 +1996,7 @@ static int parse_block (demux_matroska_t *this, size_t block_size,
       break;
 
       case MATROSKA_FIXED_SIZE_LACING: {
-        size_t frame_size;
+        int frame_size;
 
         lprintf("fixed size lacing\n");
 
@@ -2242,7 +2004,7 @@ static int parse_block (demux_matroska_t *this, size_t block_size,
         for (i = 0; i < lace_num; i++) {
           frame[i] = frame_size;
         }
-        frame[lace_num] = block_size_left - ((size_t)lace_num * frame_size);
+        frame[lace_num] = block_size_left - (lace_num * frame_size);
         block_size_left = 0;
       }
       break;
@@ -2262,13 +2024,13 @@ static int parse_block (demux_matroska_t *this, size_t block_size,
         }
         if (first_frame_size > INT_MAX) {
           xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-                  "demux_matroska: invalid first frame size (%" PRIu64 ")\n",
+                  "demux_matroska: invalid first frame size (%" PRId64 ")\n",
                   first_frame_size);
           return 0;
         }
         data += num_len; block_size_left -= num_len;
         frame[0] = (int) first_frame_size;
-        lprintf("first frame len: %zu\n", frame[0]);
+        lprintf("first frame len: %d\n", frame[0]);
         block_size_left -= frame[0];
 
         for (i = 1; i < lace_num; i++) {
@@ -2387,7 +2149,7 @@ static int parse_block_group(demux_matroska_t *this,
         if (!read_block_data(this, elem.len, this->compress_maxlen))
           return 0;
 
-        has_block = 1;
+          has_block = 1;
         break;
       case MATROSKA_ID_CL_BLOCKDURATION:
         /* should override track duration */
@@ -2743,7 +2505,7 @@ static int parse_top_level_head(demux_matroska_t *this, int *next_level) {
           return 0;
         break;
       case MATROSKA_ID_CLUSTER:
-        lprintf("Slipping Cluster\n");
+        lprintf("Cluster\n");
         if (!ebml_skip(ebml, &elem))
           return 0;
         ret_value = 2;
@@ -2800,23 +2562,23 @@ static int parse_top_level(demux_matroska_t *this, int *next_level) {
 
   switch (elem.id) {
     case MATROSKA_ID_SEEKHEAD:
-      lprintf("Skipping SeekHead\n");
+      lprintf("SeekHead\n");
       if (!ebml_skip(ebml, &elem))
         return 0;
       this->has_seekhead = 1;
       break;
     case MATROSKA_ID_INFO:
-      lprintf("Skipping Info\n");
+      lprintf("Info\n");
       if (!ebml_skip(ebml, &elem))
         return 0;
       break;
     case MATROSKA_ID_TRACKS:
-      lprintf("Skipping Tracks\n");
+      lprintf("Tracks\n");
       if (!ebml_skip(ebml, &elem))
         return 0;
       break;
     case MATROSKA_ID_CHAPTERS:
-      lprintf("Skipping Chapters\n");
+      lprintf("Chapters\n");
       if (!ebml_skip(ebml, &elem))
         return 0;
       break;
@@ -2828,17 +2590,17 @@ static int parse_top_level(demux_matroska_t *this, int *next_level) {
         return 0;
       break;
     case MATROSKA_ID_CUES:
-      lprintf("Skipping Cues\n");
+      lprintf("Cues\n");
       if (!ebml_skip(ebml, &elem))
         return 0;
       break;
     case MATROSKA_ID_ATTACHMENTS:
-      lprintf("Skipping Attachments\n");
+      lprintf("Attachments\n");
       if (!ebml_skip(ebml, &elem))
         return 0;
       break;
     case MATROSKA_ID_TAGS:
-      lprintf("Skipping Tags\n");
+      lprintf("Tags\n");
       if (!ebml_skip(ebml, &elem))
         return 0;
       break;
@@ -2850,8 +2612,6 @@ static int parse_top_level(demux_matroska_t *this, int *next_level) {
   }
   if (next_level)
     *next_level = ebml_get_next_level(ebml, &elem);
-
-  lprintf("parse_top_level() done\n");
   return 1;
 }
 
@@ -2931,12 +2691,7 @@ static void demux_matroska_send_headers (demux_plugin_t *this_gen) {
    */
 
   /* enter in the segment */
-  if (!ebml_read_master (this->ebml, &this->segment)) {
-    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
-            "demux_matroska: failed to read file header\n");
-    this->status = DEMUX_FINISHED;
-    return;
-  }
+  ebml_read_master (this->ebml, &this->segment);
 
   /* seek back to the beginning of the segment */
   next_level = 1;
@@ -3236,8 +2991,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
       return NULL;
     input->seek(input, 0, SEEK_SET);
     ebml = new_ebml_parser(stream->xine, input);
-    if (!ebml)
-      goto error;
     if (!ebml_check_header(ebml))
       goto error;
   }
@@ -3252,10 +3005,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   }
 
   this = calloc(1, sizeof(demux_matroska_t));
-  if (!this)
-    goto error;
-
-  this->class = (demux_matroska_class_t *)class_gen;
 
   this->demux_plugin.send_headers      = demux_matroska_send_headers;
   this->demux_plugin.send_chunk        = demux_matroska_send_chunk;
@@ -3273,8 +3022,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   if (!ebml) {
     ebml = new_ebml_parser(stream->xine, input);
-    if (!ebml)
-      goto error;
     if (!ebml_check_header(ebml))
       goto error;
   }
@@ -3298,8 +3045,8 @@ error:
 
   if (this != NULL && this->event_queue != NULL) {
     xine_event_dispose_queue(this->event_queue);
+    free(this);
   }
-  free(this);
 
   return NULL;
 }
@@ -3308,14 +3055,11 @@ error:
 /*
  * demux matroska class
  */
-void *demux_matroska_init_class (xine_t *xine, void *data) {
+static void *init_class (xine_t *xine, void *data) {
 
   demux_matroska_class_t     *this;
 
   this         = calloc(1, sizeof(demux_matroska_class_t));
-  if (!this)
-    return NULL;
-
   this->xine   = xine;
 
   this->demux_class.open_plugin     = open_plugin;
@@ -3328,8 +3072,18 @@ void *demux_matroska_init_class (xine_t *xine, void *data) {
   this->demux_class.extensions      = "mkv wbm webm";
   this->demux_class.dispose         = default_demux_class_dispose;
 
-  memcpy (this->literal_utf_8, "utf-8", LITERAL_UTF_8_LEN + 1);
-
   return this;
 }
 
+/*
+ * exported plugin catalog entry
+ */
+static const demuxer_info_t demux_info_matroska = {
+  10                       /* priority */
+};
+
+const plugin_info_t xine_plugin_info[] EXPORTED = {
+  /* type, API, "name", version, special_info, init_function */
+  { PLUGIN_DEMUX, 27, "matroska", XINE_VERSION_CODE, &demux_info_matroska, init_class },
+  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+};
