@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2018 the xine project
+ * Copyright (C) 2000-2013 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -283,7 +283,6 @@ struct matroska_track_s {
   matroska_sub_track_t    *sub_track;
 
   int64_t                  last_pts;
-  int64_t                  delayed_pts;
 
   void                   (*handle_content) (demux_plugin_t *this_gen,
                                             matroska_track_t *track,
@@ -318,7 +317,6 @@ struct matroska_track_s {
 #define MATROSKA_CODEC_ID_V_VP8          "V_VP8"
 #define MATROSKA_CODEC_ID_V_VP9          "V_VP9"
 #define MATROSKA_CODEC_ID_V_HEVC         "V_MPEGH/ISO/HEVC"
-#define MATROSKA_CODEC_ID_V_AV1          "V_AV1"
 
 #define MATROSKA_CODEC_ID_A_MPEG1_L1     "A_MPEG/L1"
 #define MATROSKA_CODEC_ID_A_MPEG1_L2     "A_MPEG/L2"
@@ -339,8 +337,6 @@ struct matroska_track_s {
 #define MATROSKA_CODEC_ID_A_REAL_RALF    "A_REAL/RALF"
 #define MATROSKA_CODEC_ID_A_REAL_ATRC    "A_REAL/ATRC"
 #define MATROSKA_CODEC_ID_A_FLAC         "A_FLAC"
-#define MATROSKA_CODEC_ID_A_OPUS         "A_OPUS"
-#define MATROSKA_CODEC_ID_A_TRUEHD       "A_TRUEHD"
 
 #define MATROSKA_CODEC_ID_S_TEXT_UTF8    "S_TEXT/UTF8"
 #define MATROSKA_CODEC_ID_S_TEXT_SSA     "S_TEXT/SSA"
@@ -349,10 +345,8 @@ struct matroska_track_s {
 #define MATROSKA_CODEC_ID_S_UTF8         "S_UTF8"        /* deprecated */
 #define MATROSKA_CODEC_ID_S_SSA          "S_SSA"         /* deprecated */
 #define MATROSKA_CODEC_ID_S_ASS          "S_ASS"         /* deprecated */
-#define MATROSKA_CODEC_ID_S_DVBSUB       "S_DVBSUB"
 #define MATROSKA_CODEC_ID_S_VOBSUB       "S_VOBSUB"
 #define MATROSKA_CODEC_ID_S_HDMV_PGS     "S_HDMV/PGS"
-#define MATROSKA_CODEC_ID_S_HDMV_TEXTST  "S_HDMV/TEXTST"
 
 /* block lacing */
 #define MATROSKA_NO_LACING               0x0
