@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2018 the xine project
+ * Copyright (C) 2000-2011 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -40,7 +40,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <pthread.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <math.h>
@@ -66,7 +65,6 @@
 #include <xine/video_out.h>
 #include "dxr3.h"
 #include "video_out_dxr3.h"
-#include "group_dxr3.h"
 
 #ifdef HAVE_FFMPEG_AVUTIL_H
 #  include <mem.h>
@@ -74,11 +72,44 @@
 #  include <libavutil/mem.h>
 #endif
 
+#include "compat.c"
+
 /* the amount of extra time we give the card for decoding */
 #define DECODE_PIPE_PREBUFFER 10000
 
+
+/* plugin class initialization functions */
+static void                *dxr3_x11_init_plugin(xine_t *xine, void *visual_gen);
+static void                *dxr3_aa_init_plugin(xine_t *xine, void *visual_gen);
+static dxr3_driver_class_t *dxr3_vo_init_plugin(xine_t *xine, void *visual_gen);
+
+
+/* plugin catalog information */
+#ifdef HAVE_X11
+static const vo_info_t   vo_info_dxr3_x11 = {
+  10,                  /* priority        */
+  XINE_VISUAL_TYPE_X11 /* visual type     */
+};
+#endif
+
+static const vo_info_t   vo_info_dxr3_aa = {
+  10,                  /* priority        */
+  XINE_VISUAL_TYPE_AA  /* visual type     */
+};
+
+const plugin_info_t      xine_plugin_info[] EXPORTED = {
+  /* type, API, "name", version, special_info, init_function */
+#ifdef HAVE_X11
+  { PLUGIN_VIDEO_OUT, 22, "dxr3", XINE_VERSION_CODE, &vo_info_dxr3_x11, &dxr3_x11_init_plugin },
+#endif
+  { PLUGIN_VIDEO_OUT, 22, "aadxr3", XINE_VERSION_CODE, &vo_info_dxr3_aa, &dxr3_aa_init_plugin },
+  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+};
+
+
 /* plugin class functions */
 static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const void *visual);
+static void         dxr3_vo_class_dispose(video_driver_class_t *class_gen);
 
 /* plugin instance functions */
 static uint32_t    dxr3_get_capabilities(vo_driver_t *this_gen);
@@ -120,41 +151,8 @@ static void        dxr3_update_swap_fields(void *data, xine_cfg_entry_t *entry);
 static void        dxr3_update_enhanced_mode(void *this_gen, xine_cfg_entry_t *entry);
 
 
-int dxr3_present(xine_stream_t *stream)
-{
-  int present = 0;
-
-  if (stream->video_driver)
-    present = stream->video_driver->dispose == dxr3_dispose;
-  llprintf(LOG_VID, "dxr3 %s\n", present ? "present" : "not present");
-  return present;
-}
-
-static dxr3_driver_class_t *dxr3_vo_init_plugin(xine_t *xine, const void *visual_gen)
-{
-  dxr3_driver_class_t *this;
-
-  (void)visual_gen;
-  this = calloc(1, sizeof(dxr3_driver_class_t));
-  if (!this) return NULL;
-
-  this->devnum = xine->config->register_num(xine->config,
-    CONF_KEY, 0, CONF_NAME, CONF_HELP, 10, NULL, NULL);
-
-  this->video_driver_class.open_plugin     = dxr3_vo_open_plugin;
-  this->video_driver_class.identifier      = DXR3_VO_ID;
-  this->video_driver_class.description     = N_("video output plugin displaying images through your DXR3 decoder card");
-  this->video_driver_class.dispose         = default_video_driver_class_dispose;
-
-  this->xine                               = xine;
-
-  this->instance                           = 0;
-
-  return this;
-}
-
 #ifdef HAVE_X11
-void *dxr3_x11_init_plugin(xine_t *xine, const void *visual_gen)
+static void *dxr3_x11_init_plugin(xine_t *xine, void *visual_gen)
 {
   dxr3_driver_class_t *this = dxr3_vo_init_plugin(xine, visual_gen);
 
@@ -164,7 +162,7 @@ void *dxr3_x11_init_plugin(xine_t *xine, const void *visual_gen)
 }
 #endif
 
-void *dxr3_aa_init_plugin(xine_t *xine, const void *visual_gen)
+static void *dxr3_aa_init_plugin(xine_t *xine, void *visual_gen)
 {
   dxr3_driver_class_t *this = dxr3_vo_init_plugin(xine, visual_gen);
 
@@ -173,6 +171,40 @@ void *dxr3_aa_init_plugin(xine_t *xine, const void *visual_gen)
   return &this->video_driver_class;
 }
 
+static dxr3_driver_class_t *dxr3_vo_init_plugin(xine_t *xine, void *visual_gen)
+{
+  dxr3_driver_class_t *this;
+
+  this = calloc(1, sizeof(dxr3_driver_class_t));
+  if (!this) return NULL;
+
+  this->devnum = xine->config->register_num(xine->config,
+    CONF_KEY, 0, CONF_NAME, CONF_HELP, 10, NULL, NULL);
+
+  this->video_driver_class.open_plugin     = dxr3_vo_open_plugin;
+  this->video_driver_class.identifier      = DXR3_VO_ID;
+  this->video_driver_class.description     = N_("video output plugin displaying images through your DXR3 decoder card");
+  this->video_driver_class.dispose         = dxr3_vo_class_dispose;
+
+  this->xine                               = xine;
+
+  this->instance                           = 0;
+
+  this->scr                                = dxr3_scr_init(xine);
+
+  return this;
+}
+
+static void dxr3_vo_class_dispose(video_driver_class_t *class_gen)
+{
+  dxr3_driver_class_t *class = (dxr3_driver_class_t *)class_gen;
+
+  if(class->scr)
+    class->scr->scr_plugin.exit(&class->scr->scr_plugin);
+  free(class_gen);
+}
+
+
 static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const void *visual_gen)
 {
   dxr3_driver_t *this;
@@ -181,9 +213,8 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
   char tmpstr[100];
   const char *confstr;
   int encoder, confnum;
-  static const char *available_encoders[SUPPORTED_ENCODER_COUNT + 2];
-  plugin_node_t *node = NULL; /* unused inside dxr3_lavc_init () anyway... */
-  int fd_control;
+  static char *available_encoders[SUPPORTED_ENCODER_COUNT + 2];
+  plugin_node_t *node;
 
   static const char *const videoout_modes[] = {
     "letterboxed tv", "widescreen tv",
@@ -193,26 +224,13 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
     NULL
   };
   static const char *const tv_modes[] = { "ntsc", "pal", "pal60" , "default", NULL };
-  /*
   int list_id, list_size;
   xine_sarray_t *plugin_list;
-  */
+
   if (class->instance) return NULL;
 
-  snprintf(tmpstr, sizeof(tmpstr), "/dev/em8300-%d", class->devnum);
-  llprintf(LOG_VID, "Entering video init, devname = %s.\n", tmpstr);
-  if ((fd_control = xine_open_cloexec(tmpstr, O_WRONLY)) < 0) {
-    xprintf(class->xine, XINE_VERBOSITY_LOG,
-            _("video_out_dxr3: Failed to open control device %s (%s)\n"), tmpstr, strerror(errno));
-    return NULL;
-  }
-
-
   this = calloc(1, sizeof(dxr3_driver_t));
-  if (!this) {
-    close(fd_control);
-    return NULL;
-  }
+  if (!this) return NULL;
 
   this->vo_driver.get_capabilities     = dxr3_get_capabilities;
   this->vo_driver.alloc_frame          = dxr3_alloc_frame;
@@ -228,17 +246,12 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
   this->vo_driver.gui_data_exchange    = dxr3_gui_data_exchange;
   this->vo_driver.dispose              = dxr3_dispose;
 
-  this->scr                            = dxr3_scr_init(class->xine);
-
   pthread_mutex_init(&this->video_device_lock, NULL);
   pthread_mutex_init(&this->spu_device_lock, NULL);
 
   _x_vo_scale_init(&this->scale, 0, 0, config);
   _x_alphablend_init(&this->alphablend_extra_data, class->xine);
 
-  this->fd_control = fd_control;
-  this->fd_spu = -1;
-
   this->class                          = class;
   this->swap_fields                    = config->register_bool(config,
     "dxr3.encoding.swap_fields", 0, _("swap odd and even lines"),
@@ -256,12 +269,21 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
     _("Enabling this option will utilise a smoother play mode for non-MPEG content."),
     20, dxr3_update_enhanced_mode, this);
 
+  snprintf(tmpstr, sizeof(tmpstr), "/dev/em8300-%d", class->devnum);
+  llprintf(LOG_VID, "Entering video init, devname = %s.\n", tmpstr);
+  if ((this->fd_control = xine_open_cloexec(tmpstr, O_WRONLY)) < 0) {
+
+    xprintf(this->class->xine, XINE_VERBOSITY_LOG,
+	    _("video_out_dxr3: Failed to open control device %s (%s)\n"), tmpstr, strerror(errno));
+
+    return 0;
+  }
+
   snprintf (tmpstr, sizeof(tmpstr), "/dev/em8300_mv-%d", class->devnum);
   if ((this->fd_video = xine_open_cloexec(tmpstr, O_WRONLY | O_SYNC )) < 0) {
     xprintf(this->class->xine, XINE_VERBOSITY_LOG,
 	    _("video_out_dxr3: Failed to open video device %s (%s)\n"), tmpstr, strerror(errno));
-    dxr3_dispose(&this->vo_driver);
-    return NULL;
+    return 0;
   }
   /* close now and and let the decoder/encoder reopen if they want */
   close(this->fd_video);
@@ -293,7 +315,7 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
 #endif
   if (encoder) {
     encoder = config->register_enum(config, "dxr3.encoding.encoder", 0,
-      (char **) available_encoders, _("encoder for non mpeg content"),
+      available_encoders, _("encoder for non mpeg content"),
       _("Content other than MPEG has to pass an additional reencoding stage, "
 	"because the dxr3 handles only MPEG.\nDepending on what is supported by your xine, "
 	"this setting can be \"fame\", \"rte\", \"libavcodec\" or \"none\".\n"
@@ -306,23 +328,20 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
     if ((strcmp(available_encoders[encoder], "libavcodec") == 0) && !dxr3_lavc_init(this, node)) {
       xprintf(this->class->xine, XINE_VERBOSITY_LOG,
 	      _("video_out_dxr3: Mpeg encoder libavcodec failed to init.\n"));
-      dxr3_dispose(&this->vo_driver);
-      return NULL;
+      return 0;
     }
 #ifdef HAVE_LIBRTE
     if ((strcmp(available_encoders[encoder], "rte") == 0) && !dxr3_rte_init(this)) {
       xprintf(this->class->xine, XINE_VERBOSITY_LOG,
 	      _("video_out_dxr3: Mpeg encoder rte failed to init.\n"));
-      dxr3_dispose(&this->vo_driver);
-      return NULL;
+      return 0;
     }
 #endif
 #ifdef HAVE_LIBFAME
     if ((strcmp(available_encoders[encoder], "fame") == 0) && !dxr3_fame_init(this)) {
       xprintf(this->class->xine, XINE_VERBOSITY_LOG,
 	      _("video_out_dxr3: Mpeg encoder fame failed to init.\n"));
-      dxr3_dispose(&this->vo_driver);
-      return NULL;
+      return 0;
     }
 #endif
     if (strcmp(available_encoders[encoder], "none") == 0)
@@ -493,7 +512,6 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
 
 static uint32_t dxr3_get_capabilities(vo_driver_t *this_gen)
 {
-  (void)this_gen;
   return VO_CAP_YV12 | VO_CAP_YUY2;
 }
 
@@ -547,8 +565,6 @@ static void dxr3_frame_proc_slice(vo_frame_t *frame_gen, uint8_t **src)
 static void dxr3_frame_field(vo_frame_t *vo_img, int which_field)
 {
   /* dummy function */
-  (void)vo_img;
-  (void)which_field;
 }
 
 static void dxr3_frame_dispose(vo_frame_t *frame_gen)
@@ -574,7 +590,7 @@ static void dxr3_update_frame_format(vo_driver_t *this_gen, vo_frame_t *frame_ge
     if (this->fd_video >= 0) {
       metronom_clock_t *clock = this->class->xine->clock;
 
-      clock->unregister_scr(clock, &this->scr->scr_plugin);
+      clock->unregister_scr(clock, &this->class->scr->scr_plugin);
       close(this->fd_video);
       this->fd_video = -1;
       /* inform the encoder on next frame's arrival */
@@ -627,8 +643,8 @@ static void dxr3_update_frame_format(vo_driver_t *this_gen, vo_frame_t *frame_ge
 
     /* start the scr plugin */
     time = clock->get_current_time(clock);
-    this->scr->scr_plugin.start(&this->scr->scr_plugin, time);
-    clock->register_scr(clock, &this->scr->scr_plugin);
+    this->class->scr->scr_plugin.start(&this->class->scr->scr_plugin, time);
+    clock->register_scr(clock, &this->class->scr->scr_plugin);
 
     this->scale.force_redraw = 1;
   }
@@ -680,7 +696,7 @@ static void dxr3_update_frame_format(vo_driver_t *this_gen, vo_frame_t *frame_ge
   }
 
   /* if dimensions changed, we need to re-allocate frame memory */
-  if ((frame->vo_frame.width != (int)width) || (frame->vo_frame.height != (int)height) ||
+  if ((frame->vo_frame.width != width) || (frame->vo_frame.height != height) ||
       (frame->oheight != oheight) || (frame->vo_frame.format != format)) {
     av_freep(&frame->mem);
 
@@ -814,7 +830,7 @@ static void dxr3_overlay_end(vo_driver_t *this_gen, vo_frame_t *frame_gen)
   pthread_mutex_lock(&this->spu_device_lock);
 
   /* try to open the dxr3 spu device */
-  if (this->fd_spu < 0) {
+  if (!this->fd_spu) {
     snprintf (tmpstr, sizeof(tmpstr), "/dev/em8300_sp-%d", this->class->devnum);
     if ((this->fd_spu = xine_open_cloexec(tmpstr, O_WRONLY)) < 0) {
       xprintf(this->class->xine, XINE_VERBOSITY_DEBUG,
@@ -834,10 +850,7 @@ static void dxr3_overlay_end(vo_driver_t *this_gen, vo_frame_t *frame_gen)
       0x00, 0x01, 0x00, 0x20, 0x02, 0xFF };
     /* just clear any previous spu */
     dxr3_spu_button(this->fd_spu, NULL);
-    if (write(this->fd_spu, empty_spu, sizeof(empty_spu)) != sizeof(empty_spu)) {
-      xprintf(this->class->xine, XINE_VERBOSITY_DEBUG,
-              "video_out_dxr3: spu device write failed (%s)\n", strerror(errno));
-    }
+    write(this->fd_spu, empty_spu, sizeof(empty_spu));
     pthread_mutex_unlock(&this->spu_device_lock);
     return;
   }
@@ -915,7 +928,7 @@ static void dxr3_display_frame(vo_driver_t *this_gen, vo_frame_t *frame_gen)
   if (this->overlay_enabled) {
     if (this->scale.force_redraw                         ||
 	this->scale.delivered_width  != frame_gen->width ||
-	this->scale.delivered_height != (int)frame->oheight ||
+	this->scale.delivered_height != frame->oheight   ||
 	this->scale.delivered_ratio  != frame_gen->ratio ||
 	this->scale.user_ratio       != (this->widescreen_enabled ? frame->aspect : XINE_VO_ASPECT_4_3)) {
 
@@ -952,9 +965,9 @@ static void dxr3_display_frame(vo_driver_t *this_gen, vo_frame_t *frame_gen)
 	    reg.reg = 0;
 	    reg.val = MVCOMMAND_SYNC;
 	    ioctl(this->fd_control, EM8300_IOCTL_WRITEREG, &reg);
-            pthread_mutex_lock(&this->scr->mutex);
-            this->scr->sync = 1;
-            pthread_mutex_unlock(&this->scr->mutex);
+	    pthread_mutex_lock(&this->class->scr->mutex);
+	    this->class->scr->sync = 1;
+	    pthread_mutex_unlock(&this->class->scr->mutex);
 	  }
 	}
 	this->need_update = 0;
@@ -1123,7 +1136,6 @@ static int dxr3_set_property(vo_driver_t *this_gen, int property, int value)
 static void dxr3_get_property_min_max(vo_driver_t *this_gen, int property,
   int *min, int *max)
 {
-  (void)this_gen;
   switch (property) {
   case VO_PROP_SATURATION:
   case VO_PROP_CONTRAST:
@@ -1207,10 +1219,9 @@ static void dxr3_dispose(vo_driver_t *this_gen)
     this->enc->on_close(this);
   if(this->overlay_enabled)
     ioctl(this->fd_control, EM8300_IOCTL_OVERLAY_SETMODE, &val);
-  if (this->fd_control >= 0)
-    close(this->fd_control);
+  close(this->fd_control);
   pthread_mutex_lock(&this->spu_device_lock);
-  if (this->fd_spu >= 0) {
+  if (this->fd_spu) {
     static const uint8_t empty_spu[] = {
       0x00, 0x26, 0x00, 0x08, 0x80, 0x00, 0x00, 0x80,
       0x00, 0x00, 0x00, 0x20, 0x01, 0x03, 0x00, 0x00,
@@ -1219,10 +1230,7 @@ static void dxr3_dispose(vo_driver_t *this_gen)
       0x00, 0x01, 0x00, 0x20, 0x02, 0xFF };
     /* clear any remaining spu */
     dxr3_spu_button(this->fd_spu, NULL);
-    if (write(this->fd_spu, empty_spu, sizeof(empty_spu)) != sizeof(empty_spu)) {
-      xprintf(this->class->xine, XINE_VERBOSITY_DEBUG,
-              "video_out_dxr3: spu device write failed (%s)\n", strerror(errno));
-    }
+    write(this->fd_spu, empty_spu, sizeof(empty_spu));
     close(this->fd_spu);
   }
   pthread_mutex_unlock(&this->spu_device_lock);
@@ -1231,9 +1239,6 @@ static void dxr3_dispose(vo_driver_t *this_gen)
 
   _x_alphablend_free(&this->alphablend_extra_data);
 
-  if (this->scr)
-    this->scr->scr_plugin.exit(&this->scr->scr_plugin);
-
   free(this);
 }
 
@@ -1281,7 +1286,7 @@ static void gather_screen_vars(dxr3_driver_t *this, const x11_visual_t *vis)
 #define TYPE_FLOAT 4
 
 struct lut_entry {
-    const char *name;
+    char *name;
     int type;
     void *ptr;
 };
