@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2003 the xine project
+ * Copyright (C) 2002-2009 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -24,7 +24,7 @@
 #define HAVE_MMS_H
 
 #include <inttypes.h>
-#include "xine_internal.h"
+#include <xine/xine_internal.h>
 
 typedef struct mms_s mms_t;
 
