@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2019 the xine project
+ * Copyright (C) 2000-2010 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -60,7 +60,6 @@
 #include <xine/buffer.h>
 #include <xine/xineutils.h>
 
-#include "group_w32.h"
 #include "common.c"
 
 static GUID CLSID_Voxware =
@@ -176,6 +175,10 @@ typedef struct w32v_decoder_s {
   ldt_fs_t *ldt_fs;
 } w32v_decoder_t;
 
+typedef struct {
+  video_decoder_class_t   decoder_class;
+} w32v_class_t;
+
 typedef struct w32a_decoder_s {
   audio_decoder_t   audio_decoder;
 
@@ -209,6 +212,11 @@ typedef struct w32a_decoder_s {
   ldt_fs_t *ldt_fs;
 } w32a_decoder_t;
 
+typedef struct {
+  audio_decoder_class_t   decoder_class;
+} w32a_class_t;
+
+
 /*
  * RGB->YUY2 conversion, we need is for xine video-codec ->
  * video-output interface
@@ -301,7 +309,6 @@ static void w32v_init_rgb_ycc(void)
 
 static int get_vids_codec_n_name(w32v_decoder_t *this, int buf_type)
 {
-  (void)this;
   buf_type &= 0xffff0000;
 
   switch (buf_type) {
@@ -561,8 +568,6 @@ static void w32v_init_codec (w32v_decoder_t *this, int buf_type) {
 
   lprintf ("init codec...\n");
 
-  (void)buf_type;
-
   memset(&this->o_bih, 0, sizeof(BITMAPINFOHEADER));
   this->o_bih.biSize = sizeof(BITMAPINFOHEADER);
 
@@ -830,10 +835,11 @@ static void w32v_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
     if( (int) buf->size <= 0 )
         return;
 
-    if (this->stream_id < 0)
-      this->stream_id = buf->type & 0xff;
-    else if ((unsigned int)this->stream_id != (buf->type & 0xff))
-      return;
+    if( this->stream_id < 0 )
+       this->stream_id = buf->type & 0xff;
+
+    if( this->stream_id != (buf->type & 0xff) )
+       return;
 
     if( this->size + buf->size > this->bufsize ) {
       this->bufsize = this->size + 2 * buf->size;
@@ -989,7 +995,6 @@ static void w32v_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 }
 
 static void w32v_flush (video_decoder_t *this_gen) {
-  (void)this_gen;
 }
 
 static void w32v_reset (video_decoder_t *this_gen) {
@@ -1015,7 +1020,6 @@ static void w32v_reset (video_decoder_t *this_gen) {
 }
 
 static void w32v_discontinuity (video_decoder_t *this_gen) {
-  (void)this_gen;
 }
 
 
@@ -1168,12 +1172,6 @@ static int w32a_init_audio (w32a_decoder_t *this,
   audio_buffer_t *audio_buffer;
   int audio_buffer_mem_size;
 
-  if (bufsize < (int)sizeof (WAVEFORMATEX)) {
-    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
-      "w32codec: invalid codec init info.\n");
-    return 0;
-  }
-
   in_fmt = (WAVEFORMATEX *)buf;
   in_size=in_fmt->nBlockAlign;
 
@@ -1231,7 +1229,7 @@ static int w32a_init_audio (w32a_decoder_t *this,
 
     acmStreamSize(this->srcstream, in_size, &out_size, ACM_STREAMSIZEF_SOURCE);
     out_size*=2;
-    if((int)out_size < audio_buffer_mem_size)
+    if(out_size < audio_buffer_mem_size)
       out_size=audio_buffer_mem_size;
 
     xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
@@ -1328,8 +1326,6 @@ static void w32a_decode_audio (w32a_decoder_t *this,
   int delay;
   /* DWORD srcsize=0; */
 
-  (void)frame_end;
-
   /* This code needs more testing */
   /* bitrate computing (byterate: byte per pts) */
   if (pts && (pts != this->lastpts)) {
@@ -1447,7 +1443,7 @@ static void w32a_decode_audio (w32a_decoder_t *this,
 	p += bufsize;
       }
     }
-    if(ash.cbSrcLengthUsed>=(unsigned int)this->size){
+    if(ash.cbSrcLengthUsed>=this->size){
       this->size=0;
     } else {
       this->size-=ash.cbSrcLengthUsed;
@@ -1558,11 +1554,7 @@ static video_decoder_t *open_video_decoder_plugin (video_decoder_class_t *class_
 
   w32v_decoder_t *this ;
 
-  (void)class_gen;
-
   this = (w32v_decoder_t *) calloc(1, sizeof(w32v_decoder_t));
-  if (!this)
-    return NULL;
 
   this->video_decoder.decode_data         = w32v_decode_data;
   this->video_decoder.flush               = w32v_flush;
@@ -1585,20 +1577,24 @@ static void init_routine(void) {
   w32v_init_rgb_ycc();
 }
 
-void *w32v_init_class (xine_t *xine, const void *data) {
+static void *init_video_decoder_class (xine_t *xine, void *data) {
+
+  w32v_class_t   *this;
   config_values_t *cfg;
-  static const video_decoder_class_t this = {
-    .open_plugin     = open_video_decoder_plugin,
-    .identifier      = "w32v",
-    .description     = N_("win32 binary video codec plugin"),
-    .dispose         = NULL
-  };
-  (void)data;
+
   cfg = xine->config;
-  if ((win32_def_path = get_win32_codecs_path(cfg)) == NULL)
-    return NULL;
+  if ((win32_def_path = get_win32_codecs_path(cfg)) == NULL) return NULL;
+
+  this = (w32v_class_t *) calloc(1, sizeof(w32v_class_t));
+
+  this->decoder_class.open_plugin     = open_video_decoder_plugin;
+  this->decoder_class.identifier      = "w32v";
+  this->decoder_class.description     = N_("win32 binary video codec plugin");
+  this->decoder_class.dispose         = default_video_decoder_class_dispose;
+
   pthread_once (&once_control, init_routine);
-  return (video_decoder_class_t *)&this;
+
+  return this;
 }
 
 /********************************************************
@@ -1610,11 +1606,7 @@ static audio_decoder_t *open_audio_decoder_plugin (audio_decoder_class_t *class_
 
   w32a_decoder_t *this ;
 
-  (void)class_gen;
-
   this = (w32a_decoder_t *) calloc(1, sizeof(w32a_decoder_t));
-  if (!this)
-    return NULL;
 
   this->audio_decoder.decode_data         = w32a_decode_data;
   this->audio_decoder.reset               = w32a_reset;
@@ -1634,19 +1626,63 @@ static audio_decoder_t *open_audio_decoder_plugin (audio_decoder_class_t *class_
 /*
  * audio decoder plugin class
  */
-void *w32a_init_class (xine_t *xine, const void *data) {
+static void *init_audio_decoder_class (xine_t *xine, void *data) {
+
+  w32a_class_t    *this;
   config_values_t *cfg;
-  static const audio_decoder_class_t this = {
-    .open_plugin     = open_audio_decoder_plugin,
-    .identifier      = "win32 audio",
-    .description     = N_("win32 binary audio codec plugin"),
-    .dispose         = NULL
-  };
-  (void)data;
+
   cfg = xine->config;
-  if ((win32_def_path = get_win32_codecs_path(cfg)) == NULL)
-    return NULL;
+  if ((win32_def_path = get_win32_codecs_path(cfg)) == NULL) return NULL;
+
+  this = (w32a_class_t *) calloc(1, sizeof(w32a_class_t));
+
+  this->decoder_class.open_plugin     = open_audio_decoder_plugin;
+  this->decoder_class.identifier      = "win32 audio";
+  this->decoder_class.description     = N_("win32 binary audio codec plugin");
+  this->decoder_class.dispose         = default_audio_decoder_class_dispose;
+
   pthread_once (&once_control, init_routine);
-  return (audio_decoder_class_t *)&this;
+
+  return this;
 }
 
+
+/*
+ * exported plugin catalog entry
+ */
+
+static const uint32_t video_types[] = {
+  BUF_VIDEO_MSMPEG4_V1, BUF_VIDEO_MSMPEG4_V2, BUF_VIDEO_MSMPEG4_V3,
+  BUF_VIDEO_IV50, BUF_VIDEO_IV41, BUF_VIDEO_IV32, BUF_VIDEO_IV31,
+  BUF_VIDEO_CINEPAK, /* BUF_VIDEO_ATIVCR1, */
+  BUF_VIDEO_ATIVCR2, BUF_VIDEO_I263, BUF_VIDEO_MSVC,
+  BUF_VIDEO_DV, BUF_VIDEO_WMV7, BUF_VIDEO_WMV8, BUF_VIDEO_WMV9,
+  BUF_VIDEO_VP31, BUF_VIDEO_MSS1, BUF_VIDEO_TSCC, BUF_VIDEO_UCOD,
+  BUF_VIDEO_VP4, BUF_VIDEO_VP5, BUF_VIDEO_VP6,
+  0
+ };
+
+static const decoder_info_t dec_info_video = {
+  video_types,         /* supported types */
+  1                    /* priority        */
+};
+
+static const uint32_t audio_types[] = {
+  BUF_AUDIO_WMAV1, BUF_AUDIO_WMAV2, BUF_AUDIO_WMAPRO, BUF_AUDIO_MSADPCM,
+  BUF_AUDIO_MSIMAADPCM, BUF_AUDIO_MSGSM, BUF_AUDIO_IMC, BUF_AUDIO_LH,
+  BUF_AUDIO_VOXWARE, BUF_AUDIO_ACELPNET, BUF_AUDIO_VIVOG723, BUF_AUDIO_WMAV,
+  BUF_AUDIO_WMALL,
+  0
+ };
+
+static const decoder_info_t dec_info_audio = {
+  audio_types,         /* supported types */
+  1                    /* priority        */
+};
+
+const plugin_info_t xine_plugin_info[] EXPORTED = {
+  /* type, API, "name", version, special_info, init_function */
+  { PLUGIN_VIDEO_DECODER | PLUGIN_MUST_PRELOAD, 19, "win32v", XINE_VERSION_CODE, &dec_info_video, init_video_decoder_class },
+  { PLUGIN_AUDIO_DECODER | PLUGIN_MUST_PRELOAD, 16, "win32a", XINE_VERSION_CODE, &dec_info_audio, init_audio_decoder_class },
+  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+};
