@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2012 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -28,7 +28,6 @@
 #endif
 
 #include <stdio.h>
-#include <pthread.h>
 
 #define LOG_MODULE "upmix_mono"
 #define LOG_VERBOSE
@@ -337,7 +336,7 @@ static post_plugin_t *upmix_mono_open_plugin(post_class_t *class_gen, int inputs
 /* plugin class initialization function */
 void *upmix_mono_init_plugin(xine_t *xine, void *data)
 {
-  post_class_upmix_mono_t *class = calloc(1, sizeof(post_class_upmix_mono_t));
+  post_class_upmix_mono_t *class = (post_class_upmix_mono_t *)xine_xmalloc(sizeof(post_class_upmix_mono_t));
 
   if (!class)
     return NULL;
