@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2019 the xine project
+ * Copyright (C) 2010-2013 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -25,11 +25,6 @@
  * - Implement support for teletext based subtitles
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <time.h>
 #include <pthread.h>
 #include <errno.h>
 
@@ -40,155 +35,11 @@
 #include <xine/spu.h>
 #include <xine/osd.h>
 
-#define MAX_REGIONS 16
+#define MAX_REGIONS 7
 
 #define SPU_MAX_WIDTH 1920
 #define SPU_MAX_HEIGHT 1080
 
-/* sparse_array - handle large arrays efficiently when only a few entries are used */
-
-typedef struct {
-  uint32_t key, value;
-} sparse_array_entry_t;
-
-typedef struct {
-  uint32_t sorted_entries, used_entries, max_entries;
-  sparse_array_entry_t *entries;
-} sparse_array_t;
-
-static void sparse_array_new (sparse_array_t *sa) {
-  sa->sorted_entries =
-  sa->used_entries   =
-  sa->max_entries    = 0;
-  sa->entries        = NULL;
-}
-
-static void sparse_array_delete (sparse_array_t *sa) {
-  sa->sorted_entries =
-  sa->used_entries   =
-  sa->max_entries    = 0;
-  _x_freep (&sa->entries);
-}
-
-static int _sparse_array_find (sparse_array_t *sa, uint32_t key, uint32_t *pos) {
-  uint32_t b = 0, m, e = sa->sorted_entries, l = e, k;
-  while (1) {
-    m = (b + e) >> 1;
-    if (m == l)
-      break;
-    l = m;
-    k = sa->entries[m].key;
-    if (k > key)
-      e = m;
-    else if (k < key)
-      b = m;
-    else {
-      *pos = m;
-      return 1;
-    }
-  }
-  *pos = e;
-  return 0;
-}
-
-static void _sparse_array_sort (sparse_array_t *sa) {
-  uint32_t left = sa->max_entries - sa->used_entries;
-  /* move unsorted part to end of buf */
-  uint32_t i = left + sa->sorted_entries;
-  memmove (sa->entries + i, sa->entries + sa->sorted_entries,
-    (sa->used_entries - sa->sorted_entries) * sizeof (sparse_array_entry_t));
-  /* iterate it */
-  while (i < sa->max_entries) {
-    uint32_t j, pos, startkey, stopkey, lastkey;
-    startkey = sa->entries[i].key;
-    if (_sparse_array_find (sa, startkey, &pos)) {
-      /* eliminate duplicate */
-      sa->entries[pos].value = sa->entries[i].value;
-      i++;
-      continue;
-    }
-    /* find sorted range */
-    stopkey = pos < sa->sorted_entries ? sa->entries[pos].key : 0xffffffff;
-    lastkey = startkey;
-    for (j = i + 1; j < sa->max_entries; j++) {
-      uint32_t thiskey = sa->entries[j].key;
-      if ((thiskey <= lastkey) || (thiskey >= stopkey))
-        break;
-      lastkey = thiskey;
-    }
-    j -= i;
-    if (j > left)
-      j = left;
-    /* insert it */
-    if (pos < sa->sorted_entries)
-      memmove (sa->entries + pos + j, sa->entries + pos,
-        (sa->sorted_entries - pos) * sizeof (sparse_array_entry_t));
-    memcpy (sa->entries + pos, sa->entries + i, j * sizeof (sparse_array_entry_t));
-    sa->sorted_entries += j;
-    i += j;
-  }
-  sa->used_entries = sa->sorted_entries;
-  lprintf ("sparse_array_sort: %u entries\n", (unsigned int)sa->used_entries);
-}
-
-static int sparse_array_set (sparse_array_t *sa, uint32_t key, uint32_t value) {
-  /* give some room for later sorting too */
-  if (!sa->entries || (sa->used_entries + 8 >= sa->max_entries)) {
-    uint32_t n = sa->max_entries + 128;
-    sparse_array_entry_t *se = realloc (sa->entries, n * sizeof (sparse_array_entry_t));
-    if (!se)
-      return 0;
-    sa->max_entries = n;
-    sa->entries = se;
-  }
-  sa->entries[sa->used_entries].key = key;
-  sa->entries[sa->used_entries++].value = value;
-  return 1;
-}
-
-static int sparse_array_get (sparse_array_t *sa, uint32_t key, uint32_t *value) {
-  uint32_t pos;
-  if (sa->sorted_entries != sa->used_entries)
-    _sparse_array_sort (sa);
-  if (!_sparse_array_find (sa, key, &pos))
-    return 0;
-  *value = sa->entries[pos].value;
-  return 1;
-}
-
-static void sparse_array_unset (sparse_array_t *sa, uint32_t key, uint32_t mask) {
-  sparse_array_entry_t *here = sa->entries, *p = NULL, *q = sa->entries;
-  uint32_t i, n = 0;
-  if (sa->sorted_entries != sa->used_entries)
-    _sparse_array_sort (sa);
-  key &= mask;
-  for (i = sa->used_entries; i > 0; i--) {
-    if ((here->key & mask) == key) {
-      if (p) {
-        n = here - p;
-        if (n && (p != q))
-          memmove (q, p, n * sizeof (sparse_array_entry_t));
-        p = NULL;
-        q += n;
-      }
-    } else {
-      if (!p)
-        p = here;
-    }
-    here++;
-  }
-  if (p) {
-    n = here - p;
-    if (n && (p != q))
-      memmove (q, p, n * sizeof (sparse_array_entry_t));
-    q += n;
-  }
-  sa->sorted_entries =
-  sa->used_entries   = q - sa->entries;
-}
-
-/* ! sparse_array */
-
 typedef struct {
   int			x, y;
   unsigned char	is_visible;
@@ -210,15 +61,11 @@ typedef struct {
   int			CLUT_id;
   int			objects_start;
   int			objects_end;
+  unsigned int		object_pos[65536];
   unsigned char	*img;
   osd_object_t          *osd;
 } region_t;
 
-typedef union {
-  clut_t   c;
-  uint32_t u32;
-} clut_union_t;
-
 typedef struct {
 /* dvbsub stuff */
   int			x;
@@ -230,20 +77,26 @@ typedef struct {
   int			i_bits;
   int			in_scanline;
   int			compat_depth;
-  int                   max_regions;
   page_t		page;
   region_t		regions[MAX_REGIONS];
-  clut_union_t		colours[MAX_REGIONS*256];
+  clut_t		colours[MAX_REGIONS*256];
   unsigned char		trans[MAX_REGIONS*256];
   struct {
     unsigned char	  lut24[4], lut28[4], lut48[16];
   }			lut[MAX_REGIONS];
-  sparse_array_t	object_pos;
 } dvbsub_func_t;
 
+typedef struct		dvb_spu_class_s {
+  spu_decoder_class_t	class;
+  xine_t	       *xine;
+
+  int                   ignore_pts;
+} dvb_spu_class_t;
+
 typedef struct dvb_spu_decoder_s {
   spu_decoder_t	spu_decoder;
 
+  dvb_spu_class_t      *class;
   xine_stream_t        *stream;
 
   spu_dvb_descriptor_t *spu_descriptor;
@@ -333,18 +186,24 @@ static void update_region (dvb_spu_decoder_t * this, int region_id, int region_w
 
   /* reject invalid sizes and set some limits ! */
   if ( region_width<=0 || region_height<=0 || region_width>SPU_MAX_WIDTH || region_height>SPU_MAX_HEIGHT ) {
-    _x_freep( &reg->img );
+    if ( reg->img ) {
+      free( reg->img );
+      reg->img = NULL;
+    }
     lprintf("rejected region %d = %dx%d\n", region_id, region_width, region_height );
     return;
   }
 
   if ( reg->width*reg->height<region_width*region_height ) {
     lprintf("update size of region %d = %dx%d\n", region_id, region_width, region_height);
-    _x_freep( &reg->img );
+    if ( reg->img ) {
+      free( reg->img );
+      reg->img = NULL;
+    }
   }
 
   if ( !reg->img ) {
-    if ( !(reg->img = calloc(1, region_width * region_height)) ) {
+    if ( !(reg->img=xine_xmalloc(region_width*region_height)) ) {
       lprintf( "can't allocate mem for region %d\n", region_id );
       return;
     }
@@ -432,9 +291,6 @@ static void decode_2bit_pixel_code_string (dvb_spu_decoder_t * this, int r, int
   int j;
   const uint8_t *lut = lookup_lut (dvbsub, r);
 
-  (void)object_id;
-  (void)ofs;
-
   if (dvbsub->in_scanline == 0)
     dvbsub->in_scanline = 1;
 
@@ -501,9 +357,6 @@ static void decode_4bit_pixel_code_string (dvb_spu_decoder_t * this, int r, int
   int j;
   const uint8_t *lut = lookup_lut (dvbsub, r);
 
-  (void)object_id;
-  (void)ofs;
-
   if (dvbsub->in_scanline == 0)
     dvbsub->in_scanline = 1;
 
@@ -574,9 +427,6 @@ static void decode_8bit_pixel_code_string (dvb_spu_decoder_t * this, int r, int
   dvbsub_func_t *dvbsub = this->dvbsub;
   int j;
 
-  (void)object_id;
-  (void)ofs;
-
   if (dvbsub->in_scanline == 0)
     dvbsub->in_scanline = 1;
 
@@ -621,12 +471,12 @@ static void recalculate_trans (dvb_spu_decoder_t *this)
   int i;
 
   _x_spu_get_opacity (this->stream->xine, &opacity);
-  for (i = 0; i < dvbsub->max_regions * 256; ++i) {
+  for (i = 0; i < MAX_REGIONS * 256; ++i) {
     /* ETSI-300-743 says "full transparency if Y == 0". */
-    if (dvbsub->colours[i].c.y == 0)
+    if (dvbsub->colours[i].y == 0)
       dvbsub->trans[i] = 0;
     else {
-      int v = _x_spu_calculate_opacity (&dvbsub->colours[i].c, dvbsub->colours[i].c.foo, &opacity);
+      int v = _x_spu_calculate_opacity (&dvbsub->colours[i], dvbsub->colours[i].foo, &opacity);
       dvbsub->trans[i] = v * 14 / 255 + 1;
     }
   }
@@ -641,10 +491,10 @@ static void set_clut(dvb_spu_decoder_t *this,int CLUT_id,int CLUT_entry_id,int Y
     return;
   }
 
-  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].c.y   = Y_value;
-  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].c.cr  = Cr_value;
-  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].c.cb  = Cb_value;
-  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].c.foo = T_value;
+  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].y=Y_value;
+  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].cr=Cr_value;
+  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].cb=Cb_value;
+  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].foo = T_value;
 }
 
 static void process_CLUT_definition_segment(dvb_spu_decoder_t *this) {
@@ -671,7 +521,6 @@ static void process_CLUT_definition_segment(dvb_spu_decoder_t *this) {
   j=dvbsub->i+segment_length;
 
   CLUT_id=dvbsub->buf[dvbsub->i++];
-  lprintf ("process_CLUT_definition_segment: CLUT_id %d\n", CLUT_id);
   /*CLUT_version_number=(dvbsub->buf[dvbsub->i]&0xf0)>>4;*/
   dvbsub->i++;
 
@@ -700,7 +549,7 @@ static void process_CLUT_definition_segment(dvb_spu_decoder_t *this) {
   }
 }
 
-static void process_pixel_data_sub_block (dvb_spu_decoder_t * this, int r, int o, unsigned int pos, int ofs, int n)
+static void process_pixel_data_sub_block (dvb_spu_decoder_t * this, int r, int o, int ofs, int n)
 {
   int data_type;
   int j;
@@ -709,8 +558,8 @@ static void process_pixel_data_sub_block (dvb_spu_decoder_t * this, int r, int o
 
   j = dvbsub->i + n;
 
-  dvbsub->x = pos >> 16;
-  dvbsub->y = (pos & 0xffff) + ofs;
+  dvbsub->x = (dvbsub->regions[r].object_pos[o]) >> 16;
+  dvbsub->y = ((dvbsub->regions[r].object_pos[o]) & 0xffff) + ofs;
   while (dvbsub->i < j) {
     data_type = dvbsub->buf[dvbsub->i++];
 
@@ -744,7 +593,7 @@ static void process_pixel_data_sub_block (dvb_spu_decoder_t * this, int r, int o
       break;
     case 0xf0:
       dvbsub->in_scanline = 0;
-      dvbsub->x = pos >> 16;
+      dvbsub->x = (dvbsub->regions[r].object_pos[o]) >> 16;
       dvbsub->y += 2;
       break;
     default:
@@ -780,7 +629,7 @@ static void process_page_composition_segment (dvb_spu_decoder_t * this)
   dvbsub->i++;
 
   int r;
-  for (r=0; r<dvbsub->max_regions; r++) { /* reset */
+  for (r=0; r<MAX_REGIONS; r++) { /* reset */
     dvbsub->page.regions[r].is_visible = 0;
   }
 
@@ -791,12 +640,6 @@ static void process_page_composition_segment (dvb_spu_decoder_t * this)
     dvbsub->i += 2;
     region_y = (dvbsub->buf[dvbsub->i] << 8) | dvbsub->buf[dvbsub->i + 1];
     dvbsub->i += 2;
-    lprintf ("process_page_composition_segment: page_id %d, region_id %d, x %d, y %d\n",
-      dvbsub->page.page_id, region_id, region_x, region_y);
-    if (region_id >= MAX_REGIONS)
-      continue;
-    if (region_id >= dvbsub->max_regions)
-      dvbsub->max_regions = region_id + 1;
 
     dvbsub->page.regions[region_id].x = region_x;
     dvbsub->page.regions[region_id].y = region_y;
@@ -814,6 +657,7 @@ static void process_region_composition_segment (dvb_spu_decoder_t * this)
     /*region_8_bit_pixel_code,*/ region_4_bit_pixel_code /*, region_2_bit_pixel_code*/;
   int object_id, object_type, /*object_provider_flag,*/ object_x, object_y /*, foreground_pixel_code, background_pixel_code*/;
   int j;
+  int o;
   dvbsub_func_t *dvbsub = this->dvbsub;
 
   dvbsub->page.page_id = (dvbsub->buf[dvbsub->i] << 8) | dvbsub->buf[dvbsub->i + 1];
@@ -839,14 +683,10 @@ static void process_region_composition_segment (dvb_spu_decoder_t * this)
   region_4_bit_pixel_code = (dvbsub->buf[dvbsub->i] & 0xf0) >> 4;
   /*region_2_bit_pixel_code = (dvbsub->buf[dvbsub->i] & 0x0c) >> 2;*/
   dvbsub->i++;
-  lprintf ("process_region_composition_segment: region_id %d (%d x %d)\n", region_id, region_width, region_height);
 
   if(region_id>=MAX_REGIONS)
     return;
 
-  if (region_id >= dvbsub->max_regions)
-    dvbsub->max_regions = region_id + 1;
-
   if ( dvbsub->regions[region_id].version_number == region_version_number )
     return;
 
@@ -860,7 +700,9 @@ static void process_region_composition_segment (dvb_spu_decoder_t * this)
   dvbsub->regions[region_id].objects_start = dvbsub->i;
   dvbsub->regions[region_id].objects_end = j;
 
-  sparse_array_unset (&dvbsub->object_pos, region_id << 16, 0xff0000);
+  for (o = 0; o < 65536; o++) {
+    dvbsub->regions[region_id].object_pos[o] = 0xffffffff;
+  }
 
   while (dvbsub->i < j) {
     object_id = (dvbsub->buf[dvbsub->i] << 8) | dvbsub->buf[dvbsub->i + 1];
@@ -872,7 +714,7 @@ static void process_region_composition_segment (dvb_spu_decoder_t * this)
     object_y = ((dvbsub->buf[dvbsub->i] & 0x0f) << 8) | dvbsub->buf[dvbsub->i + 1];
     dvbsub->i += 2;
 
-    sparse_array_set (&dvbsub->object_pos, (region_id << 16) | object_id, (object_x << 16) | object_y);
+    dvbsub->regions[region_id].object_pos[object_id] = (object_x << 16) | object_y;
 
     if ((object_type == 0x01) || (object_type == 0x02)) {
       /*foreground_pixel_code = dvbsub->buf[*/dvbsub->i++/*]*/;
@@ -907,12 +749,11 @@ static void process_object_data_segment (dvb_spu_decoder_t * this)
   dvbsub->i++;
 
   old_i = dvbsub->i;
-  for (r = 0; r < dvbsub->max_regions; r++) {
+  for (r = 0; r < MAX_REGIONS; r++) {
 
     /* If this object is in this region... */
     if (dvbsub->regions[r].img) {
-      uint32_t pos;
-      if (sparse_array_get (&dvbsub->object_pos, (r << 16) | object_id, &pos)) {
+      if (dvbsub->regions[r].object_pos[object_id] != 0xffffffff) {
 	dvbsub->i = old_i;
 	if (object_coding_method == 0) {
 	  top_field_data_block_length = (dvbsub->buf[dvbsub->i] << 8) | dvbsub->buf[dvbsub->i + 1];
@@ -920,7 +761,7 @@ static void process_object_data_segment (dvb_spu_decoder_t * this)
 	  bottom_field_data_block_length = (dvbsub->buf[dvbsub->i] << 8) | dvbsub->buf[dvbsub->i + 1];
 	  dvbsub->i += 2;
 
-	  process_pixel_data_sub_block (this, r, object_id, pos, 0, top_field_data_block_length);
+	  process_pixel_data_sub_block (this, r, object_id, 0, top_field_data_block_length);
 
 	  if (bottom_field_data_block_length == 0)
 	  {
@@ -929,7 +770,7 @@ static void process_object_data_segment (dvb_spu_decoder_t * this)
 	    dvbsub->i =  old_i + 4;
 	  }
 
-	  process_pixel_data_sub_block (this, r, object_id, pos, 1, bottom_field_data_block_length);
+	  process_pixel_data_sub_block (this, r, object_id, 1, bottom_field_data_block_length);
 	}
       }
     }
@@ -939,7 +780,6 @@ static void process_object_data_segment (dvb_spu_decoder_t * this)
 static void process_display_definition_segment(dvb_spu_decoder_t *this)
 {
   /* FIXME: not implemented. */
-  (void)this;
 }
 
 static void unlock_mutex_cancellation_func(void *mutex_gen)
@@ -984,9 +824,9 @@ static void* dvbsub_timer_func(void *this_gen)
     {
       /* We timed out, and no-one changed the timeout underneath us.
          Hide the OSD, then wait until we're signalled. */
-      if(this->stream && this->stream->osd_renderer)
+      if(this && this->stream && this->stream->osd_renderer)
       {
-	for ( i=0; i<this->dvbsub->max_regions; i++ ) {
+	for ( i=0; i<MAX_REGIONS; i++ ) {
 	  if ( this->dvbsub->regions[i].osd ) {
 	    this->stream->osd_renderer->hide( this->dvbsub->regions[i].osd, 0 );
 	    lprintf("thread hiding = %d\n",i);
@@ -1027,14 +867,14 @@ static void draw_subtitles (dvb_spu_decoder_t * this)
 
   /* render all regions onto the page */
 
-  for ( r=0; r<this->dvbsub->max_regions; r++ ) {
+  for ( r=0; r<MAX_REGIONS; r++ ) {
     if ( this->dvbsub->page.regions[r].is_visible )
       display++;
   }
   if ( !display )
     return;
 
-  for (r = 0; r < this->dvbsub->max_regions; r++) {
+  for (r = 0; r < MAX_REGIONS; r++) {
     if (this->dvbsub->regions[r].img) {
       if (this->dvbsub->page.regions[r].is_visible && !this->dvbsub->regions[r].empty) {
         update_osd( this, r );
@@ -1042,7 +882,7 @@ static void draw_subtitles (dvb_spu_decoder_t * this)
 	  continue;
         /* clear osd */
         this->stream->osd_renderer->clear( this->dvbsub->regions[r].osd );
-        if ( this->dvbsub->regions[r].width>dest_width && !(this->stream->video_out->get_capabilities(this->stream->video_out) & VO_CAP_CUSTOM_EXTENT_OVERLAY)) {
+        if ( this->dvbsub->regions[r].width>dest_width && !(this->stream->video_driver->get_capabilities(this->stream->video_driver) & VO_CAP_CUSTOM_EXTENT_OVERLAY)) {
 	  downscale_region_image(&this->dvbsub->regions[r], tmp, dest_width);
 	  reg = tmp;
 	  reg_width = dest_width;
@@ -1051,11 +891,7 @@ static void draw_subtitles (dvb_spu_decoder_t * this)
 	  reg = this->dvbsub->regions[r].img;
 	  reg_width = this->dvbsub->regions[r].width;
 	}
-        /* All DVB subs I have seen so far use same color matrix as main video. */
-        _X_SET_CLUT_CM (&this->dvbsub->colours[this->dvbsub->regions[r].CLUT_id*256].u32, 4);
-	this->stream->osd_renderer->set_palette( this->dvbsub->regions[r].osd,
-                                                 &this->dvbsub->colours[this->dvbsub->regions[r].CLUT_id*256].u32,
-                                                 &this->dvbsub->trans[this->dvbsub->regions[r].CLUT_id*256]);
+	this->stream->osd_renderer->set_palette( this->dvbsub->regions[r].osd, (uint32_t*)(&this->dvbsub->colours[this->dvbsub->regions[r].CLUT_id*256]), &this->dvbsub->trans[this->dvbsub->regions[r].CLUT_id*256]);
 	this->stream->osd_renderer->draw_bitmap( this->dvbsub->regions[r].osd, reg, 0, 0, reg_width, this->dvbsub->regions[r].height, NULL );
       }
     }
@@ -1063,7 +899,7 @@ static void draw_subtitles (dvb_spu_decoder_t * this)
 
   pthread_mutex_lock(&this->dvbsub_osd_mutex);
   lprintf("this->vpts=%"PRId64"\n",this->vpts);
-  for ( r=0; r<this->dvbsub->max_regions; r++ ) {
+  for ( r=0; r<MAX_REGIONS; r++ ) {
     lprintf("region=%d, visible=%d, osd=%d, empty=%d\n", r, this->dvbsub->page.regions[r].is_visible, this->dvbsub->regions[r].osd?1:0, this->dvbsub->regions[r].empty );
     if ( this->dvbsub->page.regions[r].is_visible && this->dvbsub->regions[r].osd && !this->dvbsub->regions[r].empty ) {
       this->stream->osd_renderer->set_position( this->dvbsub->regions[r].osd, this->dvbsub->page.regions[r].x, this->dvbsub->page.regions[r].y );
@@ -1101,7 +937,7 @@ static void spudec_decode_data (spu_decoder_t * this_gen, buf_element_t * buf)
       if (buf->decoder_info[2] == 0) {
         /* Hide the osd - note that if the timeout thread times out, it'll rehide, which is harmless */
         pthread_mutex_lock(&this->dvbsub_osd_mutex);
-	for ( i=0; i<this->dvbsub->max_regions; i++ ) {
+	for ( i=0; i<MAX_REGIONS; i++ ) {
 	  if ( this->dvbsub->regions[i].osd )
 	    this->stream->osd_renderer->hide( this->dvbsub->regions[i].osd, 0 );
 	}
@@ -1135,7 +971,7 @@ static void spudec_decode_data (spu_decoder_t * this_gen, buf_element_t * buf)
   /* don't ask metronom for a vpts but rather do the calculation
    * because buf->pts could be too far in future and metronom won't accept
    * further backwards pts (see metronom_got_spu_packet) */
-  if (buf->pts > 0) {
+  if (!this->class->ignore_pts && buf->pts > 0) {
     metronom_t *metronom = this->stream->metronom;
     int64_t vpts_offset = metronom->get_option( metronom, METRONOM_VPTS_OFFSET );
     int64_t spu_offset = metronom->get_option( metronom, METRONOM_SPU_OFFSET );
@@ -1231,52 +1067,43 @@ static void spudec_reset (spu_decoder_t * this_gen)
 static void spudec_discontinuity (spu_decoder_t * this_gen)
 {
   /* do nothing */
-  (void)this_gen;
-}
-
-static void spudec_dispose_internal (dvb_spu_decoder_t * this, int thread_running)
-{
-  int i;
-
-  if (thread_running) {
-    pthread_cancel(this->dvbsub_timer_thread);
-    pthread_join(this->dvbsub_timer_thread, NULL);
-  }
-  pthread_mutex_destroy(&this->dvbsub_osd_mutex);
-  pthread_cond_destroy(&this->dvbsub_restart_timeout);
-
-  _x_freep(&this->spu_descriptor);
-
-  for ( i=0; i<MAX_REGIONS; i++ ) {
-    _x_freep( &this->dvbsub->regions[i].img );
-    if ( this->dvbsub->regions[i].osd )
-      this->stream->osd_renderer->free_object( this->dvbsub->regions[i].osd );
-  }
-
-  _x_freep (&this->pes_pkt);
-
-  if (this->dvbsub) {
-    sparse_array_delete (&this->dvbsub->object_pos);
-    _x_freep (&this->dvbsub);
-  }
-
-  free (this);
 }
 
 static void spudec_dispose (spu_decoder_t * this_gen)
 {
   dvb_spu_decoder_t *this = (dvb_spu_decoder_t *) this_gen;
-  spudec_dispose_internal(this, 1);
+  int i;
+
+  pthread_cancel(this->dvbsub_timer_thread);
+  pthread_join(this->dvbsub_timer_thread, NULL);
+  pthread_mutex_destroy(&this->dvbsub_osd_mutex);
+  pthread_cond_destroy(&this->dvbsub_restart_timeout);
+
+  if(this->spu_descriptor){
+    free(this->spu_descriptor);
+    this->spu_descriptor=NULL;
+  }
+
+  for ( i=0; i<MAX_REGIONS; i++ ) {
+    if ( this->dvbsub->regions[i].img )
+      free( this->dvbsub->regions[i].img );
+    if ( this->dvbsub->regions[i].osd )
+      this->stream->osd_renderer->free_object( this->dvbsub->regions[i].osd );
+  }
+
+  if (this->pes_pkt)
+    free (this->pes_pkt);
+
+  if (this->dvbsub)
+    free (this->dvbsub);
+
+  free (this);
 }
 
 static spu_decoder_t *dvb_spu_class_open_plugin (spu_decoder_class_t * class_gen, xine_stream_t * stream)
 {
-  dvb_spu_decoder_t *this;
-
-  (void)class_gen;
-  this = calloc(1, sizeof (dvb_spu_decoder_t));
-  if (!this)
-    return NULL;
+  dvb_spu_decoder_t *this = calloc(1, sizeof (dvb_spu_decoder_t));
+  dvb_spu_class_t *class = (dvb_spu_class_t *)class_gen;
 
 #define YUVA(r, g, b, a) (clut_t) { COMPUTE_V(r, g, b), COMPUTE_U(r, g, b), COMPUTE_V(r, g, b), a }
 #define GETBIT(s, v1, v2, tr) \
@@ -1313,18 +1140,13 @@ static spu_decoder_t *dvb_spu_class_open_plugin (spu_decoder_class_t * class_gen
   this->spu_decoder.get_interact_info = NULL;
   this->spu_decoder.set_button = NULL;
 
+  this->class = class;
   this->stream = stream;
 
-  pthread_mutex_init(&this->dvbsub_osd_mutex, NULL);
-  pthread_cond_init(&this->dvbsub_restart_timeout, NULL);
-
-  this->pes_pkt        = calloc(65, 1024);
+  this->pes_pkt = calloc(65, 1024);
   this->spu_descriptor = calloc(1, sizeof(spu_dvb_descriptor_t));
-  this->dvbsub         = calloc(1, sizeof (dvbsub_func_t));
-  if (!this->pes_pkt || !this->spu_descriptor || !this->dvbsub) {
-    spudec_dispose_internal(this, 0);
-    return NULL;
-  }
+
+  this->dvbsub = calloc(1, sizeof (dvbsub_func_t));
 
   int i;
   for (i = 0; i < MAX_REGIONS; i++) {
@@ -1343,34 +1165,41 @@ static spu_decoder_t *dvb_spu_class_open_plugin (spu_decoder_class_t * class_gen
     t = _x_spu_calculate_opacity (&black, 0, &opacity);
 
     for (i = 0; i < MAX_REGIONS * 256; i++)
-      this->dvbsub->colours[i].c.foo = t;
+      this->dvbsub->colours[i].foo = t;
   }
 
-  sparse_array_new (&this->dvbsub->object_pos);
-
+  pthread_mutex_init(&this->dvbsub_osd_mutex, NULL);
+  pthread_cond_init(&this->dvbsub_restart_timeout, NULL);
   this->dvbsub_hide_timeout.tv_nsec = 0;
   this->dvbsub_hide_timeout.tv_sec = time(NULL);
-
-  if (pthread_create(&this->dvbsub_timer_thread, NULL, dvbsub_timer_func, this)) {
-    xprintf (stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": pthread_create() failed\n");
-    spudec_dispose_internal(this, 0);
-    return NULL;
-  }
+  pthread_create(&this->dvbsub_timer_thread, NULL, dvbsub_timer_func, this);
 
   return (spu_decoder_t *) this;
 }
 
-static void *init_spu_decoder_plugin (xine_t * xine, const void *data)
+static void dvb_spu_decoder_class_dispose (spu_decoder_class_t * this_gen)
 {
-  (void)xine;
-  (void)data;
-  static const spu_decoder_class_t decode_dvb_spu_class = {
-    .open_plugin = dvb_spu_class_open_plugin,
-    .identifier  = "spudvb",
-    .description = N_("DVB subtitle decoder plugin"),
-    .dispose = NULL,
-  };
-  return (void *)&decode_dvb_spu_class;
+  dvb_spu_class_t *this = (dvb_spu_class_t *) this_gen;
+
+  this->xine->config->unregister_callback(this->xine->config, "subtitles.dvb.ignore_pts");
+
+  free (this);
+}
+
+static void *init_spu_decoder_plugin (xine_t * xine, void *data)
+{
+
+  dvb_spu_class_t *this;
+  this = calloc(1, sizeof (dvb_spu_class_t));
+
+  this->class.open_plugin = dvb_spu_class_open_plugin;
+  this->class.identifier  = "spudvb";
+  this->class.description = N_("DVB subtitle decoder plugin");
+  this->class.dispose = dvb_spu_decoder_class_dispose;
+
+  this->xine = xine;
+
+  return &this->class;
 }
 
 
@@ -1378,13 +1207,13 @@ static void *init_spu_decoder_plugin (xine_t * xine, const void *data)
 static const uint32_t supported_types[] = { BUF_SPU_DVB, 0 };
 
 static const decoder_info_t spudec_info = {
-  .supported_types = supported_types,
-  .priority        = 1,
+  supported_types,		/* supported types */
+  1				/* priority        */
 };
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
 /* type, API, "name", version, special_info, init_function */
   {PLUGIN_SPU_DECODER, 17, "spudvb", XINE_VERSION_CODE, &spudec_info,
    &init_spu_decoder_plugin},
-  {PLUGIN_NONE, 0, NULL, 0, NULL, NULL}
+  {PLUGIN_NONE, 0, "", 0, NULL, NULL}
 };
