@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2018 the xine project
+ * Copyright (C) 2000-2011 the xine project
  *
  * Copyright (C) 2009 Petri Hintukainen <phintuka@users.sourceforge.net>
  *
@@ -31,18 +31,16 @@
 #include <unistd.h>
 #include <string.h>
 
-#define LOG_MODULE "spu_hdmv"
-/*
-#define LOG
-*/
-
 #include <xine/xine_internal.h>
 #include <xine/buffer.h>
 #include <xine/xineutils.h>
 #include <xine/video_out.h>
 #include <xine/video_overlay.h>
 
-#define XINE_HDMV_TRACE lprintf
+#define XINE_HDMV_TRACE(x...) printf(x)
+/*#define XINE_HDMV_TRACE(x...) */
+#define XINE_HDMV_ERROR(x...) fprintf(stderr, "spuhdmv: " x)
+/*#define XINE_HDMV_ERROR(x...) lprintf(x) */
 
 #ifndef MAX
 #  define MAX(a,b) (a>b)?(a):(b)
@@ -184,8 +182,8 @@ struct presentation_segment_s {
 static void free_subtitle_object(void *ptr)
 {
   if (ptr) {
-    _x_freep(&((subtitle_object_t*)ptr)->rle);
-    _x_freep(&((subtitle_object_t*)ptr)->raw_data);
+    free(((subtitle_object_t*)ptr)->rle);
+    free(((subtitle_object_t*)ptr)->raw_data);
     free(ptr);
   }
 }
@@ -231,7 +229,8 @@ static segment_buffer_t *segbuf_init(void)
 
 static void segbuf_dispose(segment_buffer_t *buf)
 {
-  _x_freep (&buf->buf);
+  if (buf->buf)
+    free (buf->buf);
   free (buf);
 }
 
@@ -260,7 +259,7 @@ static void segbuf_parse_segment_header(segment_buffer_t *buf)
     if ( buf->segment_type < 0x14 ||
          ( buf->segment_type > 0x18 &&
            buf->segment_type != 0x80)) {
-      XINE_HDMV_TRACE("unknown segment type 0x%02x, resetting\n", buf->segment_type);
+      XINE_HDMV_ERROR("unknown segment type 0x%02x, resetting\n", buf->segment_type);
       segbuf_reset(buf);
     }
   } else {
@@ -273,7 +272,10 @@ static void segbuf_fill(segment_buffer_t *buf, uint8_t *data, size_t len)
 {
   if (buf->len + len > buf->data_size) {
     buf->data_size = buf->len + len;
-    buf->buf = realloc(buf->buf, buf->data_size);
+    if (buf->buf)
+      buf->buf = realloc(buf->buf, buf->data_size);
+    else
+      buf->buf = malloc(buf->data_size);
   }
 
   memcpy(buf->buf + buf->len, data, len);
@@ -289,8 +291,7 @@ static int segbuf_segment_complete(segment_buffer_t *buf)
 
 static void segbuf_skip_segment(segment_buffer_t *buf)
 {
-  _x_assert (segbuf_segment_complete (buf));
-
+  if (segbuf_segment_complete (buf)) {
     buf->len -= buf->segment_len + 3;
     if (buf->len > 0)
       memmove(buf->buf, buf->buf + buf->segment_len + 3, buf->len);
@@ -298,6 +299,11 @@ static void segbuf_skip_segment(segment_buffer_t *buf)
     segbuf_parse_segment_header(buf);
 
     XINE_HDMV_TRACE("  skip_segment: %zd bytes left\n", buf->len);
+  } else {
+    XINE_HDMV_ERROR("  skip_segment: ERROR - %zd bytes queued, %d required\n",
+                    buf->len, buf->segment_len);
+    segbuf_reset (buf);
+  }
 }
 
 /*
@@ -320,7 +326,7 @@ static uint8_t segbuf_get_u8(segment_buffer_t *buf)
 {
   if (!(buf->error = ++buf->segment_data > buf->segment_end))
     return buf->segment_data[-1];
-  XINE_HDMV_TRACE("segbuf_get_u8: read failed (end of segment reached) !\n");
+  XINE_HDMV_ERROR("segbuf_get_u8: read failed (end of segment reached) !\n");
   return 0;
 }
 
@@ -340,7 +346,6 @@ static uint32_t segbuf_get_u24(segment_buffer_t *buf)
 
 static subtitle_clut_t *segbuf_decode_palette(segment_buffer_t *buf)
 {
-  subtitle_clut_t *clut;
   uint8_t palette_id             = segbuf_get_u8 (buf);
   uint8_t palette_version_number = segbuf_get_u8 (buf);
 
@@ -352,21 +357,15 @@ static subtitle_clut_t *segbuf_decode_palette(segment_buffer_t *buf)
     return NULL;
 
   if (len % 5) {
-    XINE_HDMV_TRACE("  decode_palette: segment size error (%zd ; expected %zd for %zd entries)\n",
+    XINE_HDMV_ERROR("  decode_palette: segment size error (%zd ; expected %zd for %zd entries)\n",
                     len, (5 * entries), entries);
     return NULL;
   }
-
-  (void)palette_version_number;
   XINE_HDMV_TRACE("decode_palette: %zd items (id %d, version %d)\n",
                   entries, palette_id, palette_version_number);
 
   /* convert to xine-lib clut */
-
-  clut = calloc(1, sizeof(subtitle_clut_t));
-  if (!clut)
-    return NULL;
-
+  subtitle_clut_t *clut = calloc(1, sizeof(subtitle_clut_t));
   clut->id = palette_id;
 
   for (i = 0; i < entries; i++) {
@@ -375,17 +374,10 @@ static subtitle_clut_t *segbuf_decode_palette(segment_buffer_t *buf)
     uint8_t Cr    = segbuf_get_u8 (buf);
     uint8_t Cb    = segbuf_get_u8 (buf);
     uint8_t alpha = segbuf_get_u8 (buf);
-#ifdef WORDS_BIGENDIAN
-    clut->color[index] = (Y << 8) | (Cr << 16) | (Cb << 24);
-#else
     clut->color[index] = (Y << 16) | (Cr << 8) | Cb;
-#endif
     clut->trans[index] = alpha >> 4;
   }
 
-  /* HDMV subs always use same color matrix as main video. */
-  _X_SET_CLUT_CM (clut->color, 4);
-
   return clut;
 }
 
@@ -459,18 +451,13 @@ static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf, subtitle_o
   uint8_t  version   = segbuf_get_u8 (buf);
   uint8_t  seq_desc  = segbuf_get_u8 (buf);
 
-  (void)version;
   XINE_HDMV_TRACE("  decode_object: object_id %d, version %d, seq 0x%x\n",
                   object_id, version, seq_desc);
 
   if (seq_desc & 0x80) {
-    subtitle_object_t *obj;
-
     /* new object (first-in-sequence flag set) */
 
-    obj = calloc(1, sizeof(subtitle_object_t));
-    if (!obj)
-      return NULL;
+    subtitle_object_t *obj = calloc(1, sizeof(subtitle_object_t));
 
     obj->id       = object_id;
     obj->data_len = segbuf_get_u24(buf);
@@ -491,7 +478,8 @@ static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf, subtitle_o
       XINE_HDMV_TRACE("    object length %d bytes, have only %zd bytes -> missing %d bytes\n",
                       obj->data_len, segbuf_data_length(buf), obj->data_len - (int)segbuf_data_length(buf));
 
-      _x_freep(&obj->raw_data);
+      if (obj->raw_data)
+        free(obj->raw_data);
 
       /* store partial RLE data in HDMV format */
       obj->raw_data_len  = segbuf_data_length(buf);
@@ -526,7 +514,7 @@ static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf, subtitle_o
 
   /* store partial RLE data in HDMV format */
   if (objects->raw_data_size < objects->raw_data_len + segbuf_data_length(buf)) {
-    XINE_HDMV_TRACE("object larger than object size !\n");
+    XINE_HDMV_ERROR("object larger than object size !\n");
     return NULL;
   }
   memcpy(objects->raw_data + objects->raw_data_len, buf->segment_data, segbuf_data_length(buf));
@@ -548,7 +536,8 @@ static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf, subtitle_o
     }
 
     /* free decode buffer */
-    _x_freep(&objects->raw_data);
+    free(objects->raw_data);
+    objects->raw_data      = NULL;
     objects->raw_data_len  = 0;
     objects->raw_data_size = 0;
   }
@@ -558,21 +547,15 @@ static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf, subtitle_o
 
 static window_def_t *segbuf_decode_window_definition(segment_buffer_t *buf)
 {
-  window_def_t *wnd;
-  uint8_t a;
+  window_def_t *wnd = calloc(1, sizeof(window_def_t));
 
-  wnd = calloc(1, sizeof(window_def_t));
-  if (!wnd)
-    return NULL;
-
-  a           = segbuf_get_u8 (buf);
+  uint8_t a   = segbuf_get_u8 (buf);
   wnd->id     = segbuf_get_u8 (buf);
   wnd->xpos   = segbuf_get_u16 (buf);
   wnd->ypos   = segbuf_get_u16 (buf);
   wnd->width  = segbuf_get_u16 (buf);
   wnd->height = segbuf_get_u16 (buf);
 
-  (void)a;
   XINE_HDMV_TRACE("  window: [%02x %d]  %d,%d %dx%d\n", a,
                   wnd->id, wnd->xpos, wnd->ypos, wnd->width, wnd->height);
 
@@ -590,9 +573,6 @@ static int segbuf_decode_video_descriptor(segment_buffer_t *buf)
   uint16_t height     = segbuf_get_u16(buf);
   uint8_t  frame_rate = segbuf_get_u8 (buf);
 
-  (void)width;
-  (void)height;
-  (void)frame_rate;
   XINE_HDMV_TRACE("  video_descriptor: %dx%d fps %d\n", width, height, frame_rate);
 
   return buf->error;
@@ -609,16 +589,11 @@ static int segbuf_decode_composition_descriptor(segment_buffer_t *buf, compositi
 
 static composition_object_t *segbuf_decode_composition_object(segment_buffer_t *buf)
 {
-  composition_object_t *cobj;
-  uint8_t tmp;
-
-  cobj = calloc(1, sizeof(composition_object_t));
-  if (!cobj)
-    return NULL;
+  composition_object_t *cobj = calloc(1, sizeof(composition_object_t));
 
   cobj->object_id_ref    = segbuf_get_u16 (buf);
   cobj->window_id_ref    = segbuf_get_u8 (buf);
-  tmp                    = segbuf_get_u8 (buf);
+  uint8_t tmp            = segbuf_get_u8 (buf);
   cobj->cropped_flag     = !!(tmp & 0x80);
   cobj->forced_flag      = !!(tmp & 0x40);
   cobj->xpos             = segbuf_get_u16 (buf);
@@ -646,13 +621,9 @@ static composition_object_t *segbuf_decode_composition_object(segment_buffer_t *
 
 static presentation_segment_t *segbuf_decode_presentation_segment(segment_buffer_t *buf)
 {
-  presentation_segment_t *seg;
+  presentation_segment_t *seg = calloc(1, sizeof(presentation_segment_t));
   int                     index;
 
-  seg = calloc(1, sizeof(presentation_segment_t));
-  if (!seg)
-    return NULL;
-
   segbuf_decode_video_descriptor (buf);
   segbuf_decode_composition_descriptor (buf, &seg->comp_descr);
 
@@ -665,10 +636,8 @@ static presentation_segment_t *segbuf_decode_presentation_segment(segment_buffer
 
   for (index = 0; index < seg->object_number; index++) {
     composition_object_t *cobj = segbuf_decode_composition_object (buf);
-    if (cobj) {
-      cobj->next = seg->comp_objs;
-      seg->comp_objs = cobj;
-    }
+    cobj->next = seg->comp_objs;
+    seg->comp_objs = cobj;
   }
 
   if (buf->error) {
@@ -682,12 +651,9 @@ static presentation_segment_t *segbuf_decode_presentation_segment(segment_buffer
 static rle_elem_t *copy_crop_rle(subtitle_object_t *obj, composition_object_t *cobj)
 {
   /* TODO: cropping (w,h sized image from pos x,y) */
-  (void)cobj;
 
   rle_elem_t *rle = calloc (obj->num_rle, sizeof(rle_elem_t));
-  if (rle) {
-    memcpy (rle, obj->rle, obj->num_rle * sizeof(rle_elem_t));
-  }
+  memcpy (rle, obj->rle, obj->num_rle * sizeof(rle_elem_t));
   return rle;
 }
 
@@ -696,9 +662,14 @@ static rle_elem_t *copy_crop_rle(subtitle_object_t *obj, composition_object_t *c
  * xine plugin
  */
 
+typedef struct {
+  spu_decoder_class_t decoder_class;
+} spuhdmv_class_t;
+
 typedef struct spuhdmv_decoder_s {
   spu_decoder_t    spu_decoder;
 
+  spuhdmv_class_t  *class;
   xine_stream_t    *stream;
 
   segment_buffer_t *buf;
@@ -785,8 +756,8 @@ static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, uns
 {
   video_overlay_manager_t *ovl_manager = this->stream->video_out->get_overlay_manager(this->stream->video_out);
   metronom_t              *metronom    = this->stream->metronom;
-  video_overlay_event_t    event       = {.vpts = 0};
-  vo_overlay_t             overlay     = {.x = 0};
+  video_overlay_event_t    event       = {0};
+  vo_overlay_t             overlay     = {0};
 
   /* find palette */
   subtitle_clut_t *clut = this->cluts;
@@ -833,13 +804,10 @@ static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, uns
   overlay.width     = obj->width;
   overlay.height    = obj->height;
 
+  overlay.rle       = copy_crop_rle (obj, cobj);
   overlay.num_rle   = obj->num_rle;
   overlay.data_size = obj->num_rle * sizeof(rle_elem_t);
 
-  overlay.rle       = copy_crop_rle (obj, cobj);
-  if (!overlay.rle)
-    return -1;
-
   /* */
 
   overlay.x = /*wnd->xpos +*/ cobj->xpos;
@@ -882,7 +850,7 @@ static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, uns
 
 static void hide_overlays(spuhdmv_decoder_t *this, int64_t pts)
 {
-  video_overlay_event_t event = {.vpts = 0};
+  video_overlay_event_t event = {0};
   int i = 0;
 
   while (this->overlay_handles[i] >= 0) {
@@ -892,7 +860,10 @@ static void hide_overlays(spuhdmv_decoder_t *this, int64_t pts)
     metronom_t              *metronom    = this->stream->metronom;
 
     event.object.handle = this->overlay_handles[i];
-    event.vpts = metronom->got_spu_packet (metronom, pts);
+    if (this)
+      event.vpts = metronom->got_spu_packet (metronom, pts);
+    else
+      event.vpts = 0;
     event.event_type = OVERLAY_EVENT_HIDE;
     event.object.overlay = NULL;
     ovl_manager->add_event (ovl_manager, (void *)&event);
@@ -922,8 +893,7 @@ static void update_overlays(spuhdmv_decoder_t *this)
 
       for (i = 0; i < pseg->object_number; i++) {
         if (!cobj) {
-          xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": "
-                   "Error displaying overlay %d: missing composition object\n", i);
+          XINE_HDMV_ERROR("show_overlays: composition object %d missing !\n", i);
         } else {
           show_overlay(this, cobj, pseg->palette_id_ref, i, pseg->pts, !pseg->shown);
           cobj = cobj->next;
@@ -939,27 +909,25 @@ static void update_overlays(spuhdmv_decoder_t *this)
 
 static void decode_segment(spuhdmv_decoder_t *this)
 {
-  int err = 0;
-
   XINE_HDMV_TRACE("*** new segment, pts %010"PRId64": 0x%02x (%8d bytes)\n",
                   this->pts, this->buf->segment_type, this->buf->segment_len);
 
   switch (segbuf_segment_type(this->buf)) {
   case SEGTYPE_PALETTE:
     XINE_HDMV_TRACE("  segment: PALETTE\n");
-    err = decode_palette(this);
+    decode_palette(this);
     break;
   case SEGTYPE_OBJECT:
     XINE_HDMV_TRACE("  segment: OBJECT\n");
-    err = decode_object(this);
+    decode_object(this);
     break;
   case SEGTYPE_PRESENTATION_SEGMENT:
     XINE_HDMV_TRACE("  segment: PRESENTATION SEGMENT\n");
-    err = decode_presentation_segment(this);
+    decode_presentation_segment(this);
     break;
   case SEGTYPE_WINDOW_DEFINITION:
     XINE_HDMV_TRACE("  segment: WINDOW DEFINITION\n");
-    err = decode_window_definition(this);
+    decode_window_definition(this);
     break;
   case SEGTYPE_INTERACTIVE:
     XINE_HDMV_TRACE("  segment: INTERACTIVE\n");
@@ -972,13 +940,11 @@ static void decode_segment(spuhdmv_decoder_t *this)
 #endif
     break;
   default:
-    xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": "
-             "Unknown segment type 0x%x, skipping\n", segbuf_segment_type(this->buf));
+    XINE_HDMV_ERROR("  segment type 0x%x unknown, skipping\n", segbuf_segment_type(this->buf));
     break;
   }
-  if (err || this->buf->error) {
-    xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": "
-             "Error decoding segment 0x%x\n", segbuf_segment_type(this->buf));
+  if (this->buf->error) {
+    XINE_HDMV_ERROR("*** DECODE ERROR ***\n");
   }
 
   update_overlays (this);
@@ -1062,17 +1028,7 @@ static spu_decoder_t *open_plugin (spu_decoder_class_t *class_gen, xine_stream_t
 {
   spuhdmv_decoder_t *this;
 
-  (void)class_gen;
-
   this = (spuhdmv_decoder_t *) calloc(1, sizeof (spuhdmv_decoder_t));
-  if (!this)
-    return NULL;
-
-  this->buf = segbuf_init();
-  if (!this->buf) {
-    free(this);
-    return NULL;
-  }
 
   this->spu_decoder.decode_data         = spudec_decode_data;
   this->spu_decoder.reset               = spudec_reset;
@@ -1081,36 +1037,39 @@ static spu_decoder_t *open_plugin (spu_decoder_class_t *class_gen, xine_stream_t
   this->spu_decoder.get_interact_info   = NULL;
   this->spu_decoder.set_button          = NULL;
   this->stream                          = stream;
+  this->class                           = (spuhdmv_class_t *) class_gen;
+
+  this->buf = segbuf_init();
 
   memset(this->overlay_handles, 0xff, sizeof(this->overlay_handles)); /* --> -1 */
 
   return &this->spu_decoder;
 }
 
-static void *init_plugin (xine_t *xine, const void *data)
+static void *init_plugin (xine_t *xine, void *data)
 {
-  (void)xine;
-  (void)data;
+  spuhdmv_class_t *this;
 
-  static const spu_decoder_class_t decode_spu_hdmv_class = {
-    .open_plugin = open_plugin,
-    .identifier  = "spuhdmv",
-    .description = "HDMV/BluRay bitmap SPU decoder plugin",
-    .dispose     = NULL,
-  };
-  return (void *)&decode_spu_hdmv_class;
+  this = calloc(1, sizeof (spuhdmv_class_t));
+
+  this->decoder_class.open_plugin = open_plugin;
+  this->decoder_class.identifier  = "spuhdmv";
+  this->decoder_class.description = "HDMV/BluRay bitmap SPU decoder plugin";
+  this->decoder_class.dispose     = default_spu_decoder_class_dispose;
+
+  return this;
 }
 
 /* plugin catalog information */
 static const uint32_t supported_types[] = { BUF_SPU_HDMV, 0 };
 
 static const decoder_info_t dec_info_data = {
-  .supported_types = supported_types,
-  .priority        = 5,
+  supported_types,     /* supported types */
+  5                    /* priority        */
 };
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_SPU_DECODER, 17, "spuhdmv", XINE_VERSION_CODE, &dec_info_data, &init_plugin },
-  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
+  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
