@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2020 the xine project
+ * Copyright (C) 2000-2013 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -37,6 +37,8 @@
 #include <unistd.h>
 #include <string.h>
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <ctype.h>
 
 #define LOG_MODULE "demux_sputext"
@@ -49,8 +51,6 @@
 #include <xine/xineutils.h>
 #include <xine/demux.h>
 
-#include "sputext_demuxer.h"
-
 #define ERR           (void *)-1
 #define SUB_MAX_TEXT  5
 #define SUB_BUFSIZE   1024
@@ -94,10 +94,18 @@ typedef struct {
   int                format;         /* constants see below        */
   char               next_line[SUB_BUFSIZE]; /* a buffer for next line read from file */
 
-  char              *encoding; /* charset. NULL if unknown. currently only "utf-8" autodetected. */
-
 } demux_sputext_t;
 
+typedef struct demux_sputext_class_s {
+
+  demux_class_t      demux_class;
+
+  int                max_timeout;  /* default timeout of hidding subtitles */
+
+} demux_sputext_class_t;
+
+void *init_sputext_demux_class (xine_t *xine, void *data);
+
 /*
  * Demuxer code start
  */
@@ -300,8 +308,7 @@ static subtitle_t *sub_read_line_microdvd(demux_sputext_t *this, subtitle_t *cur
 static subtitle_t *sub_read_line_subviewer(demux_sputext_t *this, subtitle_t *current) {
 
   char line[LINE_LEN + 1];
-  int a1,a2,b1,b2;
-  float a3,b3;
+  int a1,a2,a3,a4,b1,b2,b3,b4;
   char *p=NULL, *q=NULL;
   int len;
 
@@ -309,11 +316,12 @@ static subtitle_t *sub_read_line_subviewer(demux_sputext_t *this, subtitle_t *cu
 
   while (1) {
     if (!read_line_from_input(this, line, LINE_LEN)) return NULL;
-    if (sscanf (line, "%d:%d:%f,%d:%d:%f",&a1,&a2,&a3,&b1,&b2,&b3) < 6) {
+    if (sscanf (line, "%d:%d:%d.%d,%d:%d:%d.%d",&a1,&a2,&a3,&a4,&b1,&b2,&b3,&b4) < 8) {
+      if (sscanf (line, "%d:%d:%d,%d,%d:%d:%d,%d",&a1,&a2,&a3,&a4,&b1,&b2,&b3,&b4) < 8)
         continue;
     }
-    current->start = a1*360000+a2*6000+a3*100.0;
-    current->end   = b1*360000+b2*6000+b3*100.0;
+    current->start = a1*360000+a2*6000+a3*100+a4;
+    current->end   = b1*360000+b2*6000+b3*100+b4;
 
     if (!read_line_from_input(this, line, LINE_LEN))
       return NULL;
@@ -335,21 +343,17 @@ static subtitle_t *sub_read_line_subviewer(demux_sputext_t *this, subtitle_t *cu
 
 static subtitle_t *sub_read_line_subrip(demux_sputext_t *this,subtitle_t *current) {
   char line[LINE_LEN + 1];
-  int a1,a2,b1,b2;
-  float a3,b3;
+  int a1,a2,a3,a4,b1,b2,b3,b4;
   int i,end_sub;
 
   memset(current,0,sizeof(subtitle_t));
   do {
-    char *p;
     if(!read_line_from_input(this,line,LINE_LEN))
       return NULL;
-    while ((p = strchr(line, ',')))
-	*p = '.';
-    i = sscanf(line,"%d:%d:%f --> %d:%d:%f",&a1,&a2,&a3,&b1,&b2,&b3);
-  } while(i < 6);
-  current->start = a1*360000+a2*6000+a3*100.0;
-  current->end   = b1*360000+b2*6000+b3*100.0;
+    i = sscanf(line,"%d:%d:%d%*[,.]%d --> %d:%d:%d%*[,.]%d",&a1,&a2,&a3,&a4,&b1,&b2,&b3,&b4);
+  } while(i < 8);
+  current->start = a1*360000+a2*6000+a3*100+a4/10;
+  current->end   = b1*360000+b2*6000+b3*100+b4/10;
   i=0;
   end_sub=0;
   do {
@@ -417,7 +421,8 @@ static subtitle_t *sub_read_line_vplayer(demux_sputext_t *this,subtitle_t *curre
       if( !read_line_from_input(this, line, LINE_LEN) ) return NULL;
     } else {
       /* ... get the current line from buffer. */
-      strlcpy( line, this->next_line, sizeof(line));
+      strncpy( line, this->next_line, LINE_LEN);
+      line[LINE_LEN] = '\0'; /* I'm scared. This makes me feel better. */
       this->next_line[0] = '\0'; /* mark the buffer as empty. */
     }
     /* Initialize buffer with next line */
@@ -465,8 +470,7 @@ static subtitle_t *sub_read_line_rt(demux_sputext_t *this,subtitle_t *current) {
    * WARNING: full XML parses can be required for proper parsing
    */
   char line[LINE_LEN + 1];
-  int a1,a2,b1,b2;
-  float a3,b3;
+  int a1,a2,a3,a4,b1,b2,b3,b4;
   char *p=NULL,*next=NULL;
   int i,len,plen;
 
@@ -478,13 +482,19 @@ static subtitle_t *sub_read_line_rt(demux_sputext_t *this,subtitle_t *current) {
      * TODO: it seems that format of time is not easily determined, it may be 1:12, 1:12.0 or 0:1:12.0
      * to describe the same moment in time. Maybe there are even more formats in use.
      */
-    if ((len=sscanf (line, "<Time Begin=\"%d:%d:%f\" End=\"%d:%d:%f\"",&a1,&a2,&a3,&b1,&b2,&b3)) < 6)
+    if ((len=sscanf (line, "<Time Begin=\"%d:%d:%d.%d\" End=\"%d:%d:%d.%d\"",&a1,&a2,&a3,&a4,&b1,&b2,&b3,&b4)) < 8)
 
-      plen=a1=a2=a3=b1=b2=b3=0;
-    if ((len=sscanf (line, "<%*[tT]ime %*[bB]egin=\"%d:%f\" %*[Ee]nd=\"%d:%f\"%n",&a2,&a3,&b2,&b3,&plen)) < 4)
+      plen=a1=a2=a3=a4=b1=b2=b3=b4=0;
+    if (
+	((len=sscanf (line, "<%*[tT]ime %*[bB]egin=\"%d:%d\" %*[Ee]nd=\"%d:%d\"%*[^<]<clear/>%n",&a2,&a3,&b2,&b3,&plen)) < 4) &&
+	((len=sscanf (line, "<%*[tT]ime %*[bB]egin=\"%d:%d\" %*[Ee]nd=\"%d:%d.%d\"%*[^<]<clear/>%n",&a2,&a3,&b2,&b3,&b4,&plen)) < 5) &&
+	/*	((len=sscanf (line, "<%*[tT]ime %*[bB]egin=\"%d:%d.%d\" %*[Ee]nd=\"%d:%d\"%*[^<]<clear/>%n",&a2,&a3,&a4,&b2,&b3,&plen)) < 5) && */
+	((len=sscanf (line, "<%*[tT]ime %*[bB]egin=\"%d:%d.%d\" %*[Ee]nd=\"%d:%d.%d\"%*[^<]<clear/>%n",&a2,&a3,&a4,&b2,&b3,&b4,&plen)) < 6) &&
+	((len=sscanf (line, "<%*[tT]ime %*[bB]egin=\"%d:%d:%d.%d\" %*[Ee]nd=\"%d:%d:%d.%d\"%*[^<]<clear/>%n",&a1,&a2,&a3,&a4,&b1,&b2,&b3,&b4,&plen)) < 8)
+	)
       continue;
-    current->start = a1*360000+a2*6000+a3*100.0;
-    current->end   = b1*360000+b2*6000+b3*100.0;
+    current->start = a1*360000+a2*6000+a3*100+a4/10;
+    current->end   = b1*360000+b2*6000+b3*100+b4/10;
     p=line;	p+=plen;i=0;
     /* TODO: I don't know what kind of convention is here for marking multiline subs, maybe <br/> like in xml? */
     next = strstr(line,"<clear/>")+8;i=0;
@@ -508,25 +518,24 @@ static subtitle_t *sub_read_line_ssa(demux_sputext_t *this,subtitle_t *current)
   static int max_comma = 32; /* let's use 32 for the case that the */
   /*  amount of commas increase with newer SSA versions */
 
-  int hour1, min1, hour2, min2, nothing;
-  float sec1, sec2;
+  int hour1, min1, sec1, hunsec1, hour2, min2, sec2, hunsec2, nothing;
   int num;
   char line[LINE_LEN + 1], line3[LINE_LEN + 1], *line2;
   char *tmp;
 
   do {
     if (!read_line_from_input(this, line, LINE_LEN)) return NULL;
-  } while (sscanf (line, "Dialogue: Marked=%d,%d:%d:%f,%d:%d:%f,"
+  } while (sscanf (line, "Dialogue: Marked=%d,%d:%d:%d.%d,%d:%d:%d.%d,"
 		   "%[^\n\r]", &nothing,
-		   &hour1, &min1, &sec1,
-		   &hour2, &min2, &sec2,
-		   line3) < 7
+		   &hour1, &min1, &sec1, &hunsec1,
+		   &hour2, &min2, &sec2, &hunsec2,
+		   line3) < 9
 	   &&
-	   sscanf (line, "Dialogue: %d,%d:%d:%f,%d:%d:%f,"
+	   sscanf (line, "Dialogue: %d,%d:%d:%d.%d,%d:%d:%d.%d,"
 		   "%[^\n\r]", &nothing,
-		   &hour1, &min1, &sec1,
-		   &hour2, &min2, &sec2,
-		   line3) < 7	    );
+		   &hour1, &min1, &sec1, &hunsec1,
+		   &hour2, &min2, &sec2, &hunsec2,
+		   line3) < 9	    );
 
   line2=strchr(line3, ',');
   if (!line2)
@@ -546,8 +555,8 @@ static subtitle_t *sub_read_line_ssa(demux_sputext_t *this,subtitle_t *current)
   if(*line2 == ',') line2++;
 
   current->lines=0;num=0;
-  current->start = 360000*hour1 + 6000*min1 + 100.0*sec1;
-  current->end   = 360000*hour2 + 6000*min2 + 100.0*sec2;
+  current->start = 360000*hour1 + 6000*min1 + 100*sec1 + hunsec1;
+  current->end   = 360000*hour2 + 6000*min2 + 100*sec2 + hunsec2;
 
   while (((tmp=strstr(line2, "\\n")) != NULL) || ((tmp=strstr(line2, "\\N")) != NULL) ){
     current->text[num] = strndup(line2, tmp-line2);
@@ -731,17 +740,17 @@ static subtitle_t *sub_read_line_jacobsub(demux_sputext_t *this, subtitle_t *cur
 				if (sscanf
 				    (&line1[delta], "%*d:%*d:%d",
 				     &seconds)) {
-                                    sscanf(&line1[delta], "%*d:%*d:%*d.%u",
+				    sscanf(&line1[delta], "%*d:%*d:%*d.%d",
 					   &units);
 				} else {
 				    hours = 0;
-                                    sscanf(&line1[delta], "%d:%d.%u",
+				    sscanf(&line1[delta], "%d:%d.%d",
 					   &minutes, &seconds, &units);
 				    minutes *= inverter;
 				}
 			    } else {
 				hours = minutes = 0;
-                                sscanf(&line1[delta], "%d.%u", &seconds,
+				sscanf(&line1[delta], "%d.%d", &seconds,
 				       &units);
 				seconds *= inverter;
 			    }
@@ -867,10 +876,11 @@ static subtitle_t *sub_read_line_jacobsub(demux_sputext_t *this, subtitle_t *cur
 		    if (!read_line_from_input(this, directive, LINE_LEN))
 			return NULL;
 		    trail_space(directive);
-		    strlcat(line2, directive, ((LINE_LEN > 511) ? LINE_LEN-1 : 511));
+		    strncat(line2, directive,
+			    ((LINE_LEN > 511) ? LINE_LEN-1 : 511)
+			    - strlen(line2));
 		    break;
 		}
-                /* fall through */
 	    default:
 		if (!comment) {
 		    *q = *p;
@@ -1095,44 +1105,6 @@ static int sub_autodetect (demux_sputext_t *this) {
   return FORMAT_UNKNOWN;  /* too many bad lines */
 }
 
-static int detect_utf8(subtitle_t *subs, int num_subs)
-{
-  /* return:
-     -1: unknown (ASCII?)
-      0: not valid utf-8
-      1: valid utf-8
-  */
-  int i, l, utf8 = -1;
-
-  for (i = 0; i < num_subs; i++) {
-    for (l = 0; l < subs[i].lines && subs[i].text[l]; l++) {
-      const uint8_t *c = subs[i].text[l];
-      for (; *c; c++) {
-        if (*c & 0x80) {
-          if ( (c[0]>=0xC2 && c[0]<=0xDF) && (c[1]>=0x80 && c[1]<=0xBF) ) {
-            /* valid 2-byte */
-            utf8 = 1;
-            c++;
-          } else if ( ( c[0]==0xE0 && (c[1]>=0xA0 && c[1]<=0xBF) && (c[2]>=0x80 && c[1]<=0xBF)) ||
-                      ( (c[0]>=0xE1 && c[0]<=0xEC) && (c[1]>=0x80 && c[1]<=0xBF) && (c[2]>=0x80 && c[1]<=0xBF)) ||
-                      ( c[0]==0xED && (c[1]>=0x80 && c[1]<=0x9F) && (c[2]>=0x80 && c[1]<=0xBF))  ||
-                      ( c[0]==0xEE && (c[1]>=0xA4 && c[1]<=0xBF) && (c[2]>=0x80 && c[1]<=0xBF) ) ||
-                      ( c[0]==0xEF && (c[1]>=0xA4 && c[1]<=0xBF) && (c[2]>=0x80 && c[1]<=0xBF) )) {
-            /* valid 3-byte */
-            utf8 = 1;
-            c += 2;
-          } else {
-            /* TODO: 4-byte not checked */
-            return 0;
-          }
-        }
-      }
-    }
-  }
-
-  return utf8;
-}
-
 static subtitle_t *sub_read_file (demux_sputext_t *this) {
 
   int n_max;
@@ -1181,13 +1153,8 @@ static subtitle_t *sub_read_file (demux_sputext_t *this) {
   this->num=0;n_max=32;
   first = calloc(n_max, sizeof(subtitle_t));
   if(!first) return NULL;
-
-  {
-    cfg_entry_t *entry;
-    entry = this->stream->xine->config->lookup_entry (this->stream->xine->config,
-                                                      "subtitles.separate.timeout");
-    timeout = entry ? entry->num_value : 4;
-  }
+  timeout = ((demux_sputext_class_t *)
+             (this->demux_plugin.demux_class))->max_timeout;
   if (this->uses_time) timeout *= 100;
   else timeout *= 10;
 
@@ -1229,11 +1196,6 @@ static subtitle_t *sub_read_file (demux_sputext_t *this) {
       first[this->num-1].end = first[this->num-1].start + timeout;
     }
 
-  if (detect_utf8(first, this->num) > 0) {
-    xprintf (this->stream->xine, XINE_VERBOSITY_LOG, "detected utf-8 subtitles\n");
-    this->encoding = strdup("utf-8");
-  }
-
   if(this->stream->xine->verbosity >= XINE_VERBOSITY_DEBUG) {
     char buffer[1024];
 
@@ -1274,14 +1236,8 @@ static int demux_sputext_next (demux_sputext_t *this_gen) {
   *val++ = (this->uses_time) ? sub->end * 10 : sub->end;
   str = (char *)val;
   for (line = 0; line < sub->lines; line++, str+=strlen(str)+1) {
-    strlcpy(str, sub->text[line], SUB_BUFSIZE);
-  }
-
-  if (this->encoding) {
-    buf->decoder_flags |= BUF_FLAG_SPECIAL;
-    buf->decoder_info[1] = BUF_SPECIAL_CHARSET_ENCODING;
-    buf->decoder_info_ptr[2] = this->encoding;
-    buf->decoder_info[2] = strlen(buf->decoder_info_ptr[2]);
+    strncpy(str, sub->text[line], SUB_BUFSIZE-1);
+    str[SUB_BUFSIZE-1] = '\0';
   }
 
   this->stream->video_fifo->put(this->stream->video_fifo, buf);
@@ -1296,10 +1252,9 @@ static void demux_sputext_dispose (demux_plugin_t *this_gen) {
 
   for (i = 0; i < this->num; i++) {
     for (l = 0; l < this->subtitles[i].lines; l++)
-      _x_freep(&this->subtitles[i].text[l]);
+      free(this->subtitles[i].text[l]);
   }
-  _x_freep(&this->subtitles);
-  _x_freep(&this->encoding);
+  free(this->subtitles);
   free(this);
 }
 
@@ -1334,10 +1289,6 @@ static int demux_sputext_seek (demux_plugin_t *this_gen,
 
   lprintf("seek() called\n");
 
-  (void)start_pos;
-  (void)start_time;
-  (void)playing;
-
   /* simple seeking approach: just go back to start.
    * decoder will discard subtitles until the desired position.
    */
@@ -1371,7 +1322,6 @@ static void demux_sputext_send_headers(demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_sputext_get_capabilities(demux_plugin_t *this_gen) {
-  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
@@ -1379,15 +1329,12 @@ static int demux_sputext_get_optional_data(demux_plugin_t *this_gen,
 					   void *data, int data_type) {
   int channel = *((int *)data);
 
-  (void)this_gen;
-
   switch (data_type) {
   case DEMUX_OPTIONAL_DATA_SPULANG:
     if (channel == -1 || channel == 0) {
       strcpy(data, "sub");
       return DEMUX_OPTIONAL_SUCCESS;
     }
-    /* fall through */
   default:
     return DEMUX_OPTIONAL_UNSUPPORTED;
   }
@@ -1402,9 +1349,6 @@ static demux_plugin_t *open_demux_plugin (demux_class_t *class_gen, xine_stream_
   lprintf("open_plugin() called\n");
 
   this = calloc(1, sizeof (demux_sputext_t));
-  if (!this)
-    return NULL;
-
   this->stream = stream;
   this->input  = input;
 
@@ -1438,7 +1382,7 @@ static demux_plugin_t *open_demux_plugin (demux_class_t *class_gen, xine_stream_
         return NULL;
       }
     }
-    /* fall through */
+    /* falling through is intended */
 
   case METHOD_EXPLICIT:
   /* case METHOD_BY_CONTENT: */
@@ -1470,35 +1414,41 @@ static demux_plugin_t *open_demux_plugin (demux_class_t *class_gen, xine_stream_
   return NULL;
 }
 
-void *init_sputext_demux_class (xine_t *xine, const void *data) {
+static void config_timeout_cb(void *this_gen, xine_cfg_entry_t *entry) {
+  demux_sputext_class_t *this = (demux_sputext_class_t *)this_gen;
 
-  static const demux_class_t demux_class = {
-    .open_plugin     = open_demux_plugin,
-    .description     = N_("sputext demuxer plugin"),
-    .identifier      = "sputext",
-    /* do not report this mimetype, it might confuse browsers. */
-    /* "text/plain: asc txt sub srt: VIDEO subtitles;" */
-    .mimetypes       = NULL,
-    .extensions      = "asc txt sub srt smi ssa ass",
-    .dispose         = NULL,
-  };
-  config_values_t *config = xine->config;
+  this->max_timeout = entry->num_value;
+}
 
-  (void)xine;
-  (void)data;
+void *init_sputext_demux_class (xine_t *xine, void *data) {
+
+  demux_sputext_class_t *this ;
+
+  lprintf("initializing\n");
+
+  this = calloc(1, sizeof (demux_sputext_class_t));
+
+  this->demux_class.open_plugin     = open_demux_plugin;
+  this->demux_class.description     = N_("sputext demuxer plugin");
+  this->demux_class.identifier      = "sputext";
+  /* do not report this mimetype, it might confuse browsers. */
+  /* "text/plain: asc txt sub srt: VIDEO subtitles;" */
+  this->demux_class.mimetypes       = NULL;
+  this->demux_class.extensions      = "asc txt sub srt smi ssa ass";
+  this->demux_class.dispose         = default_demux_class_dispose;
 
   /*
    * Some subtitling formats, namely AQT and Subrip09, define the end of a
    * subtitle as the beginning of the following. From end-user view it's
    * better define timeout of hidding. Setting to zero means "no timeout".
    */
-  config->register_num(config,
-                       "subtitles.separate.timeout", 4,
-                       _("default duration of subtitle display in seconds"),
-                       _("Some subtitle formats do not explicitly give a duration for each subtitle. "
-                         "For these, you can set a default duration here. Setting to zero will result "
-                         "in the subtitle being shown until the next one takes over."),
-                       20, NULL, NULL);
+  this->max_timeout = xine->config->register_num(xine->config,
+                         "subtitles.separate.timeout", 4,
+			 _("default duration of subtitle display in seconds"),
+			 _("Some subtitle formats do not explicitly give a duration for each subtitle. "
+			   "For these, you can set a default duration here. Setting to zero will result "
+			   "in the subtitle being shown until the next one takes over."),
+			 20, config_timeout_cb, this);
 
-  return (void *)&demux_class;
+  return this;
 }
