@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2019 the xine project
+ * Copyright (C) 2000-2014 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -47,7 +47,11 @@
 #include <ctype.h>
 #include <pthread.h>
 
-
+#ifdef HAVE_FFMPEG_AVUTIL_H
+#  include <mem.h>
+#else
+#  include <libavutil/mem.h>
+#endif
 
 /* defines for debugging extensions only */
 /* #define GL_GLEXT_LEGACY */
@@ -132,7 +136,8 @@
 #  define YUV_SWAP_MODE      0
 #endif
 
-#define MY_2PI               (M_PI * 2)
+#define MY_PI                3.1415926
+#define MY_2PI               6.2831853
 
 
 typedef struct {
@@ -207,7 +212,6 @@ typedef struct {
 
   /* color matrix switching */
   int                cm_yuv2rgb, cm_fragprog, cm_state;
-  uint8_t            cm_lut[32];
 
   /* Frame state */
   opengl_frame_t    *frame[NUM_FRAMES_BACKLOG];
@@ -246,20 +250,10 @@ typedef struct {
 
 
 /* import common color matrix stuff */
-#define CM_LUT
 #define CM_DRIVER_T opengl_driver_t
 #include "color_matrix.c"
 
-static const int32_t Inverse_Table_6_9[8][4] = {
-  {117504, 138453, 13954, 34903}, /* no sequence_display_extension */
-  {117504, 138453, 13954, 34903}, /* ITU-R Rec. 709 (1990) */
-  {104597, 132201, 25675, 53279}, /* unspecified */
-  {104597, 132201, 25675, 53279}, /* reserved */
-  {104448, 132798, 24759, 53109}, /* FCC */
-  {104597, 132201, 25675, 53279}, /* ITU-R Rec. 624-4 System B, G */
-  {104597, 132201, 25675, 53279}, /* SMPTE 170M */
-  {117579, 136230, 16907, 35559}  /* SMPTE 240M (1987) */
-};
+extern const int32_t Inverse_Table_6_9[8][4]; /* from yuv2rgb.c */
 
 /*
  * Render functions
@@ -550,8 +544,6 @@ static int render_help_image_tiledtex (opengl_driver_t *this,
 }
 
 static int render_image_nop (opengl_driver_t *this, opengl_frame_t *frame) {
-  (void)this;
-  (void)frame;
   return 1;
 }
 
@@ -633,7 +625,7 @@ static int render_image_fp_yuv (opengl_driver_t *this, opengl_frame_t *frame) {
   if (! ret)
     return 0;
   if (ret == 1) {
-    uint8_t *tmp = calloc (this->tex_width * this->tex_height, 1);
+    char *tmp = calloc (this->tex_width * this->tex_height, 1);
     for (i = 0; i < frame->width+3; i++) {
       tmp[this->tex_width*(frame->height+1)+i] = 128;
       tmp[this->tex_width*(frame->height+h2+2)+i] = 128;
@@ -679,8 +671,8 @@ static int render_image_envtex (opengl_driver_t *this, opengl_frame_t *frame) {
       ! this->tex_width || ! this->tex_height) {
     ret = render_image_tex (this, frame);
     /* Texture matrix has to skale/shift tex origin + swap y coords */
-    mTex[0]  =  this->tex_width  ? ( 1.0 * frame->width  / this->tex_width)  :  1.0;
-    mTex[5]  =  this->tex_height ? (-1.0 * frame->height / this->tex_height) : -1.0;
+    mTex[0]  =   1.0 * frame->width  / this->tex_width;
+    mTex[5]  =  -1.0 * frame->height / this->tex_height;
     mTex[12] = (-2.0 * mTex[0]) / mTex[0];
     mTex[13] =  -mTex[5];
     glMatrixMode  (GL_TEXTURE);
@@ -699,7 +691,7 @@ static int render_help_verify_ext (opengl_driver_t *this, const char *ext) {
   int ret = 0;
   const size_t l = strlen (ext);
   const char *e;
-  for (e = (const char *) this->gl_exts; e && *e; e = strchr (e, ' ')) {
+  for (e = (char *) this->gl_exts; e && *e; e = strchr (e, ' ')) {
     while (isspace (*e))
       e++;
     if (strncmp (e, ext, l) == 0 && (e[l] == 0 || e[l] == ' ')) {
@@ -811,7 +803,6 @@ static void render_help_check_exts (opengl_driver_t *this) {
 }
 
 static int render_help_setup_tex (opengl_driver_t *this) {
-  (void)this;
   CHECKERR ("pre-tex_setup");
   glEnable        (GL_TEXTURE_2D);
   glTexEnvi       (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE,   GL_REPLACE);
@@ -1019,7 +1010,7 @@ static int render_setup_fp_yuv (opengl_driver_t *this) {
     return 0;
 
   xprintf (this->xine, XINE_VERBOSITY_LOG,
-    "video_out_opengl_fragprog: b %d c %d s %d [%s]\n",
+    "video_out_open_opengl_fragprog: b %d c %d s %d [%s]\n",
     this->brightness, this->contrast, this->saturation, cm_names[this->cm_fragprog]);
 
   if (this->fprog == (GLuint)-1)
@@ -1071,10 +1062,6 @@ static void render_gfx_vinfo (opengl_driver_t *this) {
     glxAttrib[9] = GLX_DOUBLEBUFFER;
   else
     glxAttrib[9] = None;
-
-  if (this->vinfo)
-    XFree(this->vinfo);
-
   this->vinfo = glXChooseVisual (this->display, this->screen, glxAttrib);
   CHECKERR ("choosevisual");
 }
@@ -1131,6 +1118,7 @@ static void *render_run (opengl_driver_t *this) {
     case RENDER_DRAW:
       this->render_action = RENDER_NONE;
       this->render_frame_changed = 0;
+      pthread_mutex_unlock (&this->render_action_mutex);
       if (this->context && frame) {
 	/* update fragprog if color matrix changed */
 	if (this->render_fun_id == 0) {
@@ -1138,11 +1126,9 @@ static void *render_run (opengl_driver_t *this) {
 	  if (cm != this->cm_fragprog) {
 	    this->cm_fragprog = cm;
 	    this->render_action = RENDER_SETUP;
-            pthread_mutex_unlock (&this->render_action_mutex);
 	    break;
 	  }
 	}
-        pthread_mutex_unlock (&this->render_action_mutex);
 	XLockDisplay (this->display);
 	CHECKERR ("pre-render");
 	ret = 1;
@@ -1163,8 +1149,6 @@ static void *render_run (opengl_driver_t *this) {
 	  if (render->fallback != -1 && this->gl_exts)
 	    this->config->update_num (this->config, "video.output.opengl_renderer", render->fallback);
 	}
-      } else {
-        pthread_mutex_unlock (&this->render_action_mutex);
       }
       break;
 
@@ -1303,7 +1287,6 @@ static uint32_t opengl_get_capabilities (vo_driver_t *this_gen) {
 /*   if (this->xoverlay) */
 /*     capabilities |= VO_CAP_UNSCALED_OVERLAY; */
 
-  (void)this_gen;
   return capabilities;
 }
 
@@ -1378,11 +1361,10 @@ static void opengl_frame_dispose (vo_frame_t *vo_img) {
 
   frame->yuv2rgb->dispose (frame->yuv2rgb);
 
-  xine_free_aligned (frame->vo_frame.base[0]);
-  xine_free_aligned (frame->vo_frame.base[1]);
-  xine_free_aligned (frame->vo_frame.base[2]);
-  xine_free_aligned (frame->rgb);
-  pthread_mutex_destroy (&frame->vo_frame.mutex);
+  av_free (frame->vo_frame.base[0]);
+  av_free (frame->vo_frame.base[1]);
+  av_free (frame->vo_frame.base[2]);
+  av_free (frame->rgb);
   free (frame);
 }
 
@@ -1395,15 +1377,6 @@ static vo_frame_t *opengl_alloc_frame (vo_driver_t *this_gen) {
   if (!frame)
     return NULL;
 
-  /*
-   * colorspace converter for this frame
-   */
-  frame->yuv2rgb = this->yuv2rgb_factory->create_converter (this->yuv2rgb_factory);
-  if (!frame->yuv2rgb) {
-    free(frame);
-    return NULL;
-  }
-
   pthread_mutex_init (&frame->vo_frame.mutex, NULL);
 
   /*
@@ -1415,6 +1388,11 @@ static vo_frame_t *opengl_alloc_frame (vo_driver_t *this_gen) {
   frame->vo_frame.dispose    = opengl_frame_dispose;
   frame->vo_frame.driver     = this_gen;
 
+  /*
+   * colorspace converter for this frame
+   */
+  frame->yuv2rgb = this->yuv2rgb_factory->create_converter (this->yuv2rgb_factory);
+
   return (vo_frame_t *) frame;
 }
 
@@ -1434,7 +1412,6 @@ static void opengl_update_frame_format (vo_driver_t *this_gen,
   opengl_frame_t   *frame = (opengl_frame_t *) frame_gen;
   int     g_width, g_height;
   double  g_pixel_aspect;
-  int     changed = 0;
 
   /* Check output size to signal render thread output size changes */
   this->sc.dest_size_cb (this->sc.user_data, width, height,
@@ -1454,45 +1431,34 @@ static void opengl_update_frame_format (vo_driver_t *this_gen,
   }
 
   /* Check frame size and format and reallocate if necessary */
-  if ((frame->width != (int)width) || (frame->height != (int)height) || (frame->format != format)) {
+  if ((frame->width != width)
+      || (frame->height != height)
+      || (frame->format != format)
+      || (frame->flags  != flags)) {
 /*     lprintf ("updating frame to %d x %d (ratio=%g, format=%08x)\n", width, height, ratio, format); */
 
-    /* make sure we dont pull the rug from under the renderer */
+    flags &= VO_BOTH_FIELDS;
+
     XLockDisplay (this->display);
 
     /* (re-) allocate render space */
-    xine_freep_aligned(&frame->vo_frame.base[0]);
-    xine_freep_aligned(&frame->vo_frame.base[1]);
-    xine_freep_aligned(&frame->vo_frame.base[2]);
-    xine_freep_aligned(&frame->rgb);
+    av_freep(&frame->vo_frame.base[0]);
+    av_freep(&frame->vo_frame.base[1]);
+    av_freep(&frame->vo_frame.base[2]);
+    av_freep(&frame->rgb);
 
     if (format == XINE_IMGFMT_YV12) {
       frame->vo_frame.pitches[0] = 8*((width + 7) / 8);
       frame->vo_frame.pitches[1] = 8*((width + 15) / 16);
       frame->vo_frame.pitches[2] = 8*((width + 15) / 16);
-      frame->vo_frame.base[0] = xine_mallocz_aligned(frame->vo_frame.pitches[0] * height);
-      frame->vo_frame.base[1] = xine_mallocz_aligned(frame->vo_frame.pitches[1] * ((height+1)/2));
-      frame->vo_frame.base[2] = xine_mallocz_aligned(frame->vo_frame.pitches[2] * ((height+1)/2));
+      frame->vo_frame.base[0] = av_mallocz(frame->vo_frame.pitches[0] * height);
+      frame->vo_frame.base[1] = av_mallocz(frame->vo_frame.pitches[1] * ((height+1)/2));
+      frame->vo_frame.base[2] = av_mallocz(frame->vo_frame.pitches[2] * ((height+1)/2));
     } else {
       frame->vo_frame.pitches[0] = 8*((width + 3) / 4);
-      frame->vo_frame.base[0] = xine_mallocz_aligned(frame->vo_frame.pitches[0] * height);
+      frame->vo_frame.base[0] = av_mallocz(frame->vo_frame.pitches[0] * height);
     }
-    frame->rgb = xine_mallocz_aligned(BYTES_PER_PIXEL*width*height);
-
-    changed = 1;
-
-  } else if ((frame->flags ^ flags) & VO_BOTH_FIELDS) {
-
-    XLockDisplay (this->display);
-    changed = 1;
-
-  }
-
-  frame->flags = flags;
-
-  if (changed) {
-
-    flags &= VO_BOTH_FIELDS;
+    frame->rgb = av_mallocz(BYTES_PER_PIXEL*width*height);
 
     /* set up colorspace converter */
     switch (flags) {
@@ -1535,24 +1501,23 @@ static void opengl_update_frame_format (vo_driver_t *this_gen,
 
 static void opengl_overlay_clut_yuv2rgb(opengl_driver_t  *this, vo_overlay_t *overlay,
 				      opengl_frame_t *frame) {
-  int i;
-  uint32_t *rgb;
+  int     i;
+  clut_t* clut = (clut_t*) overlay->color;
 
-  (void)this;
   if (!overlay->rgb_clut) {
-    rgb = overlay->color;
-    for (i = sizeof (overlay->color) / sizeof (overlay->color[0]); i > 0; i--) {
-      clut_t *yuv = (clut_t *)rgb;
-      *rgb++ = frame->yuv2rgb->yuv2rgb_single_pixel_fun (frame->yuv2rgb, yuv->y, yuv->cb, yuv->cr);
+    for (i = 0; i < sizeof(overlay->color)/sizeof(overlay->color[0]); i++) {
+      *((uint32_t *)&clut[i]) =
+	frame->yuv2rgb->yuv2rgb_single_pixel_fun (frame->yuv2rgb, clut[i].y,
+						  clut[i].cb, clut[i].cr);
     }
     overlay->rgb_clut++;
   }
-
   if (!overlay->hili_rgb_clut) {
-    rgb = overlay->hili_color;
-    for (i = sizeof (overlay->color) / sizeof (overlay->color[0]); i > 0; i--) {
-      clut_t *yuv = (clut_t *)rgb;
-      *rgb++ = frame->yuv2rgb->yuv2rgb_single_pixel_fun (frame->yuv2rgb, yuv->y, yuv->cb, yuv->cr);
+    clut = (clut_t*) overlay->hili_color;
+    for (i = 0; i < sizeof(overlay->color)/sizeof(overlay->color[0]); i++) {
+      *((uint32_t *)&clut[i]) =
+	frame->yuv2rgb->yuv2rgb_single_pixel_fun(frame->yuv2rgb, clut[i].y,
+						 clut[i].cb, clut[i].cr);
     }
     overlay->hili_rgb_clut++;
   }
@@ -1577,7 +1542,6 @@ static void opengl_overlay_begin (vo_driver_t *this_gen,
 static void opengl_overlay_end (vo_driver_t *this_gen, vo_frame_t *vo_img) {
   opengl_driver_t  *this  = (opengl_driver_t *) this_gen;
 
-  (void)vo_img;
   if (this->ovl_changed && this->xoverlay) {
     XLockDisplay (this->display);
     x11osd_expose(this->xoverlay);
@@ -1744,20 +1708,6 @@ static int opengl_set_property (vo_driver_t *this_gen,
   opengl_driver_t *this = (opengl_driver_t *) this_gen;
 
   switch (property) {
-
-  case VO_PROP_DISCARD_FRAMES:
-    if (value == -1) {
-      int i, n = 0;
-      for (i = NUM_FRAMES_BACKLOG - 1; i >= 0; i--) {
-        if (this->frame[i]) {
-          this->frame[i]->vo_frame.free (&this->frame[i]->vo_frame);
-          this->frame[i] = NULL;
-          n++;
-        }
-      }
-      value = n;
-    }
-    break;
   case VO_PROP_ASPECT_RATIO:
     if (value>=XINE_VO_ASPECT_NUM_RATIOS)
       value = XINE_VO_ASPECT_AUTO;
@@ -1798,7 +1748,6 @@ static void opengl_get_property_min_max (vo_driver_t *this_gen,
 				     int property, int *min, int *max) {
   /* opengl_driver_t *this = (opengl_driver_t *) this_gen;  */
 
-  (void)this_gen;
   switch (property) {
   case VO_PROP_BRIGHTNESS:
     *min = -128;    *max = 127;     break;
@@ -1914,16 +1863,15 @@ static int opengl_gui_data_exchange (vo_driver_t *this_gen,
   return 0;
 }
 
-static void opengl_dispose_internal (opengl_driver_t *this, int thread_running) {
+static void opengl_dispose (vo_driver_t *this_gen) {
+  opengl_driver_t *this = (opengl_driver_t *) this_gen;
   int i;
 
-  if (thread_running) {
-    pthread_mutex_lock    (&this->render_action_mutex);
-    this->render_action = RENDER_EXIT;
-    pthread_cond_signal   (&this->render_action_cond);
-    pthread_mutex_unlock  (&this->render_action_mutex);
-    pthread_join          (this->render_thread, NULL);
-  }
+  pthread_mutex_lock    (&this->render_action_mutex);
+  this->render_action = RENDER_EXIT;
+  pthread_cond_signal   (&this->render_action_cond);
+  pthread_mutex_unlock  (&this->render_action_mutex);
+  pthread_join          (this->render_thread, NULL);
   pthread_mutex_destroy (&this->render_action_mutex);
   pthread_cond_destroy  (&this->render_action_cond);
   pthread_cond_destroy  (&this->render_return_cond);
@@ -1942,21 +1890,11 @@ static void opengl_dispose_internal (opengl_driver_t *this, int thread_running)
     XUnlockDisplay (this->display);
   }
 
-  if (this->vinfo) {
-    XFree(this->vinfo);
-  }
-
   _x_alphablend_free(&this->alphablend_extra_data);
-  _x_vo_scale_cleanup (&this->sc, this->xine->config);
 
   free (this);
 }
 
-static void opengl_dispose (vo_driver_t *this_gen) {
-  opengl_driver_t *this = (opengl_driver_t *) this_gen;
-  opengl_dispose_internal(this, 1);
-}
-
 static void opengl_cb_render_fun (void *this_gen, xine_cfg_entry_t *entry) {
   opengl_driver_t *this = (opengl_driver_t *) this_gen;
   pthread_mutex_lock (&this->render_action_mutex);
@@ -1976,23 +1914,16 @@ static void opengl_cb_default (void *val_gen, xine_cfg_entry_t *entry) {
 static vo_driver_t *opengl_open_plugin (video_driver_class_t *class_gen, const void *visual_gen) {
   opengl_class_t       *class   = (opengl_class_t *) class_gen;
   config_values_t      *config  = class->xine->config;
-  const x11_visual_t   *visual  = (const x11_visual_t *) visual_gen;
+  x11_visual_t         *visual  = (x11_visual_t *) visual_gen;
   opengl_driver_t      *this;
   const char          **render_fun_names;
-  unsigned int          i;
+  int                   i;
 
   this = (opengl_driver_t *) calloc(1, sizeof(opengl_driver_t));
 
   if (!this)
     return NULL;
 
-  this->yuv2rgb_factory = yuv2rgb_factory_init (YUV_FORMAT, YUV_SWAP_MODE, NULL);
-  if (!this->yuv2rgb_factory) {
-    xprintf (this->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": yuv2rgb initialization failed\n");
-    free(this);
-    return NULL;
-  }
-
   this->display		    = visual->display;
   this->screen		    = visual->screen;
 
@@ -2034,6 +1965,8 @@ static vo_driver_t *opengl_open_plugin (video_driver_class_t *class_gen, const v
 
   cm_init (this);
 
+  this->yuv2rgb_factory = yuv2rgb_factory_init (YUV_FORMAT, YUV_SWAP_MODE, NULL);
+
   XLockDisplay (this->display);
   this->xoverlay = x11osd_create (this->xine, this->display, this->screen,
                                   this->drawable, X11OSD_SHAPED);
@@ -2065,7 +1998,6 @@ static vo_driver_t *opengl_open_plugin (video_driver_class_t *class_gen, const v
 						 "Environment_Mapped_Torus\n"
 						 "Show images reflected in a spinning torus. Way cool =)"),
 					       10, opengl_cb_render_fun, this);
-  free(render_fun_names);
   this->render_min_fps = config->register_range (config,
 						 "video.output.opengl_min_fps",
 						 20, 1, 120,
@@ -2084,11 +2016,7 @@ static vo_driver_t *opengl_open_plugin (video_driver_class_t *class_gen, const v
   pthread_mutex_init (&this->render_action_mutex, NULL);
   pthread_cond_init  (&this->render_action_cond, NULL);
   pthread_cond_init  (&this->render_return_cond, NULL);
-  if (pthread_create (&this->render_thread, NULL, (thread_run_t) render_run, this)) {
-    xprintf (this->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": pthread_create() failed\n");
-    opengl_dispose_internal (this, 0);
-    return NULL;
-  }
+  pthread_create (&this->render_thread, NULL, (thread_run_t) render_run, this);
 
   /* Check for OpenGL capable visual */
   pthread_mutex_lock   (&this->render_action_mutex);
@@ -2107,7 +2035,7 @@ static vo_driver_t *opengl_open_plugin (video_driver_class_t *class_gen, const v
 
   if (! this->vinfo) {
     /* no OpenGL capable visual available */
-    opengl_dispose_internal (this, 1);
+    opengl_dispose (&this->vo_driver);
     return NULL;
   }
   if (! this->context)
@@ -2143,10 +2071,8 @@ static int opengl_verify_direct (x11_visual_t *vis) {
   }
   if (! (visinfo = glXChooseVisual (vis->display, vis->screen, attribs)))
       return 0;
-  if (! (ctx = glXCreateContext (vis->display, visinfo, NULL, 1))) {
-      XFree(visinfo);
+  if (! (ctx = glXCreateContext (vis->display, visinfo, NULL, 1)))
       return 0;
-  }
   memset (&xattr, 0, sizeof (xattr));
   xattr.colormap = XCreateColormap(vis->display, root, visinfo->visual, AllocNone);
   xattr.event_mask = StructureNotifyMask | ExposureMask;
@@ -2166,12 +2092,11 @@ static int opengl_verify_direct (x11_visual_t *vis) {
   }
   glXDestroyContext (vis->display, ctx);
   XFreeColormap     (vis->display, xattr.colormap);
-  XFree             (visinfo);
 
   return ret;
 }
 
-static void *opengl_init_class (xine_t *xine, const void *visual_gen) {
+static void *opengl_init_class (xine_t *xine, void *visual_gen) {
 
   opengl_class_t *this;
 
@@ -2183,9 +2108,7 @@ static void *opengl_init_class (xine_t *xine, const void *visual_gen) {
       return NULL;
   }
 
-  this = calloc (1, sizeof(*this));
-  if (!this)
-    return NULL;
+  this = (opengl_class_t *) calloc (1, sizeof(opengl_class_t));
 
   this->driver_class.open_plugin     = opengl_open_plugin;
   this->driver_class.identifier      = "opengl";
@@ -2198,8 +2121,8 @@ static void *opengl_init_class (xine_t *xine, const void *visual_gen) {
 
 
 static const vo_info_t vo_info_opengl = {
-  .priority    = 7,
-  .visual_type = XINE_VISUAL_TYPE_X11,
+  7,                    /* priority    */
+  XINE_VISUAL_TYPE_X11  /* visual type */
 };
 
 
@@ -2210,5 +2133,5 @@ static const vo_info_t vo_info_opengl = {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_OUT, 22, "opengl", XINE_VERSION_CODE, &vo_info_opengl, opengl_init_class },
-  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
+  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
