@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004, 2008 the xine project
+ * Copyright (C) 2000-2014 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -83,6 +83,8 @@ typedef struct {
   int                max;
   Atom               atom;
 
+  int                defer;
+
   cfg_entry_t       *entry;
 
   xv_driver_t       *this;
@@ -102,6 +104,7 @@ typedef struct {
   XvImage           *image;
   XShmSegmentInfo    shminfo;
 
+  int                req_width, req_height;
 } xv_frame_t;
 
 
@@ -155,6 +158,10 @@ struct xv_driver_s {
 
   void              *user_data;
 
+  /* color matrix switching */
+  int                cm_active, cm_state;
+  Atom               cm_atom;
+  int                fullrange_mode;
 };
 
 typedef struct {
@@ -164,6 +171,10 @@ typedef struct {
   xine_t              *xine;
 } xv_class_t;
 
+/* import common color matrix stuff */
+#define CM_DRIVER_T xv_driver_t
+#include "color_matrix.c"
+
 static int gX11Fail;
 
 VIDEO_DEVICE_XV_DECL_BICUBIC_TYPES;
@@ -443,8 +454,8 @@ static void xv_update_frame_format (vo_driver_t *this_gen,
     width = (width + 7) & ~0x7;
   }
 
-  if ((frame->width != width)
-      || (frame->height != height)
+  if ((frame->req_width != width)
+      || (frame->req_height != height)
       || (frame->format != format)) {
 
     /* printf (LOG_MODULE ": updating frame to %d x %d (ratio=%d, format=%08x)\n",width,height,ratio_code,format); */
@@ -475,13 +486,21 @@ static void xv_update_frame_format (vo_driver_t *this_gen,
       frame->vo_frame.base[2] = frame->image->data + frame->image->offsets[1];
     }
 
-    frame->width  = width;
-    frame->height = height;
+    /* allocated frame size may not match requested size */
+    frame->req_width  = width;
+    frame->req_height = height;
+    frame->width  = frame->image->width;
+    frame->height = frame->image->height;
     frame->format = format;
 
     UNLOCK_DISPLAY(this);
   }
 
+  if (frame->vo_frame.width > frame->width)
+    frame->vo_frame.width  = frame->width;
+  if (frame->vo_frame.height > frame->height)
+    frame->vo_frame.height = frame->height;
+
   frame->ratio = ratio;
 }
 
@@ -617,7 +636,7 @@ static void xv_flush_recent_frames (xv_driver_t *this) {
 
 static int xv_redraw_needed (vo_driver_t *this_gen) {
   xv_driver_t  *this = (xv_driver_t *) this_gen;
-  int           ret  = 0;
+  int           ret  = !this->cm_active;
 
   if( this->cur_frame ) {
 
@@ -657,13 +676,76 @@ static double timeOfDay()
     return ((double)t.tv_sec) + (((double)t.tv_usec)/1000000.0);
 }
 
+static void xv_new_color (xv_driver_t *this, int cm) {
+  int brig = this->props[VO_PROP_BRIGHTNESS].value;
+  int cont = this->props[VO_PROP_CONTRAST].value;
+  int satu = this->props[VO_PROP_SATURATION].value;
+  int cm2, fr = 0, a, b;
+  Atom atom;
+
+  if (cm & 1) {
+    /* fullrange emulation. Do not report this via get_property (). */
+    if (this->fullrange_mode == 1) {
+      /* modification routine 1 for TV set style bcs controls 0% - 200% */
+      satu -= this->props[VO_PROP_SATURATION].min;
+      satu  = (satu * (112 * 255) + (127 * 219 / 2)) / (127 * 219);
+      satu += this->props[VO_PROP_SATURATION].min;
+      if (satu > this->props[VO_PROP_SATURATION].max)
+        satu = this->props[VO_PROP_SATURATION].max;
+      cont -= this->props[VO_PROP_CONTRAST].min;
+      cont  = (cont * 219 + 127) / 255;
+      a     = cont * (this->props[VO_PROP_BRIGHTNESS].max - this->props[VO_PROP_BRIGHTNESS].min);
+      cont += this->props[VO_PROP_CONTRAST].min;
+      b     = 256 * (this->props[VO_PROP_CONTRAST].max - this->props[VO_PROP_CONTRAST].min);
+      brig += (16 * a + b / 2) / b;
+      if (brig > this->props[VO_PROP_BRIGHTNESS].max)
+        brig = this->props[VO_PROP_BRIGHTNESS].max;
+      fr = 1;
+    }
+    /* maybe add more routines for non-standard controls later */
+  }
+  LOCK_DISPLAY (this);
+  atom = this->props[VO_PROP_BRIGHTNESS].atom;
+  if (atom != None)
+    XvSetPortAttribute (this->display, this->xv_port, atom, brig);
+  atom = this->props[VO_PROP_CONTRAST].atom;
+  if (atom != None)
+    XvSetPortAttribute (this->display, this->xv_port, atom, cont);
+  atom = this->props[VO_PROP_SATURATION].atom;
+  if (atom != None)
+    XvSetPortAttribute (this->display, this->xv_port, atom, satu);
+  UNLOCK_DISPLAY(this);
+
+  /* so far only binary nvidia drivers support this. why not nuveau? */
+  if (this->cm_atom != None) {
+    cm2 = (0xc00c >> cm) & 1;
+    LOCK_DISPLAY(this);
+    XvSetPortAttribute (this->display, this->xv_port, this->cm_atom, cm2);
+    UNLOCK_DISPLAY(this);
+    cm2 = cm2 ? 2 : 10;
+  } else {
+    cm2 = 10;
+  }
+
+  cm2 |= fr;
+  xprintf (this->xine, XINE_VERBOSITY_LOG, "video_out_xv: %s b %d  c %d  s %d  [%s]\n",
+    fr ? "modified " : "", brig, cont, satu, cm_names[cm2]);
+
+  this->cm_active = cm;
+}
+
 static void xv_display_frame (vo_driver_t *this_gen, vo_frame_t *frame_gen) {
   xv_driver_t  *this  = (xv_driver_t *) this_gen;
   xv_frame_t   *frame = (xv_frame_t *) frame_gen;
+  int cm;
   /*
   printf (LOG_MODULE ": xv_display_frame...\n");
   */
 
+  cm = cm_from_frame (frame_gen);
+  if (cm != this->cm_active)
+    xv_new_color (this, cm);
+
   /*
    * queue frames (deinterlacing)
    * free old frames
@@ -788,6 +870,15 @@ static int xv_set_property (vo_driver_t *this_gen,
 
   if ((property < 0) || (property >= VO_NUM_PROPERTIES)) return 0;
 
+  if (this->props[property].defer == 1) {
+    /* value is out of bound */
+    if((value < this->props[property].min) || (value > this->props[property].max))
+      value = (this->props[property].min + this->props[property].max) >> 1;
+    this->props[property].value = value;
+    this->cm_active = 0;
+    return value;
+  }
+
   if (this->props[property].atom != None) {
 
     /* value is out of bound */
@@ -1026,6 +1117,8 @@ static void xv_dispose (vo_driver_t *this_gen) {
 
   _x_alphablend_free(&this->alphablend_extra_data);
 
+  cm_close (this);
+
   free (this);
 }
 
@@ -1159,6 +1252,16 @@ static void xv_update_xv_pitch_alignment(void *this_gen, xine_cfg_entry_t *entry
   this->use_pitch_alignment = entry->num_value;
 }
 
+static void xv_fullrange_cb_config (void *this_gen, xine_cfg_entry_t *entry) {
+  xv_driver_t *this = (xv_driver_t *)this_gen;
+  this->fullrange_mode = entry->num_value;
+  if (this->fullrange_mode)
+    this->capabilities |= VO_CAP_FULLRANGE;
+  else
+    this->capabilities &= ~VO_CAP_FULLRANGE;
+  this->cm_active = 0;
+}
+
 static int xv_open_port (xv_driver_t *this, XvPortID port) {
   int ret;
   x11_InstallXErrorHandler (this);
@@ -1224,6 +1327,8 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
   XvAdaptorInfo        *adaptor_info;
   unsigned int          adaptor_num;
   xv_prefertype		prefer_type;
+  unsigned int          nencode = 0;
+  XvEncodingInfo       *encodings = NULL;
 
   this = (xv_driver_t *) calloc(1, sizeof(xv_driver_t));
   if (!this)
@@ -1265,7 +1370,7 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
 				  VIDEO_DEVICE_XV_PORT_HELP,
 				  20, NULL, NULL);
   prefer_type = config->register_enum (config, "video.device.xv_preferred_method", 0,
-				       prefer_labels, VIDEO_DEVICE_XV_PREFER_TYPE_HELP,
+				       (char **)prefer_labels, VIDEO_DEVICE_XV_PREFER_TYPE_HELP,
 				       10, NULL, NULL);
 
   if (xv_port != 0) {
@@ -1325,6 +1430,8 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
   this->x11_old_error_handler   = NULL;
   this->xine                    = class->xine;
 
+  this->cm_atom                 = None;
+
   LOCK_DISPLAY(this);
   XAllocNamedColor (this->display,
 		    DefaultColormap(this->display, this->screen),
@@ -1354,6 +1461,7 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
     this->props[i].min   = 0;
     this->props[i].max   = 0;
     this->props[i].atom  = None;
+    this->props[i].defer = 0;
     this->props[i].entry = NULL;
     this->props[i].this  = this;
   }
@@ -1363,6 +1471,8 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
   this->props[VO_PROP_ZOOM_X].value          = 100;
   this->props[VO_PROP_ZOOM_Y].value          = 100;
 
+  cm_init (this);
+
   /*
    * check this adaptor's capabilities
    */
@@ -1408,6 +1518,15 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
 	  xv_check_capability (this, VO_PROP_GAMMA, attr[k],
 			       adaptor_info[adaptor_num].base_id,
 			       NULL, NULL, NULL);
+	} else if(!strcmp(name, "XV_ITURBT_709")) {
+	  LOCK_DISPLAY(this);
+	  this->cm_atom = XInternAtom (this->display, name, False);
+	  if (this->cm_atom != None) {
+	    XvGetPortAttribute (this->display, this->xv_port, this->cm_atom, &this->cm_active);
+	    this->cm_active = this->cm_active ? 2 : 10;
+	    this->capabilities |= VO_CAP_COLOR_MATRIX;
+	  }
+	  UNLOCK_DISPLAY(this);
 	} else if(!strcmp(name, "XV_COLORKEY")) {
 	  this->capabilities |= VO_CAP_COLORKEY;
 	  xv_check_capability (this, VO_PROP_COLORKEY, attr[k],
@@ -1450,7 +1569,7 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
 	} else if(!strcmp(name, "XV_BICUBIC")) {
 	  int xv_bicubic =
 	    config->register_enum (config, "video.device.xv_bicubic", 2,
-				   bicubic_types, VIDEO_DEVICE_XV_BICUBIC_HELP,
+				   (char **)bicubic_types, VIDEO_DEVICE_XV_BICUBIC_HELP,
 				   20, xv_update_XV_BICUBIC, this);
 	  config->update_num(config,"video.device.xv_bicubic",xv_bicubic);
 	}
@@ -1462,6 +1581,33 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
     xprintf(this->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": no port attributes defined.\n");
   XvFreeAdaptorInfo(adaptor_info);
 
+  this->props[VO_PROP_BRIGHTNESS].defer = 1;
+  this->props[VO_PROP_CONTRAST].defer   = 1;
+  this->props[VO_PROP_SATURATION].defer = 1;
+  if ((this->props[VO_PROP_BRIGHTNESS].atom != None) &&
+    (this->props[VO_PROP_CONTRAST].atom != None)) {
+    static const char * const xv_fullrange_conf_labels[] = {"Off", "Normal", NULL};
+    this->fullrange_mode = this->xine->config->register_enum (
+      this->xine->config,
+      "video.output.xv_fullrange",
+      0,
+      (char **)xv_fullrange_conf_labels,
+      _("Fullrange color emulation"),
+      _("Emulate fullrange color by modifying brightness/contrast/saturation settings.\n\n"
+        "Off:    Let decoders convert such video.\n"
+        "        Works with all graphics cards, but needs a bit more CPU power.\n\n"
+        "Normal: Fast and better quality. Should work at least with some newer cards\n"
+        "        (GeForce 7+, 210+).\n\n"),
+      10,
+      xv_fullrange_cb_config,
+      this
+    );
+    if (this->fullrange_mode)
+      this->capabilities |= VO_CAP_FULLRANGE;
+  } else {
+    this->fullrange_mode = 0;
+  }
+
   /*
    * check supported image formats
    */
@@ -1501,6 +1647,26 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
     UNLOCK_DISPLAY(this);
   }
 
+  /*
+   * check max. supported image size
+   */
+
+  XvQueryEncodings(this->display, xv_port, &nencode, &encodings);
+  if (encodings) {
+    int n;
+    for (n = 0; n < nencode; n++) {
+      if (!strcmp(encodings[n].name, "XV_IMAGE")) {
+        xprintf(this->xine, XINE_VERBOSITY_LOG,
+                LOG_MODULE ": max XvImage size %li x %li\n",
+                encodings[n].width, encodings[n].height);
+        this->props[VO_PROP_MAX_VIDEO_WIDTH].value  = encodings[n].width;
+        this->props[VO_PROP_MAX_VIDEO_HEIGHT].value = encodings[n].height;
+        break;
+      }
+    }
+    XvFreeEncodingInfo(encodings);
+  }
+
   /*
    * try to create a shared image
    * to find out if MIT shm really works, using supported format
