@@ -619,7 +619,7 @@ void _x_blend_rgb16 (uint8_t * img, vo_overlay_t * img_overl,
 
     for (x = x1_scaled = 0; x < src_width;) {
       int rle_bite;
-      uint32_t *colors;
+      clut_t *colors;
       uint8_t *trans;
       uint16_t o;
       int clipped = (y < clip_top);
@@ -639,7 +639,7 @@ void _x_blend_rgb16 (uint8_t * img, vo_overlay_t * img_overl,
 
         rle_bite = rlelen;
         /* choose palette for surrounding area */
-        colors = img_overl->color;
+        colors = (clut_t*)img_overl->color;
         trans = img_overl->trans;
       } else {
         /* treat cases where highlight border is inside rle->len pixels */
@@ -651,14 +651,14 @@ void _x_blend_rgb16 (uint8_t * img, vo_overlay_t * img_overl,
             /* choose the largest "bite" up to palette change */
             rle_bite = hili_left - x;
             /* choose palette for surrounding area */
-            colors = img_overl->color;
+            colors = (clut_t*)img_overl->color;
             trans = img_overl->trans;
           } else {
             /* ends left */
 
             rle_bite = rlelen;
             /* choose palette for surrounding area */
-            colors = img_overl->color;
+            colors = (clut_t*)img_overl->color;
             trans = img_overl->trans;
           }
           if( x < clip_left )
@@ -671,14 +671,14 @@ void _x_blend_rgb16 (uint8_t * img, vo_overlay_t * img_overl,
             /* choose the largest "bite" up to palette change */
             rle_bite = hili_right - x;
             /* we're in the center area so choose highlight palette */
-            colors = img_overl->hili_color;
+            colors = (clut_t*)img_overl->hili_color;
             trans = img_overl->hili_trans;
           } else {
             /* starts right */
 
             rle_bite = rlelen;
             /* choose palette for surrounding area */
-            colors = img_overl->color;
+            colors = (clut_t*)img_overl->color;
             trans = img_overl->trans;
 
             if( x + rle_bite >= clip_right )
@@ -689,7 +689,7 @@ void _x_blend_rgb16 (uint8_t * img, vo_overlay_t * img_overl,
 
           rle_bite = rlelen;
           /* we're in the center area so choose highlight palette */
-          colors = img_overl->hili_color;
+          colors = (clut_t*)img_overl->hili_color;
           trans = img_overl->hili_trans;
         }
       }
@@ -786,7 +786,7 @@ void _x_blend_rgb24 (uint8_t * img, vo_overlay_t * img_overl,
 
     for (x = x1_scaled = 0; x < src_width;) {
       int rle_bite;
-      uint32_t *colors;
+      clut_t *colors;
       uint8_t *trans;
       uint16_t o;
       int clipped = (y < clip_top);
@@ -806,7 +806,7 @@ void _x_blend_rgb24 (uint8_t * img, vo_overlay_t * img_overl,
 
         rle_bite = rlelen;
         /* choose palette for surrounding area */
-        colors = img_overl->color;
+        colors = (clut_t*)img_overl->color;
         trans = img_overl->trans;
       } else {
         /* treat cases where highlight border is inside rle->len pixels */
@@ -818,14 +818,14 @@ void _x_blend_rgb24 (uint8_t * img, vo_overlay_t * img_overl,
             /* choose the largest "bite" up to palette change */
             rle_bite = hili_left - x;
             /* choose palette for surrounding area */
-            colors = img_overl->color;
+            colors = (clut_t*)img_overl->color;
             trans = img_overl->trans;
           } else {
             /* ends left */
 
             rle_bite = rlelen;
             /* choose palette for surrounding area */
-            colors = img_overl->color;
+            colors = (clut_t*)img_overl->color;
             trans = img_overl->trans;
           }
           if( x < clip_left )
@@ -838,14 +838,14 @@ void _x_blend_rgb24 (uint8_t * img, vo_overlay_t * img_overl,
             /* choose the largest "bite" up to palette change */
             rle_bite = hili_right - x;
             /* we're in the center area so choose highlight palette */
-            colors = img_overl->hili_color;
+            colors = (clut_t*)img_overl->hili_color;
             trans = img_overl->hili_trans;
           } else {
             /* starts right */
 
             rle_bite = rlelen;
             /* choose palette for surrounding area */
-            colors = img_overl->color;
+            colors = (clut_t*)img_overl->color;
             trans = img_overl->trans;
 
             if( x + rle_bite >= clip_right )
@@ -856,7 +856,7 @@ void _x_blend_rgb24 (uint8_t * img, vo_overlay_t * img_overl,
 
           rle_bite = rlelen;
           /* we're in the center area so choose highlight palette */
-          colors = img_overl->hili_color;
+          colors = (clut_t*)img_overl->hili_color;
           trans = img_overl->hili_trans;
         }
       }
@@ -865,13 +865,8 @@ void _x_blend_rgb24 (uint8_t * img, vo_overlay_t * img_overl,
 
       o = trans[clr];
       if (o && !clipped) {
-        union {
-          uint32_t u32;
-          clut_t   c;
-        } color = {colors[clr]};
-
         mem_blend24(img_pix + x1_scaled*3,
-                    color.c.cb, color.c.cr, color.c.y,
+                    colors[clr].cb, colors[clr].cr, colors[clr].y,
                     o, x2_scaled-x1_scaled);
       }
 
@@ -958,7 +953,7 @@ void _x_blend_rgb32 (uint8_t * img, vo_overlay_t * img_overl,
 
     for (x = x1_scaled = 0; x < src_width;) {
       int rle_bite;
-      uint32_t *colors;
+      clut_t *colors;
       uint8_t *trans;
       uint16_t o;
       int clipped = (y < clip_top);
@@ -978,7 +973,7 @@ void _x_blend_rgb32 (uint8_t * img, vo_overlay_t * img_overl,
 
         rle_bite = rlelen;
         /* choose palette for surrounding area */
-        colors = img_overl->color;
+        colors = (clut_t*)img_overl->color;
         trans = img_overl->trans;
       } else {
         /* treat cases where highlight border is inside rle->len pixels */
@@ -990,14 +985,14 @@ void _x_blend_rgb32 (uint8_t * img, vo_overlay_t * img_overl,
             /* choose the largest "bite" up to palette change */
             rle_bite = hili_left - x;
             /* choose palette for surrounding area */
-            colors = img_overl->color;
+            colors = (clut_t*)img_overl->color;
             trans = img_overl->trans;
           } else {
             /* ends left */
 
             rle_bite = rlelen;
             /* choose palette for surrounding area */
-            colors = img_overl->color;
+            colors = (clut_t*)img_overl->color;
             trans = img_overl->trans;
           }
           if( x < clip_left )
@@ -1010,14 +1005,14 @@ void _x_blend_rgb32 (uint8_t * img, vo_overlay_t * img_overl,
             /* choose the largest "bite" up to palette change */
             rle_bite = hili_right - x;
             /* we're in the center area so choose highlight palette */
-            colors = img_overl->hili_color;
+            colors = (clut_t*)img_overl->hili_color;
             trans = img_overl->hili_trans;
           } else {
             /* starts right */
 
             rle_bite = rlelen;
             /* choose palette for surrounding area */
-            colors = img_overl->color;
+            colors = (clut_t*)img_overl->color;
             trans = img_overl->trans;
 
             if( x + rle_bite >= clip_right )
@@ -1028,7 +1023,7 @@ void _x_blend_rgb32 (uint8_t * img, vo_overlay_t * img_overl,
 
           rle_bite = rlelen;
           /* we're in the center area so choose highlight palette */
-          colors = img_overl->hili_color;
+          colors = (clut_t*)img_overl->hili_color;
           trans = img_overl->hili_trans;
         }
       }
@@ -1126,7 +1121,7 @@ static uint8_t *(*blend_yuv_grow_extra_data(alphablend_t *extra_data, int osd_wi
 
   if (extra_data->buffer_size < needed_buffer_size) {
 
-    _x_freep(&extra_data->buffer);
+    free(extra_data->buffer);
     header = calloc(1, needed_buffer_size);
     if (!header) {
       extra_data->buffer_size = 0;
@@ -1157,7 +1152,7 @@ void _x_blend_yuv (uint8_t *dst_base[3], vo_overlay_t * img_overl,
                 alphablend_t *extra_data)
 {
   int enable_exact_blending = !extra_data->disable_exact_blending;
-  uint32_t *my_clut;
+  clut_t *my_clut;
   uint8_t *my_trans;
 
   int src_width = img_overl->width;
@@ -1188,7 +1183,7 @@ void _x_blend_yuv (uint8_t *dst_base[3], vo_overlay_t * img_overl,
 #ifdef LOG_BLEND_YUV
   printf("overlay_blend started x=%d, y=%d, w=%d h=%d\n",img_overl->x,img_overl->y,img_overl->width,img_overl->height);
 #endif
-  my_clut = img_overl->hili_color;
+  my_clut = (clut_t*) img_overl->hili_color;
   my_trans = img_overl->hili_trans;
 
   /* checks to avoid drawing overlay outside the destination buffer */
@@ -1299,7 +1294,7 @@ void _x_blend_yuv (uint8_t *dst_base[3], vo_overlay_t * img_overl,
             rle_this_bite = (hili_left - x);
             rle_remainder -= rle_this_bite;
             rlelen -= rle_this_bite;
-            my_clut = img_overl->color;
+            my_clut = (clut_t*) img_overl->color;
             my_trans = img_overl->trans;
             xmask = 0;
           } else {
@@ -1310,7 +1305,7 @@ void _x_blend_yuv (uint8_t *dst_base[3], vo_overlay_t * img_overl,
             rle_this_bite = rle_remainder;
             rle_remainder = 0;
             rlelen -= rle_this_bite;
-            my_clut = img_overl->color;
+            my_clut = (clut_t*) img_overl->color;
             my_trans = img_overl->trans;
             xmask = 0;
           }
@@ -1326,7 +1321,7 @@ void _x_blend_yuv (uint8_t *dst_base[3], vo_overlay_t * img_overl,
             rle_this_bite = (hili_right - x);
             rle_remainder -= rle_this_bite;
             rlelen -= rle_this_bite;
-            my_clut = img_overl->hili_color;
+            my_clut = (clut_t*) img_overl->hili_color;
             my_trans = img_overl->hili_trans;
             xmask++;
           } else {
@@ -1337,7 +1332,7 @@ void _x_blend_yuv (uint8_t *dst_base[3], vo_overlay_t * img_overl,
             rle_this_bite = rle_remainder;
             rle_remainder = 0;
             rlelen -= rle_this_bite;
-            my_clut = img_overl->hili_color;
+            my_clut = (clut_t*) img_overl->hili_color;
             my_trans = img_overl->hili_trans;
             xmask++;
           }
@@ -1352,7 +1347,7 @@ void _x_blend_yuv (uint8_t *dst_base[3], vo_overlay_t * img_overl,
             rle_this_bite = (src_width - x );
             rle_remainder -= rle_this_bite;
             rlelen -= rle_this_bite;
-            my_clut = img_overl->color;
+            my_clut = (clut_t*) img_overl->color;
             my_trans = img_overl->trans;
             xmask = 0;
           } else {
@@ -1363,7 +1358,7 @@ void _x_blend_yuv (uint8_t *dst_base[3], vo_overlay_t * img_overl,
             rle_this_bite = rle_remainder;
             rle_remainder = 0;
             rlelen -= rle_this_bite;
-            my_clut = img_overl->color;
+            my_clut = (clut_t*) img_overl->color;
             my_trans = img_overl->trans;
             xmask = 0;
           }
@@ -1376,7 +1371,7 @@ void _x_blend_yuv (uint8_t *dst_base[3], vo_overlay_t * img_overl,
         rle_this_bite = rle_remainder;
         rle_remainder = 0;
         rlelen -= rle_this_bite;
-        my_clut = img_overl->color;
+        my_clut = (clut_t*) img_overl->color;
         my_trans = img_overl->trans;
         xmask = 0;
       }
@@ -1402,34 +1397,29 @@ void _x_blend_yuv (uint8_t *dst_base[3], vo_overlay_t * img_overl,
         }
 
         if (o && !clipped) {
-          union {
-            uint32_t u32;
-            clut_t   c;
-          } color = {my_clut[clr]};
-
           if(o >= 15) {
-            memset(dst_y + x, color.c.y, rle_this_bite);
+            memset(dst_y + x, my_clut[clr].y, rle_this_bite);
             if (!enable_exact_blending) {
               if ((y + y_odd) & 1) {
-                memset(dst_cr + ((x + x_odd) >> 1), color.c.cr, (rle_this_bite+1) >> 1);
-                memset(dst_cb + ((x + x_odd) >> 1), color.c.cb, (rle_this_bite+1) >> 1);
+                memset(dst_cr + ((x + x_odd) >> 1), my_clut[clr].cr, (rle_this_bite+1) >> 1);
+                memset(dst_cb + ((x + x_odd) >> 1), my_clut[clr].cb, (rle_this_bite+1) >> 1);
               }
             }
           } else {
-            mem_blend8(dst_y + x, color.c.y, o, rle_this_bite);
+            mem_blend8(dst_y + x, my_clut[clr].y, o, rle_this_bite);
             if (!enable_exact_blending) {
               if ((y + y_odd) & 1) {
                 /* Blending cr and cb should use a different function, with pre -128 to each sample */
-                mem_blend8(dst_cr + ((x + x_odd) >> 1), color.c.cr, o, (rle_this_bite+1) >> 1);
-                mem_blend8(dst_cb + ((x + x_odd) >> 1), color.c.cb, o, (rle_this_bite+1) >> 1);
+                mem_blend8(dst_cr + ((x + x_odd) >> 1), my_clut[clr].cr, o, (rle_this_bite+1) >> 1);
+                mem_blend8(dst_cb + ((x + x_odd) >> 1), my_clut[clr].cb, o, (rle_this_bite+1) >> 1);
               }
             }
           }
 
           if (enable_exact_blending) {
             /* remember chroma of current line */
-            memset(&(*blend_yuv_data)[ 1 ][ (y + y_odd) & 1 ][ x + x_odd ], color.c.cr, rle_this_bite);
-            memset(&(*blend_yuv_data)[ 2 ][ (y + y_odd) & 1 ][ x + x_odd ], color.c.cb, rle_this_bite);
+            memset(&(*blend_yuv_data)[ 1 ][ (y + y_odd) & 1 ][ x + x_odd ], my_clut[ clr ].cr, rle_this_bite);
+            memset(&(*blend_yuv_data)[ 2 ][ (y + y_odd) & 1 ][ x + x_odd ], my_clut[ clr ].cb, rle_this_bite);
           }
         }
       }
@@ -1576,7 +1566,7 @@ static uint8_t *(*blend_yuy2_grow_extra_data(alphablend_t *extra_data, int osd_w
 
   if (extra_data->buffer_size < needed_buffer_size) {
 
-    _x_freep(&extra_data->buffer);
+    free(extra_data->buffer);
     header = calloc(1, needed_buffer_size);
     if (!header) {
       extra_data->buffer_size = 0;
@@ -1604,7 +1594,7 @@ void _x_blend_yuy2 (uint8_t * dst_img, vo_overlay_t * img_overl,
                  alphablend_t *extra_data)
 {
   int enable_exact_blending = !extra_data->disable_exact_blending;
-  uint32_t *my_clut;
+  clut_t *my_clut;
   uint8_t *my_trans;
 
   int src_width = img_overl->width;
@@ -1639,7 +1629,7 @@ void _x_blend_yuy2 (uint8_t * dst_img, vo_overlay_t * img_overl,
   uint8_t *dst_y = dst_img + dst_pitch * y_off + 2 * x_off;
   uint8_t *dst;
 
-  my_clut = img_overl->hili_color;
+  my_clut = (clut_t*) img_overl->hili_color;
   my_trans = img_overl->hili_trans;
 
   /* checks to avoid drawing overlay outside the destination buffer */
@@ -1734,7 +1724,7 @@ void _x_blend_yuy2 (uint8_t * dst_img, vo_overlay_t * img_overl,
             rle_this_bite = (hili_left - x);
             rle_remainder -= rle_this_bite;
             rlelen -= rle_this_bite;
-            my_clut = img_overl->color;
+            my_clut = (clut_t*) img_overl->color;
             my_trans = img_overl->trans;
           } else {
 #ifdef LOG_BLEND_YUV
@@ -1744,7 +1734,7 @@ void _x_blend_yuy2 (uint8_t * dst_img, vo_overlay_t * img_overl,
             rle_this_bite = rle_remainder;
             rle_remainder = 0;
             rlelen -= rle_this_bite;
-            my_clut = img_overl->color;
+            my_clut = (clut_t*) img_overl->color;
             my_trans = img_overl->trans;
           }
           if( x < clip_left )
@@ -1759,7 +1749,7 @@ void _x_blend_yuy2 (uint8_t * dst_img, vo_overlay_t * img_overl,
             rle_this_bite = (hili_right - x);
             rle_remainder -= rle_this_bite;
             rlelen -= rle_this_bite;
-            my_clut = img_overl->hili_color;
+            my_clut = (clut_t*) img_overl->hili_color;
             my_trans = img_overl->hili_trans;
           } else {
 #ifdef LOG_BLEND_YUV
@@ -1769,7 +1759,7 @@ void _x_blend_yuy2 (uint8_t * dst_img, vo_overlay_t * img_overl,
             rle_this_bite = rle_remainder;
             rle_remainder = 0;
             rlelen -= rle_this_bite;
-            my_clut = img_overl->hili_color;
+            my_clut = (clut_t*) img_overl->hili_color;
             my_trans = img_overl->hili_trans;
           }
         } else if (x >= hili_right) {
@@ -1783,7 +1773,7 @@ void _x_blend_yuy2 (uint8_t * dst_img, vo_overlay_t * img_overl,
             rle_this_bite = (src_width - x );
             rle_remainder -= rle_this_bite;
             rlelen -= rle_this_bite;
-            my_clut = img_overl->color;
+            my_clut = (clut_t*) img_overl->color;
             my_trans = img_overl->trans;
           } else {
           /* no cutting needed, starts outside, ends outside */
@@ -1793,7 +1783,7 @@ void _x_blend_yuy2 (uint8_t * dst_img, vo_overlay_t * img_overl,
             rle_this_bite = rle_remainder;
             rle_remainder = 0;
             rlelen -= rle_this_bite;
-            my_clut = img_overl->color;
+            my_clut = (clut_t*) img_overl->color;
             my_trans = img_overl->trans;
           }
           if( x + rle_this_bite >= clip_right )
@@ -1805,7 +1795,7 @@ void _x_blend_yuy2 (uint8_t * dst_img, vo_overlay_t * img_overl,
         rle_this_bite = rle_remainder;
         rle_remainder = 0;
         rlelen -= rle_this_bite;
-        my_clut = img_overl->color;
+        my_clut = (clut_t*) img_overl->color;
         my_trans = img_overl->trans;
       }
       o   = my_trans[clr];
@@ -1827,23 +1817,18 @@ void _x_blend_yuy2 (uint8_t * dst_img, vo_overlay_t * img_overl,
         }
 
         if (o && !clipped) {
-          union {
-            uint32_t u32;
-            clut_t   c;
-          } color = {my_clut[clr]};
-
           if (!enable_exact_blending) {
             l = rle_this_bite>>1;
             if( !((x_odd+x) & 1) ) {
-              yuy2.b[0] = color.c.y;
-              yuy2.b[1] = color.c.cb;
-              yuy2.b[2] = color.c.y;
-              yuy2.b[3] = color.c.cr;
+              yuy2.b[0] = my_clut[clr].y;
+              yuy2.b[1] = my_clut[clr].cb;
+              yuy2.b[2] = my_clut[clr].y;
+              yuy2.b[3] = my_clut[clr].cr;
             } else {
-              yuy2.b[0] = color.c.y;
-              yuy2.b[1] = color.c.cr;
-              yuy2.b[2] = color.c.y;
-              yuy2.b[3] = color.c.cb;
+              yuy2.b[0] = my_clut[clr].y;
+              yuy2.b[1] = my_clut[clr].cr;
+              yuy2.b[2] = my_clut[clr].y;
+              yuy2.b[3] = my_clut[clr].cb;
             }
           }
 	  if (o >= 15) {
@@ -1861,7 +1846,7 @@ void _x_blend_yuy2 (uint8_t * dst_img, vo_overlay_t * img_overl,
             } else {
               l = rle_this_bite;
               while (l--) {
-                *dst = color.c.y;
+                *dst = my_clut[clr].y;
                 dst += 2;
               }
             }
@@ -1881,7 +1866,7 @@ void _x_blend_yuy2 (uint8_t * dst_img, vo_overlay_t * img_overl,
             } else {
               l = rle_this_bite;
               while (l--) {
-                *dst = BLEND_BYTE(*dst, color.c.y, o);
+                *dst = BLEND_BYTE(*dst, my_clut[clr].y, o);
                 dst += 2;
               }
             }
@@ -1889,8 +1874,8 @@ void _x_blend_yuy2 (uint8_t * dst_img, vo_overlay_t * img_overl,
 
           if (enable_exact_blending) {
             /* remember chroma of current line */
-            memset(&(*blend_yuy2_data)[ 1 ][ x + x_odd ], color.c.cr, rle_this_bite);
-            memset(&(*blend_yuy2_data)[ 2 ][ x + x_odd ], color.c.cb, rle_this_bite);
+            memset(&(*blend_yuy2_data)[ 1 ][ x + x_odd ], my_clut[ clr ].cr, rle_this_bite);
+            memset(&(*blend_yuy2_data)[ 2 ][ x + x_odd ], my_clut[ clr ].cb, rle_this_bite);
           }
         } else {
           dst += rle_this_bite*2;
@@ -2178,96 +2163,11 @@ void _x_alphablend_init(alphablend_t *extra_data, xine_t *xine)
 
 void _x_alphablend_free(alphablend_t *extra_data)
 {
-  _x_freep(&extra_data->buffer);
+  if (extra_data->buffer) {
+    free(extra_data->buffer);
+    extra_data->buffer = NULL;
+  }
 
   extra_data->buffer_size = 0;
 }
 
-#define saturate(v) if (v & ~255) v = (~((uint32_t)v)) >> 24
-
-void _x_clut_yuv2rgb(uint32_t *clut, int num_items, int color_matrix)
-{
-  uint32_t *end = clut + num_items;
-  if (end <= clut) return;
-
-  switch (color_matrix >> 1) {
-
-    case 8:
-      while (clut < end) {
-        union {
-          uint32_t u32;
-          clut_t   c;
-        } tmp = { *clut };
-        int32_t y, u, v, r, g, b;
-        y = tmp.c.y;
-        u = tmp.c.cb;
-        v = tmp.c.cr;
-        /* Green Orange -- does this ever happen? */
-        r = y - u + v;
-        saturate (r);
-        g = y + u     - 128;
-        saturate (g);
-        b = y - u - v + 256;
-        saturate (b);
-        /* see clut_to_argb () */
-        tmp.c.cb = b;
-        tmp.c.cr = g;
-        tmp.c.y  = r;
-        *clut++  = tmp.u32;
-      }
-      break;
-
-    case 1:
-    case 7:
-      while (clut < end) {
-        union {
-          uint32_t u32;
-          clut_t   c;
-        } tmp = { *clut };
-        int32_t y, u, v, r, g, b;
-        y = tmp.c.y;
-        u = tmp.c.cb;
-        v = tmp.c.cr;
-        /* ITU 709 (HD), mpeg range. */
-        y *= 76304;
-        r = (y              + 117473 * v - 16224640) >> 16;
-        saturate (r);
-        g = (y  - 13972 * u  - 34918 * v  + 5069824) >> 16;
-        saturate (g);
-        b = (y + 138425 * u              - 18906496) >> 16;
-        saturate (b);
-        /* see clut_to_argb () */
-        tmp.c.cb = b;
-        tmp.c.cr = g;
-        tmp.c.y  = r;
-        *clut++  = tmp.u32;
-      }
-      break;
-
-    default:
-      while (clut < end) {
-        union {
-          uint32_t u32;
-          clut_t   c;
-        } tmp = { *clut };
-        int32_t y, u, v, r, g, b;
-        y = tmp.c.y;
-        u = tmp.c.cb;
-        v = tmp.c.cr;
-        /* ITU 601 (SD), mpeg range. */
-        y *= 76304;
-        r = (y              + 104582 * v - 14574592) >> 16;
-        saturate (r);
-        g = (y  - 25664 * u  - 53268 * v  + 8849664) >> 16;
-        saturate (g);
-        b = (y + 132186 * u              - 18107904) >> 16;
-        saturate (b);
-        /* see clut_to_argb () */
-        tmp.c.cb = b;
-        tmp.c.cr = g;
-        tmp.c.y  = r;
-        *clut++  = tmp.u32;
-      }
-      break;
-  }
-}
